Package: consort
Type: Package
Title: Create Consort Diagram
Version: 1.2.3
Authors@R: person("Alim", "Dayim",
                  email = "ad938@cam.ac.uk",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0001-9998-7463"))
Description: To make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved.
License: MIT + file LICENSE
URL: https://github.com/adayim/consort/
BugReports: https://github.com/adayim/consort/issues
Encoding: UTF-8
Imports: grid
Suggests: testthat (>= 3.0.0), knitr, DiagrammeR, rmarkdown, covr,
        stringi
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Depends: R (>= 3.5.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-25 21:37:19 UTC; alim
Author: Alim Dayim [aut, cre] (ORCID: <https://orcid.org/0000-0001-9998-7463>)
Maintainer: Alim Dayim <ad938@cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-26 02:10:02 UTC
Built: R 4.5.2; ; 2026-04-26 23:54:46 UTC; unix

Package: consort
Type: Package
Title: Create Consort Diagram
Version: 1.2.3
Authors@R: person("Alim", "Dayim",
                  email = "ad938@cam.ac.uk",
                  role = c("aut", "cre"),
                  comment = c(ORCID = "0000-0001-9998-7463"))
Description: To make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved.
License: MIT + file LICENSE
URL: https://github.com/adayim/consort/
BugReports: https://github.com/adayim/consort/issues
Encoding: UTF-8
Imports: grid
Suggests: testthat (>= 3.0.0), knitr, DiagrammeR, rmarkdown, covr,
        stringi
VignetteBuilder: knitr
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Depends: R (>= 3.5.0)
LazyData: true
NeedsCompilation: no
Packaged: 2026-04-25 21:37:19 UTC; alim
Author: Alim Dayim [aut, cre] (ORCID: <https://orcid.org/0000-0001-9998-7463>)
Maintainer: Alim Dayim <ad938@cam.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-26 02:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(heightDetails,textbox)
S3method(makeContent,textbox)
S3method(makeContext,textbox)
S3method(plot,consort)
S3method(print,consort)
S3method(widthDetails,textbox)
S3method(xDetails,textbox)
S3method(yDetails,textbox)
export(add_box)
export(add_label_box)
export(add_side_box)
export(add_split)
export(build_grid)
export(build_grviz)
export(connect_box)
export(consort_plot)
export(gen_text)
export(get_coords)
export(grid.textbox)
export(move_box)
export(textbox)
import(grid)
importFrom(stats,na.omit)
importFrom(stats,setNames)
importFrom(utils,modifyList)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# consort

<!-- badges: start -->

[![R-CMD-check](https://github.com/adayim/consort/workflows/R-CMD-check/badge.svg)](https://github.com/adayim/consort/actions)
[![CRAN
status](https://www.r-pkg.org/badges/version/consort)](https://CRAN.R-project.org/package=consort)
[![CRAN
download](https://cranlogs.r-pkg.org/badges/grand-total/consort)](https://cran.r-project.org/package=consort)
[![codecov](https://codecov.io/gh/adayim/consort/branch/main/graph/badge.svg?token=mzvaYDMPNc)](https://app.codecov.io/gh/adayim/consort)
<!-- badges: end -->

The goal of `consort` is to make it easy to create CONSORT diagrams for
the transparent reporting of participant allocation in randomized,
controlled clinical trials. This is done by creating a standardized
disposition data, and using this data as the source for the creation a
standard CONSORT diagram. Human effort by supplying text labels on the
node can also be achieved.

## Installation

You can install the released version of consort from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("consort")
```

And the development version from [GitHub](https://github.com/) with:

``` r
# install.packages("devtools")
devtools::install_github("adayim/consort")
```

## Example

This is a basic example which shows you how to solve a create CONSORT
diagram with a given subject disposition data:

``` r
library(consort)
library(grid)
## basic example code
```

``` r
set.seed(1001)
N <- 300

trialno <- sample(c(1000:2000), N)
exc <- rep(NA, N)
exc[sample(1:N, 15)] <- sample(c("Sample not collected", "MRI not collected", "Other"),
                                15, replace = T, prob = c(0.4, 0.4, 0.2))

arm <- rep(NA, N)
arm[is.na(exc)] <- sample(c("Conc", "Seq"), sum(is.na(exc)), replace = T)

fow1 <- rep(NA, N)
fow1[!is.na(arm)] <- sample(c("Withdraw", "Discontinued", "Death", "Other", NA),
                            sum(!is.na(arm)), replace = T, 
                            prob = c(0.05, 0.05, 0.05, 0.05, 0.8))
fow2 <- rep(NA, N)
fow2[!is.na(arm) & is.na(fow1)] <- sample(c("Protocol deviation", "Outcome missing", NA),
                                          sum(!is.na(arm) & is.na(fow1)), replace = T, 
                                          prob = c(0.05, 0.05, 0.9))
df <- data.frame(trialno, exc, arm, fow1, fow2)
head(df)
#>   trialno  exc  arm  fow1 fow2
#> 1    1086 <NA> Conc  <NA> <NA>
#> 2    1418 <NA>  Seq  <NA> <NA>
#> 3    1502 <NA> Conc Death <NA>
#> 4    1846 <NA> Conc  <NA> <NA>
#> 5    1303 <NA> Conc Death <NA>
#> 6    1838 <NA>  Seq  <NA> <NA>
```

``` r

set_consort_defaults(txt_gp = gpar(cex = 0.6), parse_markup = TRUE)

out <- consort_plot(data = df,
             order = c(trialno = "Population",
                          exc    = "**Excluded**",
                          arm     = "Randomized patient",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable",
                          trialno = "Final Analysis"),
             side_box = c("exc", "fow1", "fow2"),
             allocation = "arm",
             labels = c("1" = "Screening", "2" = "Randomization",
                        "5" = "Final"))

plot(out)
```

<img src="man/figures/README-diagram-1.png" alt="" width="100%" />

As the `grid` plotting is not very ideal, calculation of the coodinates
for the nodes are not easy job and tried my best. Feel free to PR if you
want to improve. Or you can produce `Graphviz` plot by setting
`grViz = TRUE` in `plot`. This will use `DiagrammeR` to print the plot.
The plot is ideal for Shiny or HTML output.

``` r
plot(out, grViz = TRUE)
```

Or save this `Graphviz` plot to `png` or `pdf`

``` r
plot(g, grViz = TRUE) |> 
    DiagrammeRsvg::export_svg() |> 
    charToRaw() |> 
    rsvg::rsvg_pdf("svg_graph.pdf")
```

txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15): \n
              \u2022 MRI not collected (n=3)\n
              \u2022 Tissues not collected (n=4)\n
              \u2022 Other (n=8)"

g <- add_box(txt = txt1)

g <- add_side_box(g, txt = txt1_side)

g <- add_box(g, txt = "Randomized (n=200)")

g <- add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100"))
g <- add_side_box(g,
  txt = c(
    "Excluded (n=15):\n
                  \u2022 MRI not collected (n=3)\n
                  \u2022 Tissues not collected (n=4)\n
                   \u2022 Other (n=8)",
    "Excluded (n=15):\n
                   \u2022 MRI not collected (n=3)\n
                   \u2022 Tissues not collected (n=4)"
  )
)

g <- add_box(g, txt = c("Final analysis (n=100)", "Final analysis (n=100"))
g <- add_label_box(g, txt = c("1" = "Screening", "3" = "Randomized", "4" = "Final analysis"))

## Prepare test data
set.seed(1001)
N <- 300

trialno <- sample(c(1000:2000), N)
exc1 <- rep(NA, N)
exc1[sample(1:N, 15)] <- sample(c("Sample not collected", "MRI not collected", "Other"),
  15,
  replace = TRUE, prob = c(0.4, 0.4, 0.2)
)

induc <- rep(NA, N)
induc[is.na(exc1)] <- trialno[is.na(exc1)]

exc2 <- rep(NA, N)
exc2[sample(1:N, 20)] <- sample(c(
  "Sample not collected", "Dead",
  "Other"
), 20,
replace = TRUE,
prob = c(0.4, 0.4, 0.2)
)
exc2[is.na(induc)] <- NA

exc <- ifelse(is.na(exc2), exc1, exc2)

arm <- rep(NA, N)
arm[is.na(exc)] <- sample(c("Conc", "Seq"), sum(is.na(exc)), replace = TRUE)
arm3 <- sample(c("Trt A", "Trt B", "Trt C"), N, replace = TRUE)
arm3[is.na(arm)] <- NA

fow1 <- rep(NA, N)
fow1[!is.na(arm)] <- sample(c("Withdraw", "Discontinued", "Death", "Other", NA),
  sum(!is.na(arm)),
  replace = TRUE,
  prob = c(0.05, 0.05, 0.05, 0.05, 0.8)
)
fow2 <- rep(NA, N)
fow2[!is.na(arm) & is.na(fow1)] <- sample(c("Protocol deviation", "Outcome missing", NA),
  sum(!is.na(arm) & is.na(fow1)),
  replace = TRUE,
  prob = c(0.05, 0.05, 0.9)
)


df <- data.frame(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2)
rm(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2, N)

## Multiple phase
p <- consort_plot(
  data = df,
  order = list(
    trialno = "Population",
    exc1 = "Excluded",
    induc = "Induction",
    exc2 = "Excluded",
    arm3 = "Randomized patient",
    fow1 = "Lost of Follow-up",
    trialno = "Finished Followup",
    fow2 = "Not evaluable",
    trialno = "Final Analysis"
  ),
  side_box = c("exc1", "exc2", "fow1", "fow2"),
  allocation = "arm3",
  labels = c(
    "1" = "Screening", "2" = "Month 4",
    "3" = "Randomization", "5" = "Month 24",
    "6" = "End of study"
  ),
  cex = 0.7
)

## Prepare test data
data(dispos.data)

df <- dispos.data[!dispos.data$arm3 %in% "Trt C", ]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                c(arm3     = "", 
                                  subjid_notdosed="Participants not treated"),
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)

## ---- include = FALSE---------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(consort)

## -----------------------------------------------------------------------------
set.seed(1001)
N <- 300

trialno <- sample(c(1000:2000), N)
exc1 <- rep(NA, N)
exc1[sample(1:N, 15)] <- sample(c("Sample not collected", "MRI not collected",
                                  "Other"), 15, replace = T, prob = c(0.4, 0.4, 0.2))

induc <- rep(NA, N)
induc[is.na(exc1)] <- trialno[is.na(exc1)]

exc2 <- rep(NA, N)
exc2[sample(1:N, 20)] <- sample(c("Sample not collected", "Dead",
                                  "Other"), 20, replace = T, prob = c(0.4, 0.4, 0.2))
exc2[is.na(induc)] <- NA

exc <- ifelse(is.na(exc2), exc1, exc2)

arm <- rep(NA, N)
arm[is.na(exc)] <- sample(c("Conc", "Seq"), sum(is.na(exc)), replace = T)
arm3 <- sample(c("Trt A", "Trt B", "Trt C"), N, replace = T)
arm3[is.na(arm)] <- NA

fow1 <- rep(NA, N)
fow1[!is.na(arm)] <- sample(c("Withdraw", "Discontinued", "Death", "Other", NA),
                            sum(!is.na(arm)), replace = T, 
                            prob = c(0.05, 0.05, 0.05, 0.05, 0.8))
fow2 <- rep(NA, N)
fow2[!is.na(arm) & is.na(fow1)] <- sample(c("Protocol deviation", "Outcome missing", NA),
                                          sum(!is.na(arm) & is.na(fow1)), replace = T, 
                                          prob = c(0.05, 0.05, 0.9))

df <- data.frame(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2)

df$reas1[!is.na(arm)] <- sample(c("Protocol deviation", "Outcome missing", NA),
                                sum(!is.na(arm)), replace = T, 
                                prob = c(0.08, 0.07, 0.85))

df$reas2[!is.na(df$reas1)] <- sample(c("Withdraw", "Discontinued", "Death", "Other"),
                                     sum(!is.na(df$reas1)), replace = T, 
                                     prob = c(0.05, 0.05, 0.05, 0.05))

rm(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2, N)

## ----cars, echo=FALSE---------------------------------------------------------
head(df)

## ----eval=FALSE---------------------------------------------------------------
#  consort_plot(data,
#               orders,
#               side_box,
#               allocation = NULL,
#               labels = NULL,
#               cex = 0.8,
#               text_width = NULL,
#               widths = c(0.1, 0.9))

## ----message=FALSE, fig.width  = 6, fig.height = 6----------------------------
out <- consort_plot(data = df,
             orders = c(trialno = "Population",
                          exc1    = "Excluded",
                          trialno = "Allocated",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable for the final analysis",
                          trialno = "Final Analysis"),
             side_box = c("exc1", "fow1", "fow2"),
             cex = 0.9)
plot(out)


## ----fig.width  = 7, fig.height = 6-------------------------------------------
out <- consort_plot(data = df,
             orders = c(trialno = "Population",
                          exc    = "Excluded",
                          arm     = "Randomized patient",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable",
                          trialno = "Final Analysis"),
             side_box = c("exc", "fow1", "fow2"),
             allocation = "arm",
             labels = c("1" = "Screening", "2" = "Randomization",
                        "5" = "Final"))

plot(out)

## ----fig.width  = 9, fig.height = 6-------------------------------------------
g <- consort_plot(data = df,
             orders = list(trialno = "Population",
                          exc1    = "Excluded",
                          induc   = "Induction",
                          exc2    = "Excluded",
                          arm3     = "Randomized patient",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable",
                          trialno = "Final Analysis"),
             side_box = c("exc1", "exc2", "fow1", "fow2"),
             allocation = "arm3",
             labels = c("1" = "Screening", "2" = "Month 4",
                        "3" = "Randomization", "5" = "Month 24",
                        "6" = "End of study"),
             cex = 0.7)
plot(g)

## ----fig.width  = 7, fig.height = 4-------------------------------------------
library(grid)
# Might want to change some settings
options(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)

## -----------------------------------------------------------------------------
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
g <- add_split(g, txt = c("Process 1 (n=140)", "Process 2 (n=140)", "Process 3 (n=142)"))

plot(g, grViz = TRUE)


## ----fig.width  = 7, fig.height = 6-------------------------------------------

df$arm <- factor(df$arm)

txt <- gen_text(df$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(df$exc, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

# Exclude subjects
df <- df[is.na(df$exc), ]

g <- add_box(g, txt = gen_text(df$arm, label = "Patients randomised")) 

txt <- gen_text(df$arm)
g <- add_split(g, txt = txt)

txt <- gen_text(split(df[,c("reas1", "reas2")], df$arm),
                label = "Lost to follow-up")
g <- add_box(g, txt = txt, just = "left")

df <- df[complete.cases(df[,c("reas1", "reas2")]), ]
txt <- gen_text(split(df$trialno, df$arm),
                label = "Primary analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "3" = "First Stage"))

plot(g)

## -----------------------------------------------------------------------------
plot(g, grViz = TRUE)

## ----eval=FALSE---------------------------------------------------------------
#  # save plots
#  png("consort_diagram.png", width = 29,
#      height = 21, res = 300, units = "cm", type = "cairo")
#  plot(g)
#  dev.off()
#  

## ----eval=FALSE---------------------------------------------------------------
#  ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))

## ----eval=FALSE---------------------------------------------------------------
#  plot(g, grViz = TRUE) |>
#      DiagrammeRsvg::export_svg() |>
#      charToRaw() |>
#      rsvg::rsvg_pdf("svg_graph.pdf")

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----setup--------------------------------------------------------------------
library(consort)
data(dispos.data)

## ----data-head, echo=FALSE----------------------------------------------------
head(dispos.data)

## ----example, eval=FALSE------------------------------------------------------
#  consort_plot(data,
#               orders,
#               side_box,
#               allocation = NULL,
#               labels = NULL,
#               cex = 0.8,
#               text_width = NULL,
#               widths = c(0.1, 0.9))

## ----patchwork, eval=FALSE----------------------------------------------------
#  library(patchwork)
#  
#  wrap_elements(build_grid(g)) + plot_annotation(
#    title = 'Consort diagram',
#    subtitle = 'Flow chart of the XX study',
#    caption = 'Disclaimer: None of these plots are insightful'
#  )

## ----dump-dotfile, eval = FALSE-----------------------------------------------
#  cat(build_grviz(g), file = "consort.gv")

## ----single-arms, message=FALSE, fig.width  = 6, fig.height = 6---------------
out <- consort_plot(data = dispos.data,
                    orders = c(trialno = "Population",
                               exclusion = "Excluded",
                               trialno = "Allocated",
                               subjid_notdosed = "Not dosed",
                               followup = "Followup",
                               lost_followup = "Not evaluable for the final analysis",
                               mitt = "Final Analysis"),
                    side_box = c("exclusion", "subjid_notdosed", "lost_followup"),
                    cex = 0.9)
plot(out)

## ----multiple-phase, fig.width  = 9, fig.height = 6---------------------------
g <- consort_plot(data = dispos.data,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm3     = "Randomized patient",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = "arm3",
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "5" = "End of study"),
                  cex = 0.7)
plot(g)

## ----multiple-split, fig.width  = 12, fig.height = 8, out.width = "95%"-------
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
g <- consort_plot(data = df,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm = "Randomized patient",
                             arm3     = "",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "6" = "End of study"),
                  cex = 0.7)
plot(g)

## ----multiple-split-stack, fig.width  = 13, fig.height = 10, out.width = "95%"----
# We will exclude one arm to avoid too many arms.
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                # The following two variables will be stacked together
                                c(arm3     = "", # Should not provide a value to show the actual arm
                                  subjid_notdosed="Participants not treated"),
                                # The following two variables will be stacked together
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"), 
                  allocation = c("arm", "arm3"), # Two level randomisation
                  kickoff_sidebox = FALSE,
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)

plot(p)

## ----providetext1, fig.width  = 7, fig.height = 4-----------------------------
library(grid)
# Might want to change some settings
options(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)

## ----providetext2-------------------------------------------------------------
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
# Length needs to be the same as previous one, use a list here.
g <- add_split(g, txt = list(c("Process 1 (n=140)", "Process 2 (n=140)",
                               "Process 3 (n=142)")))

plot(g, grViz = TRUE)


## ----manual-multisplit, fig.width  = 13, fig.height = 6, out.width = "80%"----
g <- add_box(txt = "Patient consented (n=200)")
g <- add_side_box(g, txt = "Excluded (n=10):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=7)")
g <- add_box(g, txt = "Randomised (n=190)")
g <- add_split(g, txt = c("Randomised study (n=144)", "Preference study (n=146)"))

# Provide a list here
g <- add_split(g, txt = list(c("Allocated to surgery (n=70)", 
                                "Allocated to medicine (n=74)"),
                             c("Allocated to surgery (n=47)", 
                              "Allocated to medicine (n=48)",
                               "Placebo (n=51)")))

g <- add_side_box(g, txt = c("Excluded (n=3):\n\u2022 Withdrawn before surgery (n=2)\n\u2022 Declined (n=1)", 
                             "", 
                             "Excluded (n=1):\n\u2022 Withdrawn before surgery (n=1)",
                             "Excluded (n=3):\n\u2022 Withdrawn before surgery (n=1)\n\u2022 Declined (n=1)",
                             "Excluded (n=10):6\n\u2022 Declined (n=10)"),
                  side = rep("right", 5))
g <- add_box(g, txt = c("Analysed (n=67)", "Analysed (n=74)",
                        "Analysed (n=46)", "Analysed (n=45)", "Analysed (n=41)"))

plot(g)


## ----disposition-data, fig.width  = 7, fig.height = 6.5-----------------------
options(txt_gp = gpar(cex = 0.8))

dispos.data$arm <- factor(dispos.data$arm)

txt <- gen_text(dispos.data$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(dispos.data$exclusion, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

g <- add_box(g, txt = gen_text(dispos.data$arm, label = "Patients randomised")) 

txt <- gen_text(dispos.data$arm)
g <- add_split(g, txt = txt)

# Exclude subjects
dispos.data <- dispos.data[is.na(dispos.data$exclusion), ]

txt <- gen_text(split(dispos.data[,c("subjid_notdosed")], dispos.data$arm),
                label = "Not dosed", bullet = TRUE)
g <- add_box(g, txt = txt, just = "left")

txt <- gen_text(split(dispos.data$mitt, dispos.data$arm),
                label = "Primary mITT analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "5" = "Final analysis"))

plot(g)

## ----save-plot----------------------------------------------------------------
plot(g, grViz = TRUE)

## ----eval=FALSE---------------------------------------------------------------
#  # save plots
#  png("consort_diagram.png", width = 29,
#      height = 21, res = 300, units = "cm", type = "cairo")
#  plot(g)
#  dev.off()
#  

## ----eval=FALSE---------------------------------------------------------------
#  ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))

## ----eval=FALSE---------------------------------------------------------------
#  plot(g, grViz = TRUE) |>
#      DiagrammeRsvg::export_svg() |>
#      charToRaw() |>
#      rsvg::rsvg_pdf("svg_graph.pdf")

#' Generate label and bullet points
#'
#' This function use the data to generate label and bullet points for the box.
#'
#' @param x A list or a vector to be used. \code{x} can be atomic vector, a 
#' \code{data.frame} or a \code{list}. A \code{data.frame} is particular useful
#'  if the there's a nested reason or a \code{list} split nested reasons by group.
#' The nested reasons only support two columns and the \code{bullet} will be ignored.
#' @param label A character string as a label at the beginning of the text label.
#' The count for each categories will be returned if no label is provided.
#' @param bullet If shows bullet points. If the value is \code{TRUE}, the bullet points
#' will be tabulated, default is \code{FALSE}.
#' @param bullet_char A single character used as the bullet symbol. Defaults to
#'   \code{consort_opt("bullet")}. Can be any Unicode
#'   character such as \code{"\u2013"} (en-dash), \code{"\u25CB"} (circle),
#'   \code{"\u25A0"} (square), \code{"-"}, etc.
#'
#' @return A character string of vector.
#' @export
#'
#' @rdname gen_text
#'
#' @examples
#' val <- data.frame(
#'   am = factor(ifelse(mtcars$am == 1, "Automatic", "Manual"), ordered = TRUE),
#'   vs = factor(ifelse(mtcars$vs == 1, "Straight", "V-shaped"), ordered = TRUE),
#'   car = row.names(mtcars)
#' )
#'
#' gen_text(val$car, label = "Cars in the data")
#' gen_text(val$car, label = "Cars in the data", bullet = FALSE)
#' gen_text(split(val$car, val$am), label = "Cars in the data")
#' gen_text(split(val$car, val$am), label = "Cars in the data", bullet = FALSE)
#' gen_text(split(val[,c("vs", "car")], val$am), label = "Cars in the data", bullet = FALSE)
#' gen_text(val[,c("vs", "car")], label = "Cars in the data", bullet = FALSE)
#'
#' # Use a custom bullet character
#' gen_text(val$car, label = "Cars in the data", bullet = TRUE, bullet_char = "-")
#'
#' # Or set globally via set_consort_defaults
#' set_consort_defaults(bullet = "\u25CB")
#' gen_text(val$car, label = "Cars in the data", bullet = TRUE)
gen_text <- function(x, label = NULL, bullet = FALSE,
                     bullet_char = consort_opt("bullet")) {
  
  if(!is.null(label) & length(label)>1)
    stop("label must be of length 1")
  
  if(is.data.frame(x) && ncol(x) == 1)
    x <- unlist(x)
  
  if (is.list(x)) {
    if(is.data.frame(x)){
      box_data.frame(x, label = label, bullet_char = bullet_char)
    }else {
      # Calculate numers in each split
      sp_num <- sapply(x, function(i){
        if(is.data.frame(i))
          nrow(i)
        else
          length(i)
      })
      
      if(!is.null(label))
        label <- rep(label, length(x))
      
      sapply(seq_along(x), function(indx) {
        val <- x[[indx]]
        # If the list contains a data.frame
        if(is.data.frame(val)){
          r <- box_data.frame(val, bullet_char = bullet_char)
          if(!is.null(label)){
            lab_lst <- sprintf("%s (n=%s)", 
                               label[indx], 
                               pret_num(sum(!is.na(val[[1]]))))
            r <- paste(lab_lst, r, sep = "\n")
          }
          
        }else{
          r <- box_label(val, label = label[indx], bullet = bullet,
                         bullet_char = bullet_char)
          r <- paste(r, collapse = "\n")
        }
        
        return(r)
        
      }, simplify = TRUE)
    }
    
  } else {
    box_label(x = x, label = label, bullet = bullet, bullet_char = bullet_char)
  }
}

# Calculate the numbers in the box use the data provided.
#' @keywords internal
box_data.frame <- function(x, label = NULL,
                           bullet_char = consort_opt("bullet")){
  if(ncol(x) != 2)
    stop("only two columns are supported")
  
  if(!is.null(label))
    label <- sprintf("%s (n=%s)", label, pret_num(sum(!is.na(x[[1]]))))
  
  r <- sapply(na.omit(unique(x[[1]])), function(i){
    box_label(x[[2]][x[[1]] == i], label = i, bullet = TRUE,
              bullet_char = bullet_char)
  })
  
  r <- paste(r, collapse = "\n")
  
  if(!is.null(label))
    r <- paste(label, r, sep = "\n")
  
  return(r)
}


# Calculate the numbers in the box
#' @keywords internal
box_label <- function(x, label, bullet = TRUE,
                      bullet_char = consort_opt("bullet")) {
  
  # Blank as NA
  if (is.character(x)) {
    x[x == ""] <- NA
  }
  
  # Return blank if no values and with bullet
  if (sum(!is.na(x)) == 0 & bullet) {
    return("")
  }
  
  if (is.null(label)) {
    if(is.factor(x))
      tab <- table(droplevels(x))
    else
      tab <- table(x)
    
    tp <- paste0(names(tab), " (n=", pret_num(tab), ")")
    if(!bullet){
      return(tp)
    }else{
      return(paste0(bullet_char, " ", paste(tp, collapse = paste0("\n", bullet_char, " "))))
    }
    
  }
  
  tp <- paste0(label, " (n=", pret_num(sum(!is.na(x))), ")")
  
  if (bullet) {
    if(is.factor(x))
      tab <- table(droplevels(x))
    else
      tab <- table(x)
    txt_sub <- paste0(bullet_char, " ", names(tab), " (n=", pret_num(tab), ")")
    tp <- paste0(tp, "\n", paste(txt_sub, collapse = "\n"))
  }
  
  return(tp)
}


# Format numbers
#' @keywords internal
pret_num <- function(x){
  prettyNum(x, big.mark = ",", preserve.width = "none", scientific = FALSE)
}

test_that("Generate text for the consort", {
  val <- data.frame(
    am = factor(ifelse(mtcars$am == 1, "Automatic", "Manual"), ordered = TRUE),
    vs = factor(ifelse(mtcars$vs == 1, "Straight", "V-shaped"), ordered = TRUE),
    car = row.names(mtcars)
  )
  tab1 <- table(val$car[val$am == "Manual"])
  tab2 <- table(val$am)

  tx1 <- gen_text(val$car, label = "Cars in the data")
  tx2 <- gen_text(split(val$car, val$am),
    label = "Cars in the data",
    bullet = TRUE
  )
  
  tx7 <- gen_text(split(val$car, val$am),
                  bullet = TRUE
  )
  
  expect_equal(tx7, sub(".*?\\n", "", tx2))
  
  tx3 <- gen_text(val$am)
  tx4 <- gen_text(rep("", 4), bullet = TRUE)

  # Numbers equal
  expect_equal(
    as.numeric(gsub("[^\\d]+", "", tx1, perl = TRUE)),
    nrow(val)
  )

  expect_length(tx2, 2)
  expect_equal(
    strsplit(tx2, "\n\u2022 ")[[2]][-1],
    paste0(names(tab1), " (n=", tab1, ")")
  )

  expect_equal(tx3, paste0(names(tab2), " (n=", tab2, ")"))

  expect_equal(tx4, "")
  
  # data.frame
  tx5 <- gen_text(split(val[,c("vs", "car")], val$am),
                  label = "Cars in the data", bullet = FALSE)
  tx5_ulst <- strsplit(tx5, "\n\u2022 ")[[2]]
  tab5 <- table(val$vs, val$am)
  expect_equal(
    tx5_ulst[1],
    paste0("Cars in the data (n=", sum(tab5[,2]), ")\nStraight (n=", 
           sum(tab5["Straight", 2]), ")")
  )
  
  tx6 <- gen_text(val[,c("vs", "car")], 
                  label = "Cars in the data", bullet = FALSE)
  tab6 <- table(val$vs, val$car)
  tx6_ulst <- strsplit(tx6, "\n\u2022 ")[[1]]
  
  expect_equal(
    tx6_ulst[1],
    paste0("Cars in the data (n=", nrow(mtcars), ")\nV-shaped (n=", 
           sum(tab6["V-shaped",]), ")")
  )
  
  
})

#' Add a side node
#'
#' Add an exclusion node on the right side. If the length of text label is two, then
#' the first one will be aligned on the left and the second on the right. Otherwise,
#' all the side nodes will be aligned on the right.
#'
#' @param prev_box Previous node object, the created new node will be aligned
#' at the right bottom of the `prev_box`.
#' @param side Position of the side box, `left` or `right` side of the terminal box.
#' Will be aligned on the left and right side if only two groups, right otherwise.
#' This will be ignored for for `grViz` plot, see \link[DiagrammeR]{grViz}.
#' @inheritParams add_box
#'
#' @seealso \code{\link{add_box}} \code{\link{add_split}} \code{\link{textbox}} 
#' \code{\link{add_label_box}} 
#'
#' @return A \code{consort} object.
#'
#' @export
#'
#' @example inst/examples/add-box-example.R
add_side_box <- function(prev_box,
                         txt,
                         side = NULL,
                         text_width = NULL,
                         ...) {

  dots <- list(...)
  if("name" %in% names(dots)){
    warning("`parameter name was provided but will be ignored.")
    dots$name <- NULL
  }

  # Wrap text
  if (!is.null(text_width)) {
    txt <- sapply(txt, function(tx) {
      text_wrap(unlist(tx), width = text_width)
    })
  }

  if (!inherits(prev_box, c("consort"))) {
    stop("prev_box must be consort object")
  }

  prev_nodes <- attr(prev_box, "nodes.current")
  num_nodes <- attr(prev_box, "nodes.num")

  if (all(attr(prev_box, "nodes.type") %in% c("sidebox", "label"))) {
    stop("The last box added is a side box or label, can not add side box after a sidebox!")
  }

  if (length(prev_nodes) != length(txt)) 
    stop("The txt length must be same as previous node number.")

  if (!is.null(side) & length(side) != length(txt)) {
    stop("The length of side must have the same length with txt.")
  }

  # One box on left, the other is right if only two groups given,
  # all will be on right side if not.
  if (is.null(side)) {
    if (length(txt) == 2) {
      side <- c("left", "right")
    } else {
      side <- rep("right", length(txt))
    }
  }

  # Update arguments
  args_list <- list(just = "left", box_fn = rectGrob, name = "sidebox")
  args_list <- modifyList(args_list, dots)

  nodes <- lapply(seq_along(txt), function(i){
    box <- do.call(textbox, c(list(text = txt[i]), args_list))

    # Add width to the side box, calculate horizontal width
    prev_box <- prev_box[[prev_nodes[i]]]$box_hw
    box_hw <- get_coords(box)
    # Add extra width to account for side box 
    box_hw$width <- box_hw$width + convertWidth(prev_box$half_width, "mm", valueOnly = TRUE)*1.5
    # Don't know why this is too large in the plot
    box_hw$height <- box_hw$height/2 

    list(
      text = txt[i],
      node_type = "sidebox",
      box = box,
      box_hw = box_hw,
      side = side[i],
      just = "left",
      prev_node = prev_nodes[i]
    )
  })

    names(nodes) <- paste0("node", num_nodes + seq_along(txt))

    node_list <- c(prev_box, nodes)

    class(node_list) <- union("consort", class(node_list))

    structure(node_list,
      nodes.num = length(txt) + num_nodes,
      nodes.current = prev_nodes,
      nodes.type = "sidebox",
      nodes.list = c(attr(prev_box, "nodes.list"), list(names(nodes)))
    )

}

save_png <- function(x, width = 9, height = 9) {
  path <- tempfile(fileext = ".png")
  png(path, width = width, height = height, 
      units = "in", type = "cairo-png", res = 300)
  on.exit(dev.off())
  plot(x)
  path
}

test_that("Generate consort manually", {
  txt1 <- "Population (n=300)"
  txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

  g <- add_box(txt = txt1)

  g <- add_side_box(g, txt = txt1_side)

  g <- add_box(g, txt = "Randomized (n=200)")

  g <- add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100)"))
  g <- add_side_box(g, txt = c(
    "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
    ""
  ))

  # Multiple split
  expect_error(print(add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100)"))))
  # Length 1
  expect_error(add_split(g, txt = "Arm A (n=100)"))

  # Continue plotting
  g <- add_box(g, txt = c("Final analysis (n=85)", "Final analysis (n=100)"))

  g <- add_label_box(g, txt = c(
    "1" = "Screening",
    "3" = "Randomized",
    "4" = "Final analysis"
  ))

  expect_s3_class(g, "consort")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "manually-gen.png")

})

#' Create a box with text
#'
#' Create a \link[grid]{grob} with text inside. To extract the units describing
#' grob boundary location can be accessed with \link[grid]{grobX} and
#'  \link[grid]{grobY}. The units describing width and height can be accessed with
#'  \link[grid]{grobWidth} and \link[grid]{grobHeight}.
#'
#' @param text A character text to be passed to \link[grid]{textGrob}.
#' @param x A number or unit object specifying x-location.
#' @param y A number or unit object specifying y-location.
#' @param just The justification of the text, `"left"`, `"right` and `"center"`.
#' See \link[grid]{textGrob} for more details.
#' @param txt_gp An object of class \link[grid]{gpar} style to be applied to the
#' text. Defaults are read from \code{\link{set_consort_defaults}}. For
#' example, to set a font size for all text inside boxes, use
#' \code{set_consort_defaults(txt_gp = gpar(cex = 0.8))}.
#' @param leftrotate If the text box will be rotated 90 degrees counter-clockwise.
#' Default is \code{FALSE}.
#' @param box_fn Function to create box for the text. Parameters of `x=0.5`,
#' `y=0.5` and `box_gp` will be passed to this function and return a \code{grob}
#' object.
#' @param box_gp An object of class \link[grid]{gpar} style to be applied to the
#' box.
#' @param name A character identifier.
#'
#' @return A text box grob. grid.textbox() returns the value invisibly.
#'
#' @export
#' @import grid
#' @rdname textbox
#'
#' @examples
#' fg <- textbox(text = "This is a test")
#' grid::grid.draw(fg)
#' grid.textbox(text = "This is a test")
textbox <- function(text,
                    x = unit(.5, "npc"),
                    y = unit(.5, "npc"),
                    just = c("center", "left", "right"),
                    txt_gp = consort_opt("txt_gp"),
                    leftrotate = FALSE,
                    box_fn = roundrectGrob,
                    box_gp = consort_opt("box_gp"),
                    name = "textbox") {
  just <- match.arg(just)

  if (!is.logical(leftrotate) || length(leftrotate) != 1 || is.na(leftrotate)) {
    stop("`leftrotate` must be a single TRUE/FALSE value.")
  }

  if (!is.unit(x)) x <- unit(x, units = "npc")
  if (!is.unit(y)) y <- unit(y, units = "npc")

  angle <- if (isTRUE(leftrotate)) 90 else 0

  # class(fg) <- union("box", class(fg))
  name <- paste(name, auto_index(), sep = ".")

  gTree(
    label = text, x = x, y = y, just = just,
    txt_gp = txt_gp, box_fn = box_fn,
    box_gp = box_gp, angle = angle,
    name = name,
    cl = "textbox"
  )
}


#' @param ... Parameters passed to \code{textbox}
#' @rdname textbox
#' @examples
#' grid.textbox(text = "This is a test")
#' @export
grid.textbox <- function(...) {
  grid::grid.draw(textbox(...))
}

get_hw <- function(x) {
  if (has_markup(x$label)) {
    return(get_hw_formatted(x))
  }

  t <- textGrob(label = x$label, gp = x$txt_gp)
  # Add padding
  padding <- unit(1 * ifelse(is.null(x$txt_gp$cex), 1,
    x$txt_gp$cex
  ), "char")
  # height <- grobHeight(t) + padding
  height <- convertHeight(grobHeight(t), "char") + padding
  # width <- grobWidth(t) + padding
  width <- convertWidth(grobWidth(t), "char") + padding

  list(width = width, height = height)
}

# Measure width/height for formatted (markup) text
get_hw_formatted <- function(x) {
  segments <- parse_markup(x$label)
  lines    <- split_segments_by_newline(segments)

  cex     <- ifelse(is.null(x$txt_gp$cex), 1, x$txt_gp$cex)
  padding <- unit(1 * cex, "char")

  # Width of each line in inches
  line_widths <- sapply(lines, function(line_segs) {
    sum(sapply(line_segs, function(seg) {
      gp <- segment_gpar(seg$style, x$txt_gp)
      t  <- textGrob(label = seg$text, gp = gp)
      convertWidth(grobWidth(t), "inches", valueOnly = TRUE)
    }))
  })

  width <- convertWidth(unit(max(line_widths), "inches"), "char") + padding

  # Height from a dummy multi-line textGrob (correct line spacing)
  n_lines    <- length(lines)
  dummy_text <- paste(rep("Xg", n_lines), collapse = "\n")
  dummy      <- textGrob(label = dummy_text, gp = x$txt_gp)
  height     <- convertHeight(grobHeight(dummy), "char") + padding

  # Extra room for super/subscript offsets
  has_ss <- any(sapply(segments, function(s)
    s$style %in% c("superscript", "subscript")))
  if (has_ss) {
    height <- height + unit(0.3 * cex, "char")
  }

  list(width = width, height = height)
}

#' @export
makeContext.textbox <- function(x) {
  hw <- get_hw(x)

  tbvp <- viewport(x$x, x$y,
    width = hw$width,
    height = hw$height,
    angle = x$angle,
    name = paste0(x$name, ".vp")
  )

  if (is.null(x$vp)) {
    x$vp <- tbvp
  } else {
    x$vp <- vpStack(x$vp, tbvp)
  }
  x
}

#' @export
makeContent.textbox <- function(x) {

  # Add padding
  padding <- unit(3 * ifelse(is.null(x$txt_gp$cex), 1, x$txt_gp$cex), "mm")

  if (has_markup(x$label)) {
    text_grobs <- make_formatted_content(x)
  } else {
    # Align text
    tx_x <- switch(x$just,
      "right" = unit(1, "npc") - 0.5 * padding,
      "left"  = 0.5 * padding,
      "center" = unit(.5, "npc")
    )

    t <- textGrob(
      label = x$label,
      x = tx_x,
      y = 0.5,
      just = x$just,
      gp = x$txt_gp,
      name = paste0(x$name, ".text")
    )
    text_grobs <- gList(t)
  }

  hw <- get_hw(x)

  rr <- do.call(x$box_fn, list(
    x = .5,
    y = .5,
    width = hw$width,
    height = hw$height,
    gp = x$box_gp,
    name = paste0(x$name, ".box")
  ))

  setChildren(x, do.call(gList, c(list(rr), as.list(text_grobs))))
}

# Render formatted (markup) text as positioned grid grobs
make_formatted_content <- function(x) {
  segments <- parse_markup(x$label)
  lines    <- split_segments_by_newline(segments)
  n_lines  <- length(lines)

  cex     <- ifelse(is.null(x$txt_gp$cex), 1, x$txt_gp$cex)
  padding <- unit(3 * cex, "mm")

  grob_list <- list()

  for (i in seq_along(lines)) {
    line_segs <- lines[[i]]

    # Vertical position: evenly distribute lines top-to-bottom
    y_frac <- 1 - (2 * i - 1) / (2 * n_lines)
    y_base <- unit(y_frac, "npc")

    # Measure each segment width (in inches, resolved now)
    seg_info <- lapply(seq_along(line_segs), function(j) {
      seg <- line_segs[[j]]
      gp  <- segment_gpar(seg$style, x$txt_gp)
      tg  <- textGrob(seg$text, gp = gp, just = "left")
      w   <- convertWidth(grobWidth(tg), "inches", valueOnly = TRUE)
      list(grob = tg, style = seg$style, width_in = w)
    })

    total_width_in <- sum(sapply(seg_info, "[[", "width_in"))

    # Horizontal start position based on justification
    start_x <- switch(x$just,
      "center" = unit(0.5, "npc") - unit(total_width_in / 2, "inches"),
      "left"   = padding * 0.5,
      "right"  = unit(1, "npc") - padding * 0.5 - unit(total_width_in, "inches"),
      unit(0.5, "npc") - unit(total_width_in / 2, "inches")
    )

    cum_x_in <- 0

    for (j in seq_along(seg_info)) {
      si <- seg_info[[j]]

      # Vertical offset for super/subscript
      y_off <- switch(si$style,
        "superscript" = unit(0.4 * cex, "char"),
        "subscript"   = unit(-0.3 * cex, "char"),
        unit(0, "mm")
      )

      si$grob$x    <- start_x + unit(cum_x_in, "inches")
      si$grob$y    <- y_base + y_off
      si$grob$name <- sprintf("%s.text.%d.%d", x$name, i, j)

      grob_list[[length(grob_list) + 1L]] <- si$grob

      # Draw underline beneath the segment
      if (si$style == "underline") {
        seg_w <- unit(si$width_in, "inches")
        ul_y  <- y_base - unit(0.55 * cex, "char")
        ul_x0 <- start_x + unit(cum_x_in, "inches")
        ul <- segmentsGrob(
          x0 = ul_x0, x1 = ul_x0 + seg_w,
          y0 = ul_y,  y1 = ul_y,
          gp = gpar(
            col = if (!is.null(x$txt_gp$col)) x$txt_gp$col else "black",
            lwd = max(0.5, 0.8 * cex)
          ),
          name = sprintf("%s.ul.%d.%d", x$name, i, j)
        )
        grob_list[[length(grob_list) + 1L]] <- ul
      }

      cum_x_in <- cum_x_in + si$width_in
    }
  }

  do.call(gList, grob_list)
}

#' @export
xDetails.textbox <- function(x, theta) {
  hw <- get_hw(x)
  rr <- do.call(x$box_fn, list(
    x = .5,
    y = .5,
    width = hw$width,
    height = hw$height
  ))
  grobX(rr, theta)
}

#' @export
yDetails.textbox <- function(x, theta) {
  hw <- get_hw(x)
  rr <- do.call(x$box_fn, list(
    x = .5,
    y = .5,
    width = hw$width,
    height = hw$height
  ))

  grobY(rr, theta)
}

#' @export
widthDetails.textbox <- function(x) {
  hw <- get_hw(x)
  rr <- do.call(x$box_fn, list(
    x = .5,
    y = .5,
    width = hw$width,
    height = hw$height
  ))
  grobWidth(rr)
}

#' @export
heightDetails.textbox <- function(x) {
  hw <- get_hw(x)
  rr <- do.call(x$box_fn, list(
    x = .5,
    y = .5,
    width = hw$width,
    height = hw$height
  ))
  grobHeight(rr)
}

test_that("Box width and height", {
  bx1 <- textbox(text = "This is a test")
  bx2 <- textbox(text = "This is a test", x = 0.2, y = 0.4)
  bx3 <- textbox(text = "BBB", x = 0.2, y = 0.4)
  bx4 <- textbox(text = "This is a test\n BBB")
  bx5 <- textbox(text = "Ths\nBBB")

  # Width
  expect_equal(get_coords(bx1)$width, get_coords(bx2)$width)
  # expect_equal(get_coords(bx3)$width, get_coords(bx5)$width)

  # Height
  expect_equal(get_coords(bx1)$height, get_coords(bx2)$height)
  expect_equal(get_coords(bx1)$height, get_coords(bx3)$height)
  expect_equal(get_coords(bx4)$height, get_coords(bx5)$height)
})

test_that("Box options", {
  old <- set_consort_defaults(
    txt_gp = gpar(cex = 0.5),
    box_gp = gpar(fill = "red")
  )
  on.exit(set_consort_defaults(
    txt_gp = old$txt_gp,
    box_gp = old$box_gp
  ), add = TRUE)

  bx1 <- textbox(text = "This is a test")

  # Text size
  expect_equal(bx1$txt_gp$cex, 0.5)
  # Box fill
  expect_equal(bx1$box_gp$fill, "red")
})


test_that("Expect class type", {
  tx <- textGrob(label = "text")
  bx1 <- textbox(text = "This is a test")

  # expect_error(get_coords(tx), "Object x must be textbox.")
  expect_s3_class(bx1, "textbox")
})

test_that("Box width and height", {
  bx1 <- textbox(text = "This is a test")
  bx2 <- move_box(bx1, x = 0.2)
  bx2_r <- move_box(bx1, x = -0.3, pos_type = "relative")
  bx3 <- move_box(bx1, y = 0.4)
  bx3_r <- move_box(bx1, y = -0.1, pos_type = "relative")
  bx4 <- move_box(bx1, x = 0.2, y = 0.4)
  bx5 <- textbox(text = "This is a test", x = 0.2, y = 0.4)


  expect_equal(get_coords(bx1)$y, get_coords(bx2)$y)
  expect_equal(get_coords(bx2)$x, get_coords(bx2_r)$x)

  expect_equal(get_coords(bx1)$x, get_coords(bx3)$x)
  expect_equal(get_coords(bx3)$y, get_coords(bx3_r)$y)

  for (i in c(
    "left", "right", "bottom", "top",
    "x", "y", "width", "height", "half_width", "half_height"
  )) {
    expect_equal(get_coords(bx4)[[i]], get_coords(bx5)[[i]])
  }
})

#' Move a box grob
#'
#' This function can be used to move the box to a given position with
#' \link[grid]{editGrob} changing the \code{x} and \code{y} value.
#'
#' @param obj A \code{box} object.
#' @param x A unit element or a number that can be converted to \code{npc}, see
#' \link[grid]{unit}.
#' @param y A unit element or a number that can be converted to \code{npc}, see
#' \link[grid]{unit}.
#' @param pos_type If the provided coordinates are \code{absolute} position the
#' box will be moved to or it's a \code{relative} position to it's current.
#'
#' @return A box object with updated x and y coordinates.
#'
#' @export
#'
#' @examples
#' fg <- textbox(text = "This is a test")
#' fg2 <- move_box(fg, 0.3, 0.3)
move_box <- function(obj, x = NULL, y = NULL, pos_type = c("absolute", "relative")) {
  pos_type <- match.arg(pos_type)

  if (!inherits(obj, "textbox")) {
    stop("Object obj must be textbox.")
  }

  args <- list(x = obj$x, y = obj$y, width = obj$width, height = obj$width)

  if (!is.null(x)) {
    if (!is.unit(x)) {
      x <- unit(x, units = "npc")
    }

    if (pos_type == "relative") {
      args$x <- args$x + x
    } else {
      args$x <- x
    }
  }

  if (!is.null(y)) {
    if (!is.unit(y)) {
      y <- unit(y, units = "npc")
    }

    if (pos_type == "relative") {
      args$y <- args$y + y
    } else {
      args$y <- y
    }
  }

  fg <- editGrob(obj, x = args$x, y = args$y)

  return(fg)
}

library(testthat)
library(consort)

test_check("consort")


# vdiffr always fails on MAC and Ubuntu, don't know why
to_grviz <- function(x) {
  path <- tempfile(fileext = ".gv")
  cat(x, file = path)
  path
}

to_grviz <- function(x) {
  path <- tempfile(fileext = ".gv")
  x <- gsub("\u2022", "", x) 
  cat(x, file = path)
  path
}

save_png <- function(x, width = 9, height = 9) {
  path <- tempfile(fileext = ".png")
  png(path, width = width, height = height, 
      units = "in", type = "cairo-png", res = 300)
  on.exit(dev.off())
  plot(x)
  path
}

test_that("Auto generate", {

  df <- readRDS('dat1.rds')
  g <- consort_plot(
    data = df,
    orders = c(
      trialno = "Population",
      exc = "Excluded",
      arm = "Allocated",
      fow1 = "Lost of Follow-up",
      trialno = "Finished Followup",
      fow2 = "Not evaluable for the final analysis",
      trialno = "Final Analysis"
    ),
    allocation = "arm",
    side_box = c("exc", "fow1", "fow2"),
    labels = c(
      "1" = "Screening", "2" = "Randomization",
      "5" = "Final"
    ),
    cex = 0.9
  )

  expect_s3_class(g, "consort")
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "auto-grviz.gv")

  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "autogen.png")

})

test_that("Allocation last node", {
  r <- readRDS('dat2.rds')
  r$qual[!is.na(r$exc)] <- NA
  g <- consort_plot(r,
                    orders = c(id      = 'Screened',
                               exc     = 'Excluded',
                               qual    = 'Qualified for Randomization',
                               consent = 'Consented',
                               tx      = 'Randomized'),
                    side_box   = 'exc',
                    allocation = 'tx',
                    labels=c('1'='Screening', '2'='Consent') #, '3'='Randomization')
  )
  expect_s3_class(g, "consort")
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "auto-last-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "autogen-last.png")
  
})


test_that("Allocation no label", {

  r <- readRDS('dat2.rds')
  # No label
  g <- consort_plot(r,
                    orders = c(id      = 'Screened',
                               exc     = 'Excluded',
                               qual    = 'Qualified for Randomization',
                               consent = 'Consented',
                               tx      = 'Randomized'),
                    side_box = 'exc',
                    allocation = 'tx'
  )
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "auto-nolab-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "autogen-nolab.png")
  
})

to_grviz <- function(x) {
  path <- tempfile(fileext = ".gv")
  x <- gsub("\u2022", "", x) 
  cat(x, file = path)
  path
}

save_png <- function(x, width = 9, height = 9) {
  path <- tempfile(fileext = ".png")
  png(path, width = width, height = height, 
      units = "in", type = "cairo-png", res = 300)
  on.exit(dev.off())
  plot(x)
  path
}

data(dispos.data)

test_that("Multiple split generate", {

  p <- consort_plot(data = dispos.data,
                    orders = c(trialno = "Population",
                               exclusion = "Excluded",
                               arm     = "Randomized patient",
                               arm3     = "",
                               subjid_notdosed = "Lost of Follow-up",
                               followup = "Followup-up",
                               lost_followup = "Lost to follow-up",
                               mitt = "Final Analysis"),
                    side_box = c("exclusion", "subjid_notdosed", "lost_followup"),
                    allocation = c("arm", "arm3"))

  txt <- build_grviz(p)
  expect_snapshot_file(to_grviz(txt), "multiple-split.gv")

  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(p), "multiple-split.png")

})


test_that("Multiple split and no kickoff", {

  df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
  p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                # The following two variables will be stacked together
                                c(arm3     = "", # Should not provide a value to show the actual arm
                                  subjid_notdosed="Participants not treated"),
                                # The following two variables will be stacked together
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"), 
                  allocation = c("arm", "arm3"), # Two level randomisation
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  kickoff_sidebox = FALSE,
                  cex = 0.7)

  txt <- build_grviz(p)
  expect_snapshot_file(to_grviz(txt), "multiple-split-nokick.gv")

  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(p), "multiple-split-nokick.png")

})

#' Build consort diagram
#'
#' Build a \code{grob} consort diagram, use this if you want
#' to save plots with \code{\link[ggplot2]{ggsave}}. \code{build_grid}
#' does not support multiple split for the moment, please use 
#'  \code{\link{build_grviz}} or \code{plot(g, grViz = TRUE)} for 
#' multiple split nodes instead.
#'
#' @param x A consort object.
#' 
#' @return A \code{gList} object
#' @export 
#'
#' @seealso \code{\link[grid]{gList}}
#' @examples
#' \dontrun{
#' txt1 <- "Population (n=300)"
#' txt1_side <- "Excluded (n=15): \n
#'               \u2022 MRI not collected (n=3)\n
#'               \u2022 Tissues not collected (n=4)\n
#'               \u2022 Other (n=8)"
#'
#' g <- add_box(txt = txt1)
#'
#' g <- add_side_box(g, txt = txt1_side)
#'
#' g <- add_box(g, txt = "Randomized (n=200)")
#' # g <- ggsave("consort_diagram.pdf", plot = build_grid(g))
#' }
#' 
build_grid <- function(x) {

  if (!inherits(x, c("consort")))
    stop("x must be consort object")
  
  if(any(grepl("label", names(x)))){
    consort_plot <- x[grepl("node", names(x))]
    attr(consort_plot, "nodes.list") <- attr(x, "nodes.list")
    label_plot <- x[grepl("label", names(x))]
  }else{
    consort_plot <- x
  }
  
  # Coordination
  nodes_coord <- calc_coords(consort_plot)
  
  # Generate connection
  nodes_connect <- get_connect(consort_plot)

  # Move all nodes to the left if there are labels nodes
  # based on the width of the label nodes
  vp_height <- nodes_coord$max_height
  vp_width <- nodes_coord$max_width
  nodes_coord$y <- (vp_height - nodes_coord$y)/vp_height

  if(any(grepl("label", names(x)))){
    label_coord <- calc_coords_label(label_plot,
                                     nodes_coord$nd_y,
                                     max_h = nodes_coord$max_height)
    vp_width <- sum(label_coord$width[1], vp_width)

    nodes_coord$x <- (nodes_coord$x + label_coord$width[1])/vp_width

    # Convert label x from char units to NPC so they scale with the viewport
    label_coord$x <- label_coord$x / vp_width

  }else{
    nodes_coord$x <- (nodes_coord$x)/vp_width
  }
  
  # Change nodes coordinates
  nodes <- sapply(names(consort_plot), function(i){
    r <- move_box(consort_plot[[i]]$box,
                  x = unit(nodes_coord$x[i], "npc"),
                  y = unit(nodes_coord$y[i], "npc"))
    r$name <- i
    
    # Skip empty side box
    if(is_empty(consort_plot[[i]]$text))
      return(NULL)
      
    return(r)
  }, simplify = FALSE)
  
  grobs_list <- gList()
  for (i in seq_along(nodes)) {
    if(!is.null(nodes[[i]]))
      grobs_list <- gList(grobs_list, nodes[[i]])
  }
  
  # Connections
  for(i in seq_along(nodes_connect)){
    nd <- nodes_connect[[i]]
    
    if(is.null(nodes[[nd$node[1]]]))
      next
    
    for(j in 2:length(nd$node)){
      if(is.null(nodes[[nd$node[j]]])){
        nd_name <- nodes_connect[[nd$node[j]]]$node[2]
      }else{
        nd_name <- nd$node[j]
      }
      connect_gb <- connect_box(nodes[[nd_name]], nodes[[nd$node[1]]], 
                                connect = nd$connect, type = "p")
      grobs_list <- gList(grobs_list, connect_gb)
    }
  }
  
  if(any(grepl("label", names(x)))){
    
    # Align labels
    for(i in seq_along(label_plot)){
      nam <- names(label_plot)[i]
      r <- move_box(label_plot[[nam]]$box,
                    x = unit(label_coord$x[nam], "npc"),
                    y = unit(label_coord$y[nam], "npc"))
      r$name <- nam
      
      lab_grobs <- if (i == 1) gList(r) else gList(lab_grobs, r)
    }
    
    grobs_list <- gList(grobs_list, lab_grobs)
    
  }
  
  grobTree(grobs_list,
           name = "consort",
           vp = viewport(width = unit(0.98, "npc"),
                         height = unit(0.98, "npc")))

}

# For indexing invisible node
pkgenv <- new.env(parent = emptyenv())
pkgenv$p <- 0

get_invs <- function() {
  pkgenv$p
}
set_invs <- function(value) {
  old <- pkgenv$p
  pkgenv$p <- value
  invisible(old)
}

# Convert R gpar to Graphviz node attributes
#' @keywords internal
gpar_to_grviz_attrs <- function(txt_gp, box_gp, box_fn = NULL) {
  attrs <- character()
  style_parts <- character()

  # Rounded corners for roundrectGrob
  if (!is.null(box_fn) && identical(box_fn, grid::roundrectGrob)) {
    style_parts <- c(style_parts, "rounded")
  }

  # Box fill color
  if (!is.null(box_gp$fill) && nzchar(box_gp$fill)) {
    attrs <- c(attrs, sprintf('fillcolor="%s"', box_gp$fill))
    style_parts <- c(style_parts, "filled")
  }

  # Box border color
  if (!is.null(box_gp$col)) {
    attrs <- c(attrs, sprintf('color="%s"', box_gp$col))
  }

  # Box border width
  if (!is.null(box_gp$lwd)) {
    attrs <- c(attrs, sprintf('penwidth=%s', box_gp$lwd))
  }

  # Line type
  if (!is.null(box_gp$lty)) {
    lty_val <- box_gp$lty
    lty_map <- c("2" = "dashed", "3" = "dotted", "4" = "dotdash",
                 "5" = "longdash", "6" = "twodash",
                 "dashed" = "dashed", "dotted" = "dotted",
                 "dotdash" = "dotdash", "longdash" = "longdash",
                 "twodash" = "twodash")
    lty_str <- as.character(lty_val)
    if (lty_str %in% names(lty_map)) {
      style_parts <- c(style_parts, lty_map[lty_str])
    }
  }

  # Style attribute
  if (length(style_parts) > 0) {
    attrs <- c(attrs, sprintf('style="%s"', paste(style_parts, collapse = ",")))
  }

  # Text color
  if (!is.null(txt_gp$col)) {
    attrs <- c(attrs, sprintf('fontcolor="%s"', txt_gp$col))
  }

  # Font size
  if (!is.null(txt_gp$fontsize)) {
    attrs <- c(attrs, sprintf('fontsize=%s', txt_gp$fontsize))
  } else if (!is.null(txt_gp$cex) && txt_gp$cex != 1) {
    attrs <- c(attrs, sprintf('fontsize=%s', round(14 * txt_gp$cex, 1)))
  }

  # Font family
  if (!is.null(txt_gp$fontfamily) && nzchar(txt_gp$fontfamily)) {
    attrs <- c(attrs, sprintf('fontname="%s"', txt_gp$fontfamily))
  }

  if (length(attrs) == 0) return(NULL)
  paste(attrs, collapse = " ")
}

# Convert arrow settings to Graphviz edge attributes
#' @keywords internal
arrow_to_grviz_attrs <- function(arrow_gp, arrow_type, arrow_length) {
  attrs <- character()

  # Edge color
  if (!is.null(arrow_gp$col)) {
    attrs <- c(attrs, sprintf('color="%s"', arrow_gp$col))
  }

  # Edge line width
  if (!is.null(arrow_gp$lwd)) {
    attrs <- c(attrs, sprintf('penwidth=%s', arrow_gp$lwd))
  }

  # Edge line type — always emit style to reset from label edges' style=invis
  edge_style <- ""
  if (!is.null(arrow_gp$lty)) {
    lty_map <- c("2" = "dashed", "3" = "dotted", "4" = "dotdash",
                 "5" = "longdash", "6" = "twodash",
                 "dashed" = "dashed", "dotted" = "dotted",
                 "dotdash" = "dotdash", "longdash" = "longdash",
                 "twodash" = "twodash")
    lty_str <- as.character(arrow_gp$lty)
    if (lty_str %in% names(lty_map)) {
      edge_style <- lty_map[lty_str]
    }
  }
  attrs <- c(attrs, sprintf('style="%s"', edge_style))

  # Arrowhead type: R "closed" -> Graphviz "normal", R "open" -> "vee"
  if (!is.null(arrow_type)) {
    ah <- if (arrow_type == "open") "vee" else "normal"
    attrs <- c(attrs, sprintf('arrowhead="%s"', ah))
  }

  # Arrow size: scale relative to default 0.1 inches
  if (!is.null(arrow_length) && is.numeric(arrow_length)) {
    attrs <- c(attrs, sprintf('arrowsize=%.2f', arrow_length / 0.1))
  }

  if (length(attrs) == 0) return("")
  paste(attrs, collapse = " ")
}

# Make subgraph same rank
#' @keywords internal
mk_subgraph_rank <- function(x){
  sprintf("subgraph {
  rank = same; rankdir = LR; %s;
  }", paste(x, collapse = "; "))
}

# Make invisible connections
#' @keywords internal
mk_invs_connect <- function(x){
  sprintf("%s [arrowhead = none];", paste(x, collapse = " -> "))
}

# Make text alignment
#' @keywords internal
mk_text_align <- function(text, just, group = NULL, grviz_style = NULL){

  if (has_markup(text)) {
    # Graphviz HTML-like label: <html> instead of "plain"
    html <- markup_to_html(text)

    if (just == "left") {
      html <- gsub("<br/>", '<br align="left"/>', html, fixed = TRUE)
      html <- paste0(html, '<br align="left"/>')
    } else if (just == "right") {
      html <- gsub("<br/>", '<br align="right"/>', html, fixed = TRUE)
      html <- paste0(html, '<br align="right"/>')
    }

    attr_parts <- sprintf("label = <%s>", html)
  } else {
    # Original plain-text label
    jst <- ifelse(just == "center", "",
                  ifelse(just == "left", "\\l", "\r"))

    if(just %in% c("left", "right")){
      text <- unlist(strsplit(text, "\n"))
      text <- ifelse(just == "left",
                     paste(text, collapse = "\\l"),
                     paste(text, collapse = "\r"))
    }

    attr_parts <- sprintf('label = "%s%s"', text, jst)
  }

  # Build attributes
  if(!is.null(group)) attr_parts <- paste(attr_parts, sprintf("group=%s", group))
  if(!is.null(grviz_style)) attr_parts <- paste(attr_parts, grviz_style)

  r <- sprintf("[%s]", attr_parts)

  if(is_empty(text)){
    r <- gsub('.{1}$', ' shape=none height=0 width=0]', r)
  }
  return(r)
}

# Make invisible nodes for split nodes
#' @keywords internal
mk_invs_split <- function(from_node, 
                          to_node,
                          consort_plot,
                          current_group,
                          next_group){
  
  # Get middle element
  mid_pos <- function(vec) ceiling(length(vec)/2)

  one2more <- function(node1, node2, gp1, gp2){
    # This is one to more split
    #       Node1
    #         |
    #   +-----+------+
    #   |     |      |
    # Node2  Node2  Node2

    # Determine which one should be in the middle
    if((length(node2) %% 2) == 0) {
      invs_nd <- paste0("P", seq_len(length(node2)+1) + get_invs())
      inv2nd <- sprintf("%s -> %s;", invs_nd[-mid_pos(invs_nd)], node2)
      # Insert the previous one in the middile
      gp2 <- append(gp2, gp1, after = length(gp2)/2)
    } else {
      invs_nd <- paste0("P", seq_len(length(node2)) + get_invs())
      inv2nd <- sprintf("%s -> %s;", invs_nd, node2)
    }
    nd2inv <- sprintf("%s -> %s [arrowhead = none];", node1, invs_nd[mid_pos(invs_nd)])
    
    nd2inv <- c(nd2inv, inv2nd, mk_invs_connect(invs_nd))
    
    set_invs(get_invs() + length(invs_nd))

    invs_nd_gp <- sprintf("%s [group=%s]", invs_nd, gp2)
    
    return(list(connect = nd2inv, invs = invs_nd, invs_gp = invs_nd_gp))
  }

  more2one <- function(node1, node2, gp1, gp2){
    # This is more to one
    # Node1  Node1  Node1
    #   |     |      |
    #   +-----+------+
    #         |
    #       Node2
    # Determine which one should be in the middle
    if((length(node1) %% 2) == 0) {
      invs_nd <- paste0("P", seq_len(length(node1)+1) + get_invs())
      inv2nd <- sprintf("%s -> %s [arrowhead = none];", node1, invs_nd[-mid_pos(invs_nd)])
      # Insert the previous one in the middile
      gp1 <- append(gp1, gp2, after = length(gp1)/2)
    } else {
      invs_nd <- paste0("P", seq_len(length(node1)) + get_invs())
      inv2nd <- sprintf("%s -> %s [arrowhead = none];", node1, invs_nd)
    }
    nd2inv <- sprintf("%s -> %s;", invs_nd[mid_pos(invs_nd)], node2)
    
    nd2inv <- c(nd2inv, inv2nd, mk_invs_connect(invs_nd))
    
    set_invs(get_invs() + length(invs_nd))

    invs_nd_gp <- sprintf("%s [group=%s]", invs_nd, gp1)
    
    return(list(connect = nd2inv, invs = invs_nd, invs_gp = invs_nd_gp))
  }
  
  if(anyDuplicated(from_node) > 0){
    # One to more
    nds <- lapply(unique(from_node), function(x){
      nd2 <- to_node[which(from_node == x)]
      gp1 <- current_group[names(current_group) %in% x]
      gp2 <- next_group[names(next_group) %in% nd2]
      one2more(x, nd2, gp1, gp2)
    })
    nd_rank <- mk_subgraph_rank(unique(to_node))
  }else{
    # More to one
    nds <- lapply(unique(to_node), function(x){
      nd1 <- from_node[which(to_node == x)]
      gp1 <- current_group[names(current_group) %in% nd1]
      gp2 <- next_group[names(next_group) %in% x]
      more2one(nd1, x, gp1, gp2)
    })
    nd_rank <- mk_subgraph_rank(unique(from_node))
  }
  
  invs <- unlist(lapply(nds, "[[", "invs"), use.names = FALSE)
  connect <- unlist(lapply(nds, "[[", "connect"), use.names = FALSE)
  
  if(length(nds) > 1){
    invs_samelev <- sapply(1:(length(nds) - 1), function(i){
      prev_invs <- nds[[i]]$invs
      next_invs <- nds[[i+1]]$invs
      sprintf("%s -> %s [style=invis];", 
              prev_invs[length(prev_invs)], 
              next_invs[1])
    })
    connect <- c(connect, invs_samelev)
  }

  sm_rnk <- c(mk_subgraph_rank(invs), nd_rank)
  
  invs_gp <- unlist(lapply(nds, "[[", "invs_gp"), use.names = FALSE)
  
  return(list("invs" = invs_gp, 
              "connect" = connect,
              "rank" = sm_rnk))
  }


# Make invisible side nodes
#' @keywords internal
mk_invs_side <- function(node1, node2, node3, group = NULL){  
  # For side nodes, nodes is the side node
  # This is for:
  #    Node
  #     |->ND
  #    Node

  invs_nd <- sprintf("P%i", get_invs()+1)

  nd2inv <- sprintf("%s -> %s [arrowhead = none];", node1, invs_nd)
  inv2nd <- sprintf("%s -> %s;", invs_nd, c(node2, node3))
  nd2inv <- c(nd2inv, inv2nd)
  sm_rnk <- mk_subgraph_rank(c(invs_nd, node2))
  
  set_invs(get_invs() + length(invs_nd))

  if(!is.null(group))
    invs_nd <- sprintf("%s [group=%s]", invs_nd, group) 
  
  return(list("invs" = invs_nd, 
              "connect" = nd2inv,
              "rank" = sm_rnk))
}

#' Self generating consort diagram
#'
#' Create CONSORT diagram from a participant disposition data.
#'
#' @param data Data set with disposition information for each participants.
#' @param orders A named vector or a list, names as the variable in the dataset
#' and values as labels in the box. The order of the diagram will be based on this.
#' A list can be used to report multiple variable in a single node, the first 
#' variable in a list element will be used to report the total and the exact items
#' will be summarised for the remaining variable. This is limitted to non-side box.
#' @param side_box Variable vector, appeared as side box in the diagram. The next
#'  box will be the subset of the missing values of these variables.
#' @param allocation Name of the grouping/treatment variable (optional), the
#'  diagram will split into branches on this variables forward. For a factorial 
#' design, with two splits for example, a character vector with a maximum of 
#' length two can be provided. The extra box will be skipped if the values
#'  in the \code{orders} blank.
#' @param labels Named vector, names is the location of the terminal node. The
#' position location should plus 1 after the allocation variables if the allocation
#' is defined.
#' @param kickoff_sidebox remove (default) the side box observations from the 
#' following counting.
#' @param cex Multiplier applied to font size, default is 0.8. Prefer using
#' \code{\link{set_consort_defaults}(txt_gp = gpar(cex = ...))} instead.
#' @param text_width a positive integer giving the target column for wrapping
#' lines in the output. String will not be wrapped if not defined (default).
#' The \code{\link[stringi]{stri_wrap}} function will be used if \code{stringi}
#' package installed, otherwise \code{\link[base]{strwrap}} will be used.
#'
#' @details
#' The calculation of numbers is as in an analogous to Kirchhoff's Laws of
#' electricity. The numbers in terminal nodes must sum to those in the ancestor
#'  nodes. All the drop outs will be populated as a side box. Which was different
#'   from the official CONSORT diagram template, which has dropout inside a
#'   vertical node.
#'
#' @return A \code{consort} object.
#'
#' @export
#'
#' @seealso \code{\link{add_side_box}},\code{\link{add_split}},
#' \code{\link{add_side_box}} \code{\link{textbox}} 
#' \code{\link{set_consort_defaults}}
#' @example inst/examples/consort-plot-example.R
#' @import grid
#' @importFrom stats na.omit
#'
consort_plot <- function(data,
                         orders,
                         side_box,
                         allocation = NULL,
                         labels = NULL,
                         kickoff_sidebox = TRUE,
                         cex = 0.8,
                         text_width = NULL) {

  if(!is.null(cex)){
    old <- set_consort_defaults(txt_gp = gpar(cex = cex), label_txt_gp = gpar(cex = cex))
    on.exit(consort_global$defaults$txt_gp <- old$txt_gp, add = TRUE)
    on.exit(consort_global$defaults$label_txt_gp <- old$label_txt_gp, add = TRUE)
  }

  data <- as.data.frame(data)

  if(!is.list(orders)){
    lst <- vector("list", length = length(orders))
    for(i in seq_along(orders)){
      lst[[i]] <- orders[i]
    }
    orders <- lst
  }

  # Number of variables in each orders.
  ord_len <- sapply(orders, length)

  # Make sure all the side box and allocation is single variable
  side_alloc <- sapply(orders, function(x){
    any(names(x) %in% side_box)
  })
  
  if(any(ord_len != 1 & side_alloc))
    stop("The sidebox must be single variable.")
  
  if(names(orders[[1]]) %in% allocation)
    stop("The first variable can not be a allocation variable.")

  # If all defined variables included in the orders
  if (!all(c(side_box, allocation) %in% names(unlist(orders)))) {
    not_in <- which(!c(side_box, allocation) %in% names(unlist(orders)))
    not_in <- c(side_box, allocation)[not_in]
    stop("Variable ", paste(not_in, collapse = ", "), " not included in the `orders`")
  }

  # If all the orders variables included in the dataset
  if (!all(names(unlist(orders)) %in% names(data))) {
    not_in <- which(!names(unlist(orders)) %in% names(data))
    not_in <- names(unlist(orders))[not_in]
    stop("Variable ", paste(not_in, collapse = ", "), " can not be found in the data")
  }
  
  if(!is.null(allocation)){
    if(length(allocation) > 2)
      stop("A maximum of two treatment allocation is supported.")
    
    for(i in allocation){
      if(!is.factor(data[[i]]))
        data[[i]] <- as.factor(data[[i]])
    }
    
  }
  
  split_ctn <- 0

  for (indx in seq_along(orders)) {
    
    i <- orders[[indx]]
    nd <- setNames(names(i), i)
    
    text_just <- ifelse(length(nd) > 1, "left", "center")

    if (indx == 1) {
      txt <- sapply(seq_along(nd), function(x){
        gen_text(data[[nd[x]]], label = names(nd[x]))
      })
      gp_list <- add_box(txt = txt, 
                         text_width = text_width)
      # data <- data[!is.na(data[[i]]), ]
    } else {
      
      val <- get_val(data, nd)

      if (any(nd %in% side_box)) {
        txt <- gen_text(x = unlst(val), 
                        label = names(nd),
                        bullet = TRUE)

        gp_list <- add_side_box(gp_list,
          txt = txt,
          text_width = text_width
        )

        if(kickoff_sidebox)
          data <- subset_missing(data, nd)
        
      } else if (any(nd %in% allocation)) {
        if(!nd[1] %in% allocation)
          stop("The first element must be in the allocation.")
        
        data <- subset_nonmissing(data, nd[1])
        
        split_ctn <- split_ctn + 1
        
        if(length(nd) > 1 & trimws(names(nd[1])) != "")
          stop("No allocation label is allowed for multiple variable split.")

        if(!trimws(names(nd[1])) == ""){
          txt1 <- gen_text(val[,1], label = names(nd[1]))
          gp_list <- add_box(gp_list,
                             txt = txt1,
                             text_width = text_width
          )
        }
        
        if(split_ctn == 2){
          txt2 <- lapply(val, function(x){
            make_text(value = x, varialbe = nd)
          })
        }else{
          txt2 <- make_text(value = val, varialbe = nd)
          # txt2 <- gen_text(val)
        }
        
        gp_list <- add_split(gp_list,
          txt = txt2,
          just = text_just,
          text_width = text_width
        )
        
        data <- split_data(data, nd[1])
        
      } else {
        # txt <- gen_text(x = val, label = orders[indx], bullet = FALSE)
        if(split_ctn == 2){
          txt <- lapply(val, function(x){
            r <- lapply(x, function(y){
              make_text(value = y, varialbe = nd, 
                        label = TRUE, split = FALSE)
            })
            unlist(r)
          })
          txt <- unlist(txt)
        }else if(split_ctn == 1){
          txt <- lapply(val, function(x){
            make_text(value = x, varialbe = nd, 
                      label = TRUE, split = FALSE)
          })
          txt <- unlist(txt)
        }else{
          txt <- make_text(value = val, varialbe = nd, 
                           label = TRUE, split = FALSE)
        }

        gp_list <- add_box(gp_list,
          txt = txt,
          just = text_just,
          text_width = text_width
        )
      }
    }
  }

  if (!is.null(labels)) {
    if (any(is.na(as.numeric(names(labels))))) {
      stop("Labels must be a named vector with names as the position of
           the node excluding side node.")
    }

    gp_list <- add_label_box(gp_list, txt = labels, only_terminal = TRUE)
  }

  return(gp_list)
}

# Make text for multiple variable
#' @keywords internal
make_text <- function(value, varialbe, label = FALSE, split = TRUE){
  
  if(!label | trimws(names(varialbe[1])) == ""){
    r1 <- gen_text(value[,1])
  }else{
    r1 <- gen_text(value[,1], label = names(varialbe[1]))
  }
  
  if(length(varialbe) > 1){
    r2 <- sapply(2:length(varialbe), function(j){
      y <- varialbe[j]
      if(split)
        gen_text(split(value[,y], value[,1]), label = names(y), bullet = TRUE)
      else
        gen_text(value[,y], label = names(y), bullet = TRUE)
    })
    paste(r1, r2, sep = "\n")
  }else{
    return(r1)
  }
}

# Flatten the list
#' @keywords internal
unlst <- function(lst){
  if (!any((inds <- sapply(lst, is.list)))) return(lst)
  c(lst[!inds], unlst(unlist(lst[inds], recursive = FALSE)))
}


# Split data for the next step
#' @keywords internal
split_data <- function(data, variable){
  if(!is.data.frame(data)){
    sapply(data, function(x){
      split(x, as.factor(x[[variable]]))
    }, simplify = FALSE)
  }else{
    split(data, as.factor(data[[variable]]))
  }
}


# Subset missing data for the next step
#' @keywords internal
subset_missing <- function(data, variable) {
  if (!is.data.frame(data)) {
    sapply(data, function(x){
      if(!is.data.frame(x))
        sapply(x, function(y) y[is.na(y[[variable]]), ],
               simplify = FALSE
        )
      else
        x[is.na(x[[variable]]), ]
    },simplify = FALSE)
  } else {
    data[is.na(data[[variable]]), ]
  }
}

# Subset non-missing data for the next step
#' @keywords internal
subset_nonmissing <- function(data, variable) {
  if (!is.data.frame(data)) {
    sapply(data, function(x){
      if(!is.data.frame(x))
        sapply(x, function(y) y[!is.na(y[[variable]]), ],
               simplify = FALSE
        )
      else
        x[!is.na(x[[variable]]), ]
    },simplify = FALSE)
  } else {
    data[!is.na(data[[variable]]), ]
  }
}

# Extract variable values
# Extract values of a variables form a `data.frame`, `list` or a nested `list`.
#' @keywords internal
get_val <- function(dat, variables){
  if (is.data.frame(dat)) {
    val <- dat[, variables, drop = FALSE]
  } else {
    val <- sapply(dat, function(x){
      if(is.data.frame(x))
        x[, variables, drop = FALSE]
      else{
        sapply(x, function(y) y[, variables, drop = FALSE], simplify = FALSE)
      }
    }, simplify = FALSE)
    
    # if(any(sapply(val, is.list)))
    #   val <- unlist(val, recursive = FALSE)
  }
  
  return(val)
}

# Stack rows top-to-bottom, with extra padding at split/merge transitions
#' @keywords internal
calc_y_coords <- function(consort_plot, nodes_layout, pad_u) {
  nd_y <- vector("list", length = length(nodes_layout))
  prev_bt <- 0

  for (i in seq_along(nodes_layout)) {
    heights <- sapply(consort_plot[nodes_layout[[i]]], function(x)
      get_coords(x$box)$height
    )

    if (i == 1) {
      nd_y[[i]] <- heights / 2 + pad_u / 2
      prev_bt <- max(heights)
    } else {
      # Extra padding when column count changes (split/merge transition)
      extra_pad <- if (length(heights) != length(nd_y[[i - 1]])) 2 * pad_u else pad_u
      nd_y[[i]] <- prev_bt + extra_pad + heights / 2
      prev_bt <- prev_bt + extra_pad + max(heights)
    }
    names(nd_y[[i]]) <- names(heights)
  }

  list(nd_y = nd_y, total_height = prev_bt + pad_u)
}

# Calculate X positions for multi-column rows, accounting for sidebox extents.
# main_col_widths: max node width per column from non-sidebox rows
# sb_wd_mat: matrix of widths from sidebox rows (rows x columns), or NULL
# sb_sd_mat: matrix of sides ("left"/"right") from sidebox rows, or NULL
#' @keywords internal
calc_column_x <- function(main_col_widths, sb_wd_mat, sb_sd_mat, pad_u) {
  n_cols <- length(main_col_widths)
  has_sb <- !is.null(sb_sd_mat)
  pos <- numeric(n_cols)

  # Left extent of column 1
  left_space <- main_col_widths[1] / 2
  if (has_sb && any(sb_sd_mat[, 1] == "left")) {
    sb_wd <- sb_wd_mat[sb_sd_mat[, 1] == "left", 1]
    left_space <- max(c(left_space, max(sb_wd)))
  }
  pos[1] <- left_space + pad_u

  if (n_cols > 1) {
    for (j in 2:n_cols) {
      # Right extent of previous column
      right_space <- main_col_widths[j - 1] / 2
      prev_wide_right <- FALSE
      if (has_sb && any(sb_sd_mat[, j - 1] == "right")) {
        sb_wd <- sb_wd_mat[sb_sd_mat[, j - 1] == "right", j - 1]
        right_space <- max(c(right_space, max(sb_wd)))
        prev_wide_right <- right_space > main_col_widths[j - 1] / 2
      }

      # Left extent of current column
      left_space <- main_col_widths[j] / 2
      if (has_sb && any(sb_sd_mat[, j] == "left")) {
        sb_wd <- sb_wd_mat[sb_sd_mat[, j] == "left", j]
        left_space <- max(c(left_space, max(sb_wd)))
      } else if (prev_wide_right) {
        # Previous column's sidebox already extends past its center,
        # so reduce this column's left extent to avoid excessive spacing
        left_space <- pad_u / 2
      }

      pos[j] <- pos[j - 1] + right_space + pad_u + left_space
    }
  }

  pos - mean(pos)
}

# Offset sidebox nodes from their parent column's X position
#' @keywords internal
place_sideboxes <- function(col_x, sb_widths, sb_sides, pad_u) {
  n_cols <- length(col_x)
  sb_x <- numeric(n_cols)
  for (k in seq_len(n_cols)) {
    sb_x[k] <- if (sb_sides[k] == "right") {
      col_x[k] + sb_widths[k] / 2 + pad_u / 2
    } else {
      col_x[k] - sb_widths[k] / 2 - pad_u / 2
    }
  }
  sb_x
}

# Recenter parent nodes at the midpoint of their children after a second split
#' @keywords internal
adjust_multisplit <- function(nd_x, nd_type, nodes_layout, consort_plot) {
  n_splits <- sum(nd_type == "splitbox")
  if (n_splits <= 1) return(nd_x)
  if (n_splits > 2) stop("More than two splits are not supported.")

  split_idx <- which(nd_type == "splitbox")[-1]
  prev_nodes <- sapply(unlist(nodes_layout[split_idx]), function(y) {
    consort_plot[[y]]$prev_node
  }, simplify = FALSE)
  prev_nodes <- unlist(prev_nodes)

  for (parent in unique(prev_nodes)) {
    children_x <- nd_x[[split_idx]][names(prev_nodes[prev_nodes == parent])]
    nd_x[[split_idx - 1]][parent] <- mean(range(children_x))
  }

  nd_x
}

# Shift all X coordinates so minimum is 0 and compute final bounds
#' @keywords internal
normalize_x <- function(nd_x, nd_wd, nodes_layout) {
  nd_minmax <- lapply(seq_along(nodes_layout), function(i) {
    n_cols <- length(nodes_layout[[i]])
    if (n_cols > 1) {
      wd_mat <- do.call(rbind, nd_wd[i])
      x_mat  <- do.call(rbind, nd_x[i])
      c(minx = min(x_mat[, 1] - wd_mat[, 1] / 2),
        maxx = max(x_mat[, n_cols] + wd_mat[, n_cols] / 2))
    } else {
      c(minx = nd_x[[i]] - nd_wd[[i]] / 2,
        maxx = nd_x[[i]] + nd_wd[[i]] / 2)
    }
  })

  bounds <- do.call(rbind, Filter(Negate(is.null), nd_minmax))
  min_val <- min(bounds[, 1])
  max_val <- max(bounds[, 2])

  for (i in seq_along(nodes_layout)) {
    nd_x[[i]] <- nd_x[[i]] - min_val
  }

  list(nd_x = nd_x, max_width = max_val - min_val)
}

# Calculate coordinates
#' @keywords internal
#' @importFrom stats setNames
calc_coords <- function(consort_plot) {

  nodes_layout <- attr(consort_plot, "nodes.list")

  # Node type per row
  nd_type <- sapply(nodes_layout, function(x)
    unique(sapply(consort_plot[x], "[[", "node_type"))
  )

  if (nd_type[length(nd_type)] == "sidebox")
    stop("The last node can not be a side box.")

  pad_u <- consort_opt("pad_u")
  if (!is.numeric(pad_u) || length(pad_u) != 1L || is.na(pad_u))
    stop("`pad_u` must be a single, non-NA numeric value.")

  # --- Phase 1: Y coordinates ---
  y_result <- calc_y_coords(consort_plot, nodes_layout, pad_u)
  nd_y <- y_result$nd_y

  # --- Phase 2: Gather node widths and sides ---
  nd_wd <- lapply(nodes_layout, function(nd) {
    sapply(consort_plot[nd], function(x) get_coords(x$box)$width)
  })

  nd_sides <- lapply(nodes_layout, function(nd) {
    unlist(sapply(consort_plot[nd], function(x) x$side))
  })

  # --- Phase 3: X coordinates ---
  nd_x <- vector("list", length = length(nodes_layout))
  col_counts <- sapply(nodes_layout, length)
  row_groups <- gp_consecutive(col_counts)

  for (gp in unique(row_groups)) {
    gp_rows <- which(row_groups == gp)
    n_cols <- unique(col_counts[gp_rows])
    gp_types <- nd_type[gp_rows]

    if (n_cols == 1) {
      # Single-column: main nodes at center, sideboxes offset
      for (r in gp_rows) {
        if (nd_type[r] != "sidebox") {
          nd_x[[r]] <- 0
        } else {
          nd_x[[r]] <- ifelse(nd_sides[[r]] == "right",
                              nd_wd[[r]] / 2 + pad_u,
                              -nd_wd[[r]] / 2 - pad_u)
        }
        names(nd_x[[r]]) <- nodes_layout[[r]]
      }
    } else {
      # Multi-column
      wd_mat <- do.call(rbind, nd_wd[gp_rows])
      is_sb <- gp_types == "sidebox"

      if (any(is_sb)) {
        # Column positions with sidebox-aware spacing
        main_col_widths <- apply(wd_mat[!is_sb, , drop = FALSE], 2, max)
        sb_wd_mat <- wd_mat[is_sb, , drop = FALSE]
        sb_sd_mat <- do.call(rbind, nd_sides[gp_rows][is_sb])

        col_x <- calc_column_x(main_col_widths, sb_wd_mat, sb_sd_mat, pad_u)

        for (r in gp_rows) {
          if (nd_type[r] != "sidebox") {
            nd_x[[r]] <- col_x
          } else {
            nd_x[[r]] <- place_sideboxes(col_x, nd_wd[[r]], nd_sides[[r]], pad_u)
          }
          names(nd_x[[r]]) <- nodes_layout[[r]]
        }
      } else {
        # No sideboxes: simple equal spacing
        col_widths <- apply(wd_mat, 2, max)
        col_x <- col_widths / 2 + c(0, cumsum(col_widths[-length(col_widths)] + 4 * pad_u))
        col_x <- col_x - mean(col_x)

        for (r in gp_rows) {
          nd_x[[r]] <- col_x
          names(nd_x[[r]]) <- nodes_layout[[r]]
        }
      }
    }
  }

  # --- Phase 4: Multiple split adjustment ---
  nd_x <- adjust_multisplit(nd_x, nd_type, nodes_layout, consort_plot)

  # --- Phase 5: Normalize to positive coordinates ---
  x_result <- normalize_x(nd_x, nd_wd, nodes_layout)

  list(
    x = unlist(x_result$nd_x),
    y = unlist(nd_y),
    nodes_hw = nd_wd,
    nd_x = x_result$nd_x,
    nd_y = nd_y,
    max_width = x_result$max_width,
    max_height = y_result$total_height
  )
}

# Calculate coordinates
#' @keywords internal
#'
calc_coords_label <- function(label_plot, node_y, max_h){

  lab_wd <- sapply(label_plot, function(x){
    c(w = get_coords(x$box)$width, h = get_coords(x$box)$height)
  })
  
  lab_pos <- sapply(label_plot, function(x){
    x$prev_node
  })
  
  lab_y <- node_y[lab_pos]
  lab_y <- sapply(lab_y, mean)
  lab_y <- (max_h - lab_y)/max_h
  names(lab_y) <- colnames(lab_wd)

  lab_x <- (lab_wd["w",]/2)
  names(lab_x) <- colnames(lab_wd)

  return(list(width = max(lab_wd["w",]),
              x = lab_x, # Put inside
              y = lab_y))
  
}

# Create groups if consecutive 
#' @keywords internal
#'
#'
gp_consecutive <- function(x){
  int <- 1
  gp <- vector("character", length = length(x))
  gp[1] <- letters[int]
  if(length(x) > 1){
    for(i in 2:length(x)){
      if(x[i] != x[i-1])
        int <- int + 1
      gp[i] <- letters[int]
    }
  }
  return(gp)
}

#' Add methods to print function
#'
#' Method for plot objects and display the output in on a grid device.
#'
#' @param x A \code{consort} object.
#' @param grViz If use \link[DiagrammeR]{grViz} to print the plot. 
#' Default is \code{FALSE} to use \link[grid]{grid.draw}
#' @param ... Not used.
#'
#' @seealso \code{\link{add_side_box}},\code{\link{add_split}},
#' \code{\link{add_side_box}}, \link[grid]{grid.draw}
#'
#' @return None.
#'
#' @rdname plot.consort
#' @export
plot.consort <- function(x, grViz = FALSE, ...) {
  if(!grViz){
    r <- build_grid(x)
    grid.newpage()
    grid.draw(r)
  }else{
    if (requireNamespace("DiagrammeR", quietly = TRUE)) {
      grviz_txt <- build_grviz(x)
      DiagrammeR::grViz(grviz_txt)
    } else {
      stop("package `DiagrammeR` is needed to draw grViz plot.")
    }
  }
}

#' @rdname plot.consort
#' @export
print.consort <- plot.consort

# Check if empty
#' @keywords internal
#'
is_empty <- function(x){
  x <- gsub("[[:space:]]", "", x)
  is.null(x) | x == "" | is.na(x)
}


# Wrap text
#' @keywords internal
#'
text_wrap <- function(txt, width = 0.9) {
  if (length(txt) > 1) {
    stop("Vector does not supported!")
  }

  # Split text by line break
  s_txt <- unlist(strsplit(txt, split = "\n"))
  if (requireNamespace("stringi", quietly = TRUE)) {
    s_txt <- stringi::stri_wrap(s_txt, width, 0)
  } else {
    s_txt <- strwrap(s_txt, width)
  }
  paste(s_txt, collapse = "\n")
}

#' @keywords internal
init_auto_index <- function() {
  index <- 0
  function() {
    index <<- index + 1
    index
  }
}

auto_index <- init_auto_index()

# Generate connections
#' @keywords internal
get_connect <- function(consort_plot){
  
  connection <- sapply(names(consort_plot), function(x){
    prev_node <- consort_plot[[x]]$prev_node
    
    if(is.null(prev_node))
      return(NULL)
    else
      list(node = c(x, prev_node),
           connect = ifelse(is.null(consort_plot[[x]]$side), "bt",
                            ifelse(consort_plot[[x]]$side== "right", "bl", "br")) )
    
  })
  Filter(Negate(is.null), connection)
  
}

#' Add a vertically aligned label nodes on the left side.
#'
#' In a consort diagram, this can be used to indicate different stage.
#'
#' @param prev_box A completed diagram created with \code{add_box}, \code{add_side_box} etc.
#' @param txt Text in the node. If a character string is provided, the label will be aligned
#'  to the last box if a character is provided. If a named vector, the labels will align to
#' corresponding row of the node. And the names is the number indicating row number of box to
#'  horizontally align with and value is the text in the box.
#' @param only_terminal If the txt is only for the terminal box, default. Otherwise, the side box will
#' also be accounted for.
#' @param just The justification for the text: center (default), left or right.
#' @param box_fn The function to create the box grob. Default is \code{\link[grid]{roundrectGrob}}.
#' @param ... Other parameters pass to \link{textbox}. 
#'
#' @export
#' @seealso \code{\link{add_side_box}} \code{\link{add_split}} \code{\link{textbox}} 
#' \code{\link{add_box}}
#' @return A \code{consort} object.
#' 
#' @importFrom utils modifyList
#' @example inst/examples/add-box-example.R

add_label_box <- function(prev_box,
                          txt,
                          only_terminal = TRUE,
                          just = c("center", "left", "right"),
                          box_fn = roundrectGrob,
                          ...) {

  just <- match.arg(just)

  dots <- list(...)
  if("name" %in% names(dots)){
    warning("`parameter name was provided but will be ignored.")
    dots$name <- NULL
  }

  if (length(txt) > 1 & is.null(names(txt))) {
    stop("txt must be a named vector.")
  }

  if (length(txt) > 1 & is.null(names(txt)) & !any(is.na(as.numeric(names(txt))))) {
    stop("txt names must be number indicating the row position of the terminal
          node to be aligned.")
  }

  if (!inherits(prev_box, c("consort"))) {
    stop("prev_box must be consort object")
  }

  if(!"txt_gp" %in% names(dots)){
    dots$txt_gp <- consort_opt("label_txt_gp")
  }

  if(!"box_gp" %in% names(dots)){
    dots$box_gp <- consort_opt("label_box_gp")
  }

  # Set default values
  args_list <- list()
  # args_list$text <- txt
  # args_list$txt_gp <- label_txt_gp
  # args_list$box_gp <- label_box_gp
  # args_list$box_fn <- roundrectGrob
  args_list$name <- "label"

  args_list <- modifyList(args_list, dots)
  
  # Node type of each
  nodes_layout <- attr(prev_box, "nodes.list")
  nd_type <- sapply(nodes_layout, function(x) 
    unique(sapply(prev_box[x], "[[", "node_type"))
  )
  
  if (only_terminal) {
    # Get the index of the vertical box
    grob_index <- which(nd_type %in% c("vertbox", "splitbox"))
  } else {
    grob_index <- seq_along(nodes_layout)
  }
  
  num_nodes <- attr(prev_box, "nodes.num")

  if (length(txt) == 1 & is.null(names(txt)))
    names(txt) <- 1

  if(length(txt) > length(nd_type))
    stop("provided text is larger than diagram node numbers.")

  nodes <- lapply(seq_along(txt), function(i){
    box <- do.call(textbox, c(list(text = txt[i]), args_list))
    list(
      text = txt[i],
      node_type = "label",
      box = box,
      box_hw = get_coords(box),
      side = NULL,
      just = just,
      gpar = args_list[c("txt_gp", "box_gp")],
      prev_node = grob_index[as.numeric(names(txt[i]))]
    )
  })
  
  names(nodes) <- paste0("label", num_nodes + seq_along(txt))
  
  node_list <- c(prev_box, nodes)
  
  class(node_list) <- union("consort", class(node_list))
  
  structure(node_list,
            nodes.num = length(txt) + num_nodes,
            nodes.current = names(nodes),
            nodes.type = "label",
            nodes.list = attr(prev_box, "nodes.list")
  )
}

#' Build consort diagram
#'
#' Build a \code{grob} consort diagram, use this if you want
#' to save plots with \code{\link[ggplot2]{ggsave}}
#'
#' @param x A conosrt object.
#' 
#' @return A \code{Graphviz} code
#'
#' @seealso \code{\link[DiagrammeR]{grViz}}
#' @export 
#' @examples
#' \dontrun{
#' txt1 <- "Population (n=300)"
#' txt1_side <- "Excluded (n=15): \n
#'               \u2022 MRI not collected (n=3)\n
#'               \u2022 Tissues not collected (n=4)\n
#'               \u2022 Other (n=8)"
#'
#' g <- add_box(txt = txt1)
#'
#' g <- add_side_box(g, txt = txt1_side)
#'
#' g <- add_box(g, txt = "Randomized (n=200)")
#' # plot(g, grViz = TRUE)
#' }
build_grviz <- function(x) {

  if (!inherits(x, c("consort")))
    stop("x must be consort object")
  
  # Get the maximum of height and width of each node
  nodes_layout <- attr(x, "nodes.list")

  # Generate connection
  nodes_connect <- get_connect(x)
  nodes_connect <- sapply(nodes_connect, function(x)x$node[2])
  
  if(any(grepl("label", names(x)))){
    consort_plot <- x[grepl("node", names(x))]
    label_plot <- x[grepl("label", names(x))]
    
    lab_txt <- lapply(names(label_plot), function(x){
      box <- label_plot[[x]]$box
      grviz_style <- gpar_to_grviz_attrs(box$txt_gp, box$box_gp, box$box_fn)
      txt_lab <- mk_text_align(label_plot[[x]]$text, label_plot[[x]]$just,
                               grviz_style = grviz_style)
      c(nam = x,
        node = paste(x, txt_lab),
        pos = label_plot[[x]]$prev_node)
    })
    lab_txt <- do.call(rbind, lab_txt)

    lab_nd <- paste(lab_txt[,"node"], collapse = "\n")
    lab_edge <- sprintf("edge[style=invis];\n%s;\n", 
                        paste(lab_txt[,"nam"], collapse = " -> "))
    lab_rnk <- nodes_layout[as.numeric(lab_txt[,"pos"])]
    names(lab_rnk) <- lab_txt[,"nam"]
    
  }else{
    consort_plot <- x
    lab_nd <- NULL
    lab_edge <- NULL
    lab_rnk <- NULL
  }
  
  # Reset the indexing to 0
  set_invs(0)
  
  # Node type of each
  nd_type <- sapply(nodes_layout, function(x) 
    unique(sapply(consort_plot[x], "[[", "node_type"))
  )
  
  if(nd_type[length(nd_type)] == "sidebox")
    stop("last box cannot be a side box.")
  
  main_txt <- lapply(names(consort_plot), function(nd){
    text <- consort_plot[[nd]]$text
    just <- consort_plot[[nd]]$just
    box <- consort_plot[[nd]]$box
    grviz_style <- gpar_to_grviz_attrs(box$txt_gp, box$box_gp, box$box_fn)
    style_str <- if(is.null(grviz_style)) "" else grviz_style
    txt_lab <- mk_text_align(text, just, grviz_style = grviz_style)
    c(nam = nd, text = text, just = just, grviz_style = style_str, node = paste(nd, txt_lab))
  })
  main_txt <- do.call(rbind, main_txt)
  # Remove empty label
  null_indx <- is_empty(main_txt[,"text"]) & !main_txt[,"nam"] %in% unlist(nodes_layout[nd_type == "vertbox"])
  main_txt <- main_txt[!null_indx,]
  
  rnk_nd <- vector("character") # Ranking
  inv_nd <- vector("character") # Invisible node
  con_nd <- vector("character") # Connections
  
  # For multiple split
  if(sum(nd_type == "splitbox") > 2)
    stop("More than two splits are not supported.")
  
  # Group name
  # Number of nodes at each level
  nd_len <- unique(sapply(nodes_layout, length))
  names(nd_len) <- LETTERS[1:length(nd_len)]
  group_name <- lapply(nodes_layout, function(x){
    r <- names(nd_len[nd_len == length(x)])
    r <- paste0(r, seq_along(x))
    names(r) <- x
    return(r)
  })
  
  # Main nodes
  for(i in seq_along(nd_type)){
    
    nd <- nodes_layout[[i]]

    if(nd_type[i] != "sidebox"){
      # Update node labels
      main_txt <- update_node_label(main_txt = main_txt,
                                    node = nd,
                                    group_name = group_name[[i]])
    }
    
    # Skip if this is the last one
    if(i == length(nd_type))
      next
    
    if(length(nodes_layout[[i+1]]) != length(nd)){
      
      prev_node <- lapply(nodes_layout[[i+1]], function(ndx){
        fm_node <- consort_plot[[ndx]]$prev_node
        data.frame(from = fm_node,
                   to = rep(ndx, length(fm_node)))
      })
      prev_node <- do.call(rbind, prev_node)
      
      nd_lst <- mk_invs_split(from_node = prev_node$from, 
                              to_node = prev_node$to,
                              consort_plot = consort_plot,
                              current_group = group_name[[i]],
                              next_group = group_name[[i+1]])
      
      rnk_nd <- c(rnk_nd, nd_lst[["rank"]])
      inv_nd <- c(inv_nd, nd_lst[["invs"]])
      con_nd <- c(con_nd, nd_lst[["connect"]])
      
      next
    }
    
    # The next one and current one is not a side box
    if(length(nodes_layout[[i+1]]) == length(nd) & all(!c(nd_type[i], nd_type[i+1]) %in% "sidebox")){
      
      for(j in seq_along(nd)){

        current_nd <- nodes_layout[[i]][j]
        if(current_nd %in% nodes_connect & !is_empty(consort_plot[[current_nd]]$text)){
          connect_to <- max(which(nodes_connect == current_nd))
          connect_to <- names(nodes_connect[connect_to])
          if(!is_empty(consort_plot[[connect_to]]$text))
            con_nd <- c(con_nd, sprintf("%s -> %s;", 
                                          current_nd,
                                          connect_to))
          else
            con_nd <- c(con_nd, sprintf("%s -> %s [arrowhead = none];", 
                                          current_nd,
                                          connect_to))
        }
        
      }
      rnk_nd <- c(rnk_nd, mk_subgraph_rank(nd))
      rnk_nd <- c(rnk_nd, mk_subgraph_rank(nodes_layout[[i+1]]))

      next
    }
    
    if(nd_type[i] == "sidebox"){
      sp_box <- nd
      if(length(nd) == 1){
        # Strip white space
        if(is_empty(consort_plot[[nd]]$text)){
          con_nd <- c(con_nd, sprintf("%s -> %s;", 
                                      nodes_layout[[i-1]], 
                                      nodes_layout[[i+1]]))
        }else{
          nd_lst <- mk_invs_side(nodes_layout[[i-1]], 
                                 nd, 
                                 nodes_layout[[i+1]],
                                 group = group_name[[i]])
          rnk_nd <- c(rnk_nd, nd_lst[["rank"]])
          inv_nd <- c(inv_nd, nd_lst[["invs"]])
          con_nd <- c(con_nd, nd_lst[["connect"]])
        }
      }else{
        for(j in seq_along(nd)){

          # Skip if text is blank
          if(is_empty(consort_plot[[nd[j]]]$text)){
            # nd_next <- nodes_layout[[i+1]][j]
            nd_next <- max(which(nodes_connect == nodes_layout[[i-1]][j]))
            nd_next <- names(nodes_connect[nd_next])
            # No arrow if next link has no text
            if(!is_empty(consort_plot[[nd_next]]$text)){
              con_nd <- c(con_nd, sprintf("%s -> %s;", 
                                          nodes_layout[[i-1]][j],
                                          nd_next))
            }
            sp_box <- sp_box[-j]
          }else{
            nd_lst <- mk_invs_side(nodes_layout[[i-1]][j], 
                                   nd[j], 
                                   nodes_layout[[i+1]][j],
                                   group = group_name[[i]][j])
            
            rnk_nd <- c(rnk_nd, nd_lst[["rank"]])
            inv_nd <- c(inv_nd, nd_lst[["invs"]])
            con_nd <- c(con_nd, nd_lst[["connect"]])
          }
          
          if(length(nodes_layout[[i+1]]) > 1)
            rnk_nd <- c(rnk_nd, mk_subgraph_rank(nodes_layout[[i+1]]))
          
          if(length(sp_box) > 1)
            rnk_nd <- c(rnk_nd, mk_subgraph_rank(sp_box))
        }
      }
      
      next  
    }
    
    if(nd_type[i] == "vertbox"){
      rnk_nd <- c(rnk_nd, mk_subgraph_rank(nodes_layout[[i]]))
    }
  }
  
  # Make sure all connections are linked
  if(any(!grepl(paste(nodes_connect, collapse = "|"), con_nd))){
    nd_none <- sapply(nodes_connect, function(x){
      !grepl(x, con_nd)
    })
    nd_none <- apply(nd_none, 2, all)
    nd_none <- nodes_connect[nd_none]
    
    con_nd_extra <- sapply(seq_along(nd_none), function(idx){
      con_from <- nd_none[idx]
      con_to <- names(nd_none[idx])
      if(consort_plot[[con_to]]$node_type == "sidebox"){
        return("")
      }else{
        if(!is_empty(consort_plot[[con_to]]$text))
          sprintf("%s -> %s;", con_from, con_to)
        else
          sprintf("%s -> %s [arrowhead = none];", con_from, con_to)
      }
    })
    
  }else{
    con_nd_extra <- ""
  }

  
  rnk_nd <- unique(rnk_nd)
  inv_nd <- unique(inv_nd)
  con_nd <- unique(c(con_nd, con_nd_extra))
  
  if(!is.null(lab_rnk)){
    for(i in names(lab_rnk)){
      indx <- grepl(paste(sprintf("\\<%s\\>", lab_rnk[[i]]), collapse = "|"),
                    rnk_nd)
      if(any(indx)){
        repl_str <- sprintf("\\1 %s; \\2", i)
        rnk_nd[indx] <- gsub(rnk_nd[indx], 
                             pattern = "(.{39})(.*)",
                             replacement = repl_str)
      }else{
        rnk_nd <- c(rnk_nd, mk_subgraph_rank(c(i, lab_rnk[[i]])))
      }
      
    }
    
  }
  
  # Remove empty labels
  # main_txt <- main_txt[!is_empty(main_txt[,"text"]),]
  
  # Build dot

  # Edge attributes from arrow settings
  edge_attrs <- arrow_to_grviz_attrs(consort_opt("arrow_gp"),
                                     consort_opt("arrow_type"),
                                     consort_opt("arrow_length"))
  edge_stmt <- sprintf("edge[%s];", edge_attrs)

  grviz_txt <- paste("digraph consort_diagram {
  graph [layout = dot, splines=ortho]",
  "node [shape = rectangle]",
  lab_nd,
  lab_edge,
  "# node definitions with substituted label text",
  paste(main_txt[,"node"], collapse = "\n"), # Node
  "\n## Invisible point node for joints",
  "node [shape = point, width = 0, style=invis]",
  paste(inv_nd, collapse = "\n"),             # Invisible node
  paste(rnk_nd, collapse = "\n"),            # Ranks
  edge_stmt,
  paste(con_nd, collapse = "\n"),            # Connections
  "\n}\n",
  sep = "\n\n")
  
  return(grviz_txt)
  
}


# Update node label
#' @keywords internal

update_node_label <- function(main_txt, node, group_name){

  for(j in seq_along(node)){
    idx_row <- main_txt[,1] == node[j]
    style <- main_txt[idx_row, "grviz_style"]
    if(nchar(style) == 0) style <- NULL
    main_txt[idx_row, "node"] <- paste(node[j],
                                       mk_text_align(main_txt[idx_row, "text"],
                                                     main_txt[idx_row, "just"],
                                                     group = group_name[j],
                                                     grviz_style = style))
  }

  return(main_txt)
}

#' Create Consort diagram
#'
#' To make it easy to create CONSORT diagrams for the transparent reporting
#'  of participant allocation in randomized, controlled clinical trials. This
#'   is done by creating a standardized disposition data, and using this data
#'    as the source for the creation a standard CONSORT diagram. Human effort
#'     by supplying text labels on the node can also be achieved.
#'     
#' @docType package
#' @name consort
#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import grid
#' @importFrom utils modifyList
## usethis namespace: end
NULL

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  dpi = 300,
  collapse = TRUE,
  comment = "#>",
  message = FALSE
)

## ----setup--------------------------------------------------------------------
library(consort)
data(dispos.data)

## ----data-head, echo=FALSE----------------------------------------------------
head(dispos.data)

## ----example, eval=FALSE------------------------------------------------------
# consort_plot(data,
#              orders,
#              side_box,
#              allocation = NULL,
#              labels = NULL,
#              cex = 0.8,
#              text_width = NULL,
#              widths = c(0.1, 0.9))

## ----patchwork, eval=FALSE----------------------------------------------------
# library(patchwork)
# 
# wrap_elements(build_grid(g)) + plot_annotation(
#   title = 'Consort diagram',
#   subtitle = 'Flow chart of the XX study',
#   caption = 'Disclaimer: None of these plots are insightful'
# )

## ----dump-dotfile, eval = FALSE-----------------------------------------------
# cat(build_grviz(g), file = "consort.gv")

## ----defualts-----------------------------------------------------------------
get_consort_defaults()

## ----set-restore, eval = FALSE------------------------------------------------
# set_consort_defaults(arrow_gp = gpar(col = "red", lwd = 2))
# # ... draw diagram ...
# init_consort_defaults() # restore

## ----markup-grid, fig.width = 7, fig.height = 5, out.width="70%"--------------
library(grid)
set_consort_defaults(txt_gp = gpar(cex = 0.8), parse_markup = TRUE)

g <- add_box(txt = "**Patients screened** (n=300)") |>
  add_side_box(txt = "Excluded^{1} (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=12)") |>
  add_box(txt = "__Randomised__ (n=285)") |>
  add_split(txt = c("*Arm A* (n=143)", "*Arm B* (n=142)")) |>
  add_box(txt = c("Analysed (n=128)", "Analysed (n=135)")) |>
  add_label_box(txt = c("1" = "Screening", "3" = "Randomisation"))
plot(g)

## ----markup-grviz, out.width="70%", fig.width  = 3, fig.height = 1.5----------
plot(g, grViz = TRUE)

## ----single-arms, fig.width  = 6, fig.height = 6, out.width="70%"-------------
out <- consort_plot(data = dispos.data,
                    orders = c(trialno = "Population",
                               exclusion = "Excluded",
                               trialno = "Allocated",
                               subjid_notdosed = "Not dosed",
                               followup = "Followup",
                               lost_followup = "Not evaluable for the final analysis",
                               mitt = "Final Analysis"),
                    side_box = c("exclusion", "subjid_notdosed", "lost_followup"),
                    cex = 0.9)
plot(out)

## ----multiple-phase, fig.width  = 9, fig.height = 6, out.width="90%"----------
g <- consort_plot(data = dispos.data,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm3     = "Randomized patient",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = "arm3",
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "5" = "End of study"),
                  cex = 0.7)
plot(g)

## ----multiple-split, fig.width  = 12, fig.height = 8, out.width = "95%"-------
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
g <- consort_plot(data = df,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm = "Randomized patient",
                             arm3     = "",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "6" = "End of study"),
                  cex = 0.7)
plot(g)

## ----multiple-split-stack, fig.width  = 13, fig.height = 10, out.width = "95%"----
# We will exclude one arm to avoid too many arms.
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                # The following two variables will be stacked together
                                c(arm3     = "", # Should not provide a value to show the actual arm
                                  subjid_notdosed="Participants not treated"),
                                # The following two variables will be stacked together
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"), 
                  allocation = c("arm", "arm3"), # Two level randomisation
                  kickoff_sidebox = FALSE,
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)

plot(p)

## ----providetext1, fig.width  = 7, fig.height = 4, out.width="60%"------------
library(grid)
# Might want to change some settings
set_consort_defaults(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)

## ----providetext2, fig.width  = 3, fig.height = 1.5, out.width="80%"----------
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
# Length needs to be the same as previous one, use a list here.
g <- add_split(g, txt = list(c("Process 1 (n=140)", "Process 2 (n=140)",
                               "Process 3 (n=142)")))

plot(g, grViz = TRUE)


## ----manual-multisplit, fig.width  = 13, fig.height = 6, out.width = "80%"----
g <- add_box(txt = "Patient consented (n=200)")
g <- add_side_box(g, txt = "Excluded (n=10):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=7)")
g <- add_box(g, txt = "Randomised (n=190)")
g <- add_split(g, txt = c("Randomised study (n=144)", "Preference study (n=146)"))

# Provide a list here
g <- add_split(g, txt = list(c("Allocated to surgery (n=70)", 
                                "Allocated to medicine (n=74)"),
                             c("Allocated to surgery (n=47)", 
                              "Allocated to medicine (n=48)",
                               "Placebo (n=51)")))

g <- add_side_box(g, txt = c("Excluded (n=3):\n\u2022 Withdrawn before surgery (n=2)\n\u2022 Declined (n=1)", 
                             "", 
                             "Excluded (n=1):\n\u2022 Withdrawn before surgery (n=1)",
                             "Excluded (n=3):\n\u2022 Withdrawn before surgery (n=1)\n\u2022 Declined (n=1)",
                             "Excluded (n=10):6\n\u2022 Declined (n=10)"),
                  side = rep("right", 5))
g <- add_box(g, txt = c("Analysed (n=67)", "Analysed (n=74)",
                        "Analysed (n=46)", "Analysed (n=45)", "Analysed (n=41)"))

plot(g)


## ----disposition-data, fig.width  = 7, fig.height = 6.5, out.width="70%"------
set_consort_defaults(txt_gp = gpar(cex = 0.8))

dispos.data$arm <- factor(dispos.data$arm)

txt <- gen_text(dispos.data$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(dispos.data$exclusion, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

g <- add_box(g, txt = gen_text(dispos.data$arm, label = "Patients randomised")) 

txt <- gen_text(dispos.data$arm)
g <- add_split(g, txt = txt)

# Exclude subjects
dispos.data <- dispos.data[is.na(dispos.data$exclusion), ]

txt <- gen_text(split(dispos.data[,c("subjid_notdosed")], dispos.data$arm),
                label = "Not dosed", bullet = TRUE)
g <- add_box(g, txt = txt, just = "left")

txt <- gen_text(split(dispos.data$mitt, dispos.data$arm),
                label = "Primary mITT analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "5" = "Final analysis"))

plot(g)

## ----save-plot, fig.width  = 3, fig.height = 1.5, out.width="70%"-------------
plot(g, grViz = TRUE)

## ----eval=FALSE---------------------------------------------------------------
# # save plots
# png("consort_diagram.png", width = 29,
#     height = 21, res = 300, units = "cm", type = "cairo")
# plot(g)
# dev.off()
# 

## ----eval=FALSE---------------------------------------------------------------
# ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))

## ----eval=FALSE---------------------------------------------------------------
# plot(g, grViz = TRUE) |>
#     DiagrammeRsvg::export_svg() |>
#     charToRaw() |>
#     rsvg::rsvg_pdf("svg_graph.pdf")

consort_global <- new.env(parent = emptyenv())

# Default settings for consort diagram 
consort_defaults_settings <- list(
    arrow_gp = gpar(col = "black", lwd = 1),
    txt_gp = gpar(cex = 1, col = "black"),
    box_gp = gpar(fill = "white"),
    label_txt_gp = gpar(col = "#4F81BD", cex = 1, fontface = "bold"),
    label_box_gp = gpar(fill = "#A9C7FD"),
    arrow_length = 0.1,
    arrow_type = "closed",
    pad_u = 3,
    bullet = "\u2022",
    parse_markup = FALSE
)

consort_global$defaults <- consort_defaults_settings

# Internal accessor for a single default value
#' @keywords internal
consort_opt <- function(name) {
  consort_global$defaults[[name]]
}

#' Set consort diagram default options
#'
#' Modify the default graphical parameters and other settings for consort diagrams.
#' Any parameter set to \code{NULL} (the default) will remain unchanged.
#'
#' @param arrow_gp A \code{\link[grid]{gpar}} object for the arrow line.
#' @param txt_gp A \code{\link[grid]{gpar}} object for the text inside boxes.
#' @param box_gp A \code{\link[grid]{gpar}} object for the box border and fill.
#' @param label_txt_gp A \code{\link[grid]{gpar}} object for the label text.
#' @param label_box_gp A \code{\link[grid]{gpar}} object for the label box.
#' @param arrow_length Numeric, length of the arrowhead in inches.
#' @param arrow_type Character, arrow type: \code{"closed"} or \code{"open"}.
#' @param pad_u Numeric, padding between nodes.
#' @param bullet Character, bullet character for side box items.
#' @param parse_markup Logical, whether to parse lightweight markup syntax
#'   (\code{**bold**}, \code{*italic*}, \code{^{superscript}},
#'   \code{_{subscript}}, \code{__underline__}) in node labels.
#'   Default is \code{FALSE}.
#'
#' @return Invisibly returns the previous defaults (a \code{consort_defaults} object).
#' @export
#' @examples
#' # Change text color and box fill
#' old <- set_consort_defaults(
#'   txt_gp = grid::gpar(col = "navy", cex = 0.9),
#'   box_gp = grid::gpar(fill = "#F0F0F0")
#' )
#'
#' # View current defaults
#' get_consort_defaults()
#'
#' # Restore previous defaults
#' set_consort_defaults(
#'   txt_gp = old$txt_gp,
#'   box_gp = old$box_gp
#' )
set_consort_defaults <- function(
    arrow_gp = NULL,
    txt_gp = NULL,
    box_gp = NULL,
    label_txt_gp = NULL,
    label_box_gp = NULL,
    arrow_length = NULL,
    arrow_type = NULL,
    pad_u = NULL,
    bullet = NULL,
    parse_markup = NULL
) {

  old <- get_consort_defaults()

  # Map parameter names to option names
  args <- list(
    arrow_gp     = arrow_gp,
    txt_gp       = txt_gp,
    box_gp       = box_gp,
    label_txt_gp = label_txt_gp,
    label_box_gp = label_box_gp,
    arrow_length = arrow_length,
    arrow_type   = arrow_type,
    pad_u        = pad_u,
    bullet       = bullet,
    parse_markup = parse_markup
  )

  # Keep only non-NULL arguments
  args <- Filter(Negate(is.null), args)

  if (length(args) == 0) return(invisible(old))

  # Validate and merge gpar arguments into existing defaults
  gpar_params <- c("arrow_gp", "txt_gp", "box_gp",
                    "label_txt_gp", "label_box_gp")
  for (nm in intersect(names(args), gpar_params)) {
    if (!inherits(args[[nm]], "gpar"))
      stop(sprintf("`%s` must be a gpar() object.", nm))
    args[[nm]] <- do.call(gpar,
                          utils::modifyList(as.list(consort_global$defaults[[nm]]),
                                            as.list(args[[nm]])))
  }

  if (!is.null(arrow_length)) {
    if (!is.numeric(arrow_length) || length(arrow_length) != 1 || is.na(arrow_length))
      stop("`arrow_length` must be a single numeric value.")
  }

  if (!is.null(arrow_type)) {
    arrow_type <- match.arg(arrow_type, c("closed", "open"))
    args$arrow_type <- arrow_type
  }

  if (!is.null(pad_u)) {
    if (!is.numeric(pad_u) || length(pad_u) != 1 || is.na(pad_u))
      stop("`pad_u` must be a single numeric value.")
  }

  if (!is.null(bullet)) {
    if (!is.character(bullet) || length(bullet) != 1)
      stop("`bullet` must be a single character string.")
  }

  if (!is.null(parse_markup)) {
    if (!is.logical(parse_markup) || length(parse_markup) != 1 || is.na(parse_markup))
      stop("`parse_markup` must be a single TRUE/FALSE value.")
  }

  # Update stored defaults
  consort_global$defaults <- utils::modifyList(consort_global$defaults, args)

  invisible(old)
}

#' Get consort diagram default options
#'
#' @return A \code{consort_defaults} object containing all current default settings.
#' @export
#' @rdname set_consort_defaults
#' @examples
#' get_consort_defaults()
get_consort_defaults <- function() {
  x <- consort_global$defaults
  class(x) <- "consort_defaults"
  x
}

#' @rdname set_consort_defaults
#' @export
init_consort_defaults <- function() {
  x <- consort_defaults_settings
  consort_global$defaults <- x
  class(x) <- "consort_defaults"
  invisible(x)
}

#' @param x A \code{consort_defaults} object.
#' @param ... Not used.
#' @rdname set_consort_defaults
#' @export
print.consort_defaults <- function(x, ...) {
  cat("Consort diagram default settings:\n\n")

  for (nm in names(x)) {
    label <- nm
    val <- x[[nm]]

    if (inherits(val, "gpar")) {
      parts <- vapply(names(val), function(p) {
        v <- val[[p]]
        if (is.numeric(v) && !is.null(names(v))) {
          # gpar stores fontface as named numeric (e.g., font = c(bold = 2))
          sprintf('%s = "%s"', p, names(v))
        } else if (is.character(v)) {
          sprintf('%s = "%s"', p, v)
        } else {
          sprintf("%s = %s", p, v)
        }
      }, character(1))
      cat(sprintf("  %-14s: gpar(%s)\n", label, paste(parts, collapse = ", ")))
    } else if (is.character(val)) {
      cat(sprintf("  %-14s: \"%s\"\n", label, val))
    } else {
      cat(sprintf("  %-14s: %s\n", label, val))
    }
  }

  invisible(x)
}

test_that("set_consort_defaults merges gpar values", {

  old <- set_consort_defaults(txt_gp = gpar(lwd = 1))
  on.exit(consort_global$defaults <- old, add = TRUE)

  new_txt <- consort_opt("txt_gp")
  # lwd was added
  expect_equal(new_txt$lwd, 1)
  # existing col was preserved
  expect_equal(new_txt$col, "black")

  set_consort_defaults(arrow_gp = gpar(col = "red"))
  new_arrow <- consort_opt("arrow_gp")
  # col was updated
  expect_equal(new_arrow$col, "red")
  # existing lwd was preserved
  expect_equal(new_arrow$lwd, 1)

})

to_grviz <- function(x) {
  path <- tempfile(fileext = ".gv")
  x <- gsub("\u2022", "", x) 
  cat(x, file = path)
  path
}

save_png <- function(x, width = 9, height = 9) {
  path <- tempfile(fileext = ".png")
  png(path, width = width, height = height, 
      units = "in", type = "cairo-png", res = 300)
  on.exit(dev.off())
  plot(x)
  path
}

test_that("Check plot creation", {
  g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)"))
  g <- add_box(g, txt = "Included All (n=20)")
  g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
  g <- add_box(g, txt = "Randomised")
  g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
  g <- add_box(g, txt = c("Follow-up (n=20)",
                          "Follow-up (n=7)"))
  
  g <- add_side_box(g, txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                               "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)"))
  
  g <- add_box(g, txt = c("Final analysis (n=128)", "Final analysis (n=135)"))
  
  g <- add_label_box(g,
                     txt = c("1" = "Screening", "3" = "Randomized", "6" = "Final analysis"))

  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "build-grviz.png")
  
})


test_that("New options", {
  
  set_consort_defaults(
    arrow_gp = gpar(col = "green"),
    label_txt_gp = gpar(col = "red")
  )
  g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)"))
  g <- add_box(g, txt = "Included All (n=20)")
  g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
  g <- add_box(g, txt = "Randomised")
  g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
  g <- add_box(g, txt = c("Follow-up (n=20)",
                          "Follow-up (n=7)"))
  
  g <- add_side_box(g, txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                               "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)"))
  
  g <- add_box(g, txt = c("Final analysis (n=128)", "Final analysis (n=135)"))
  
  g <- add_label_box(g,
                     txt = c("1" = "Screening", "3" = "Randomized", "6" = "Final analysis"))
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "grviz-withopts.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "build-grviz-withopts.png")
  
})

init_consort_defaults()

test_that("Missing in some nodes", {
  g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)"))
  g <- add_box(g, txt = "Included All (n=20)")
  g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
  g <- add_box(g, txt = "Randomised")
  g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
  g <- add_side_box(g, txt = c("", "Exclude (n=3"))
  g <- add_box(g, txt = c("", "From Arm B"))
  g <- add_box(g, txt = c("This is it", "From Arm B"))

  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "multi-miss-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "multi-miss-grviz.png")
  
})


test_that("End with missing", {
  g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)"))
  g <- add_box(g, txt = "Included All (n=20)")
  g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
  g <- add_box(g, txt = "Randomised")
  g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
  g <- add_side_box(g, txt = c("", "Exclude (n=3"))
  g <- add_box(g, txt = c("", "From Arm B"))
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "end-miss-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  skip_on_ci()
  expect_snapshot_file(save_png(g), "end-miss-grviz.png")
  
})


test_that("Split and combine", {
  g <- add_box(txt = c("Study 1 (n=8)", "Study 2 And this is long (n=12)", "Study 3 (n=12)", "Study 3 (n=12)", "Study 3 (n=12)"))
  g <- add_box(g, txt = "Included All (n=20)")
  g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
  g <- add_box(g, txt = "Randomised")
  g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
  g <- add_box(g, txt = c("", "From Arm B"))
  g <- add_box(g, txt = "Combine all")
  g <- add_split(g, txt = list(c("Process 1 (n=140)", "Process 2 (n=140)", "Process 3 (n=142)")))
  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "split-comb-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  # expect_snapshot_file(save_png(g), "split-comb-grviz.png")
  
})


test_that("Empty in the middle", {
  g <- add_box(
    txt = c("Cohort 1 (n=6)",
            "Cohort 2 (n=6)",
            "Cohort 3 (n=6)")
  ) |> 
    add_side_box(
      txt = c("Excluded (n=1)",
              "Excluded (n=3)",
              "")
    ) |>
    add_box(
      txt = c("Cohort 1 (n=5)",
              "Cohort 2 (n=3)",
              "")
    ) |> 
    add_box(
      txt = c("Total (n=14)")
    )  
  txt <- build_grviz(g)
  expect_snapshot_file(to_grviz(txt), "empty-middle-grviz.gv")
  
  # skip_if_not(tolower(.Platform$OS.type) == "windows")
  # expect_snapshot_file(save_png(g), "split-comb-grviz.png")
  
})

#' Demo clinical trial disposition data
#'
#' This is a demo data to demonstrate the use of the package. One row per
#' participant. Participants who are excluded should provide a reason, missing
#' otherwise. 
#'
#' @format 
#' A data frame with 300 rows and 11 columns:
#' \describe{
#'   \item{trialno}{Participants ID of the participants}
#'   \item{exclusion1, exclusion2}{Exclusion reason before and after induction}
#'   \item{induction}{Participants ID of the participants who are included in the induction
#'   phase, an extra treatment before randomisation.}
#'   \item{exclusion}{Exclusion reason before randomisation, including before and after
#'   induction}
#'   \item{arm, arm3}{Arms pariticipants randomised to.}
#'   \item{sbujid_dosed}{Participants ID of the participants who had at least one dose of 
#'   the protocol treatment.}
#'   \item{subjid_notdosed}{Reason for participants not dosed.}
#'   \item{followup}{Participants ID planned for follow-up.}
#'   \item{lost_followup}{Reason for participants not dosed.}
#'   \item{assessed}{Participants ID participants attended assessment.}
#'   \item{no_value}{Reason for participants missing final assessment.}
#'   \item{mitt}{Participants ID included in the mITT analysis.}
#' }
"dispos.data"

#' Add nodes
#'
#' Create/add vertically aligned labeled nodes or side nodes.
#'
#'
#' @param prev_box Previous node object, the created new node will be vertically
#' aligned with this node. Left this as `NULL` if this is the first node. The first
#' node will be aligned in the top center.
#' @param txt Text in the node. If the `prev_box` is a horizontally aligned multiple
#' nodes, a vector of with the same length must be provided.
#' @param just The justification for the text: left, center or right.
#' @param text_width a positive integer giving the target column for wrapping
#' lines in the output. String will not be wrapped if not defined (default).
#' The \code{\link[stringi]{stri_wrap}} function will be used if \code{stringi}
#' package installed as it is a better options for non-Latin language, otherwise
#'  \code{\link[base]{strwrap}} will be used.
#' @param ... Other parameters pass to \link{textbox},
#'
#' @seealso \code{\link{add_side_box}} \code{\link{add_split}} \code{\link{textbox}} 
#' \code{\link{add_label_box}}
#' @return A \code{consort} object.
#'
#' @export
#'
#' @example inst/examples/add-box-example.R
add_box <- function(prev_box = NULL,
                    txt,
                    just = c("center", "left", "right"),
                    text_width = NULL,
                    ...) {

  dots <- list(...)
  if("name" %in% names(dots)){
    warning("`parameter name was provided but will be ignored.")
    dots$name <- NULL
  }

  # Update arguments
  args_list <- list(box_fn = rectGrob, just = just, name = "vertbox")
  args_list <- modifyList(args_list, dots)

  just <- match.arg(just)
  
  # Wrap text
  if (!is.null(text_width)) {
    txt <- sapply(txt, function(tx) {
      text_wrap(unlist(tx), width = text_width)
    })
  }

  if (!is.null(prev_box)) {
    if (!inherits(prev_box, c("consort"))) {
      stop("prev_box must be consort object")
    }

    if(attr(prev_box, "nodes.type") == "label")
      stop("The last box added is a label, can not add box after a label!")

    prev_nodes <- attr(prev_box, "nodes.current")
    num_nodes <- attr(prev_box, "nodes.num")

    if (length(txt) != 1 & !length(prev_nodes) %in% c(1, length(txt))) {
      stop("Text with length of 1 or same node number as `prev_box`.")
    }

    if(length(prev_nodes) != length(txt))
      prev_nodes <- rep(prev_nodes, length(txt))

    # Create node
    nodes <- lapply(seq_along(txt), function(i){
      box <- do.call(textbox, c(list(text = txt[i]), args_list))
      if(length(txt) == 1){
        prev_nd <- prev_nodes
      }else{
        prev_nd <- prev_nodes[i]
      }

      if(length(txt) == length(prev_nodes) && is_empty(prev_box[[prev_nd]]$text))
        prev_nd <- prev_box[[prev_nd]]$prev_node
        

      list(
        text = txt[i],
        node_type = "vertbox",
        box = box,
        box_hw = get_coords(box),
        just = just,
        side = NULL,
        prev_node = prev_nd
      )
    })

    names(nodes) <- paste0("node", num_nodes + seq_along(txt))

    node_list <- c(prev_box, nodes)

    class(node_list) <- union("consort", class(node_list))

    structure(node_list,
      nodes.num = length(txt) + num_nodes,
      nodes.current = names(nodes),
      nodes.type = "vertbox",
      nodes.list = c(attr(prev_box, "nodes.list"), list(names(nodes)))
    )

  } else {

    nodes <- lapply(txt, function(x){
      box <- do.call(textbox, c(list(text = x), args_list))
      list(
        text = x,
        node_type = "vertbox",
        box = box,
        box_hw = get_coords(box),
        side = NULL,
        just = just,
        inv_join = NULL,
        prev_node = NULL
      )
    })

    names(nodes) <- paste0("node", seq_along(txt))

    class(nodes) <- union("consort", class(nodes))

    structure(nodes,
      nodes.num = length(txt),
      nodes.current = names(nodes),
      nodes.type = "vertbox",
      nodes.list = list(names(nodes))
    )

  }
}

#' Get the coordinates of the textbox object
#'
#' This function will get the coordinates of the textbox object.
#'
#' @param x A textbox object
#'
#' @return A list of coordinates will return:
#' \item{left}{Left (x-min) side coordinate.}
#' \item{right}{Right (x-max) side coordinate.}
#' \item{bottom}{Bottom (y-min) side coordinate.}
#' \item{top}{Top (y-max) side coordinate.}
#' \item{top.mid }{Coordinate vector of top middle, measured by grob.}
#' \item{left.mid}{Coordinate vector of left middle, measured by grob.}
#' \item{bottom.mid}{Coordinate vector of bottom middle, measured by grob.}
#' \item{right.mid}{Coordinate vector of right middle, measured by grob.}
#' \item{x}{X (center x) coordinate.}
#' \item{y}{Y (center y) coordinate.}
#' \item{width}{Width of the textbox, derived with \code{grobWidth}.}
#' \item{height}{Height of the textbox, derived with \code{grobHeight}.}
#' \item{half_width}{Half width of the box.}
#' \item{half_height}{Half height of the box.}
#'
#' @export
#'
#' @examples
#' fg <- textbox(text = "This is a test")
#' get_coords(fg)
get_coords <- function(x) {
  # if (!inherits(x, "textbox")) {
  #   stop("Object x must be textbox.")
  # }

  width <- convertWidth(grobWidth(x), "char", valueOnly = TRUE)
  height <- convertHeight(grobHeight(x), "char", valueOnly = TRUE)

  half_height <- unit(0.5 * height, "char")
  half_width <- unit(0.5 * width, "char")

  list(
    left = convertX(grobX(x, 90), "npc") - half_width,
    right = convertX(grobX(x, 90), "npc") + half_width,
    bottom = convertY(grobY(x, 0), "npc") - half_height,
    top = convertY(grobY(x, 0), "npc") + half_height,
    top.mid = unit.c(grobX(x, 90), grobY(x, 90)),
    left.mid = unit.c(grobX(x, 180), grobY(x, 180)),
    bottom.mid = unit.c(grobX(x, 90), grobY(x, 270)),
    right.mid = unit.c(grobX(x, 0), grobY(x, 0)),
    x = convertX(grobX(x, 90), "npc"),
    y = convertY(grobY(x, 0), "npc"),
    width = width,
    height = height,
    half_width = half_width,
    half_height = half_height
  )
}

#' Add a splitting box
#'
#' This function will create a horizontally aligned nodes. The horizontal coordinate
#' will be automatically calculated if the coordinates not provided.
#'
#' @param prev_box Previous node that the newly created split box will be aligned.
#' @inheritParams add_box
#'
#'
#' @seealso \code{\link{add_box}}, \code{\link{add_side_box}} \code{\link{textbox}} 
#'
#' @return A \code{consort.list} object.
#'
#' @export
#'
#' @example inst/examples/add-box-example.R
add_split <- function(prev_box,
                      txt,
                      just = c("center", "left", "right"),
                      text_width = NULL,
                      ...) {

  dots <- list(...)
  if("name" %in% names(dots)){
    warning("`parameter name was provided but will be ignored.")
    dots$name <- NULL
  }

  just <- match.arg(just)

  # Wrap text
  if (!is.null(text_width)) {
    txt <- sapply(txt, function(tx) {
      text_wrap(unlist(tx), width = text_width)
    })
  }

  if (!inherits(prev_box, c("consort"))) {
    stop("prev_box must be consort object")
  }

  prev_nodes <- attr(prev_box, "nodes.current")
  num_nodes <- attr(prev_box, "nodes.num")

  if (any(sapply(prev_box, "[[", "node_type") %in% "splitbox") & !is.list(txt) & length(prev_nodes) > 1) {
    stop("txt should be a list for a nested splits.")
  }

  if(is.list(txt) & length(txt) != length(prev_nodes))
    stop("The length of the text should be the same as the number of previous nodes.")

  if(attr(prev_box, "nodes.type") == "label")
      stop("The last box added is a label, can not add box after a label!")

  if(attr(prev_box, "nodes.type") == "sidebox")
    stop("The last box added is a sidebox, can not split box after a sidebox! Add another box before split.")
  
  # if (length(txt) == 1) {
  #   stop("The length of txt should be larger than 1, please use add_box instead.")
  # }
  # Update arguments
  args_list <- list(just = just, box_fn = rectGrob, name = "splitbox")
  args_list <- modifyList(args_list, dots)

  if(!is.list(txt)){
    nodes <- lapply(seq_along(txt), function(i){
    box <- do.call(textbox, c(list(text = txt[i]), args_list))
      list(
        text = txt[i],
        node_type = "splitbox",
        box = box,
        box_hw = get_coords(box),
        side = NULL,
        just = just,
        prev_node = prev_nodes
      )
    })
  }else{
    # For nested splits
    nodes <- lapply(seq_along(txt), function(i){
      lapply(txt[[i]], function(x){
        box <- do.call(textbox, c(list(text = x), args_list))
        list(
          text = x,
          node_type = "splitbox",
          box = box,
          box_hw = get_coords(box),
          side = NULL,
          just = just,
          prev_node = prev_nodes[i]
        )
      })
    })
    nodes <- unlist(nodes, recursive = FALSE)
  }

  names(nodes) <- paste0("node", num_nodes + seq_along(unlist(txt)))

  node_list <- c(prev_box, nodes)

  class(node_list) <- union("consort", class(node_list))

  structure(node_list,
    nodes.num = length(unlist(txt)) + num_nodes,
    nodes.current = names(nodes),
    nodes.type = "splitbox",
    nodes.list = c(attr(prev_box, "nodes.list"), list(names(nodes)))
  )
}

# Markup parsing and conversion utilities
#
# Supported markup syntax:
#   **bold**         -> bold text
#   *italic*         -> italic text
#   ^{superscript}   -> superscript text
#   _{subscript}     -> subscript text
#   __underline__    -> underlined text

# Check if markup parsing is enabled and text contains markup patterns
#' @keywords internal
has_markup <- function(text) {
  if (!isTRUE(consort_opt("parse_markup")))
    return(FALSE)
  if (is.null(text) || !is.character(text) || length(text) != 1 ||
      is.na(text) || !nzchar(text))
    return(FALSE)
  grepl("\\*\\*.+?\\*\\*|__.+?__|\\*.+?\\*|\\^\\{.+?\\}|_\\{.+?\\}",
        text, perl = TRUE)
}

# Parse markup text into a list of segments
# Each segment is list(text = "...", style = "plain"|"bold"|"italic"|...)
#' @keywords internal
parse_markup <- function(text) {
  if (is.null(text) || !nzchar(text)) {
    return(list(list(text = "", style = "plain")))
  }

  # Combined pattern — order matters: ** before *, __ before _{}
  pattern <- "\\*\\*(.+?)\\*\\*|__(.+?)__|\\*(.+?)\\*|\\^\\{(.+?)\\}|_\\{(.+?)\\}"

  matches <- gregexpr(pattern, text, perl = TRUE)[[1]]

  if (matches[1] == -1) {
    return(list(list(text = text, style = "plain")))
  }

  match_starts  <- as.integer(matches)
  match_lengths <- attr(matches, "match.length")

  segments <- list()
  pos <- 1L

  for (i in seq_along(match_starts)) {
    # Plain text before this match
    if (match_starts[i] > pos) {
      segments <- c(segments, list(list(
        text  = substr(text, pos, match_starts[i] - 1L),
        style = "plain"
      )))
    }

    matched <- substr(text, match_starts[i],
                      match_starts[i] + match_lengths[i] - 1L)

    # Determine which alternative matched
    if (grepl("^\\*\\*(.+?)\\*\\*$", matched, perl = TRUE)) {
      inner <- sub("^\\*\\*(.+?)\\*\\*$", "\\1", matched, perl = TRUE)
      segments <- c(segments, list(list(text = inner, style = "bold")))

    } else if (grepl("^__(.+?)__$", matched, perl = TRUE)) {
      inner <- sub("^__(.+?)__$", "\\1", matched, perl = TRUE)
      segments <- c(segments, list(list(text = inner, style = "underline")))

    } else if (grepl("^\\*(.+?)\\*$", matched, perl = TRUE)) {
      inner <- sub("^\\*(.+?)\\*$", "\\1", matched, perl = TRUE)
      segments <- c(segments, list(list(text = inner, style = "italic")))

    } else if (grepl("^\\^\\{(.+?)\\}$", matched, perl = TRUE)) {
      inner <- sub("^\\^\\{(.+?)\\}$", "\\1", matched, perl = TRUE)
      segments <- c(segments, list(list(text = inner, style = "superscript")))

    } else if (grepl("^_\\{(.+?)\\}$", matched, perl = TRUE)) {
      inner <- sub("^_\\{(.+?)\\}$", "\\1", matched, perl = TRUE)
      segments <- c(segments, list(list(text = inner, style = "subscript")))
    }

    pos <- match_starts[i] + match_lengths[i]
  }

  # Remaining text after last match
  if (pos <= nchar(text)) {
    segments <- c(segments, list(list(
      text  = substr(text, pos, nchar(text)),
      style = "plain"
    )))
  }

  segments
}

# Split parsed segments into lines at \n boundaries in plain segments
#' @keywords internal
split_segments_by_newline <- function(segments) {
  lines        <- list()
  current_line <- list()

  for (seg in segments) {
    if (seg$style == "plain" && grepl("\n", seg$text, fixed = TRUE)) {
      # Sentinel preserves trailing empty strings dropped by strsplit
      parts <- strsplit(paste0(seg$text, "\a"), "\n", fixed = TRUE)[[1]]
      parts[length(parts)] <- sub("\a$", "", parts[length(parts)])

      for (j in seq_along(parts)) {
        if (j > 1L) {
          # Close current line
          if (length(current_line) == 0L)
            current_line <- list(list(text = "", style = "plain"))
          lines[[length(lines) + 1L]] <- current_line
          current_line <- list()
        }
        if (nzchar(parts[j])) {
          current_line[[length(current_line) + 1L]] <-
            list(text = parts[j], style = "plain")
        }
      }
    } else {
      current_line[[length(current_line) + 1L]] <- seg
    }
  }

  # Final line
  if (length(current_line) == 0L)
    current_line <- list(list(text = "", style = "plain"))
  lines[[length(lines) + 1L]] <- current_line

  lines
}

# Build a grid gpar for a given markup style, based on a base gpar
#' @keywords internal
segment_gpar <- function(style, base_gp) {
  if (is.null(base_gp)) base_gp <- gpar()
  gp <- base_gp
  if (style == "bold") {
    gp$fontface <- "bold"
  } else if (style == "italic") {
    gp$fontface <- "italic"
  } else if (style %in% c("superscript", "subscript")) {
    cex    <- if (is.null(gp$cex)) 1 else gp$cex
    gp$cex <- cex * 0.7
  }
  gp
}

# Convert markup text to HTML for Graphviz HTML-like labels
#' @keywords internal
markup_to_html <- function(text) {
  if (is.null(text) || !nzchar(text)) return(text)

  # Escape HTML special characters first
  text <- gsub("&", "&amp;",  text, fixed = TRUE)
  text <- gsub("<", "&lt;",   text, fixed = TRUE)
  text <- gsub(">", "&gt;",   text, fixed = TRUE)

  # Replace markup with HTML tags (order: ** before *, __ before _{})
  text <- gsub("\\*\\*(.+?)\\*\\*", "<b>\\1</b>",     text, perl = TRUE)
  text <- gsub("__(.+?)__",         "<u>\\1</u>",      text, perl = TRUE)
  text <- gsub("\\*(.+?)\\*",       "<i>\\1</i>",      text, perl = TRUE)
  text <- gsub("\\^\\{(.+?)\\}",    "<sup>\\1</sup>",  text, perl = TRUE)
  text <- gsub("_\\{(.+?)\\}",      "<sub>\\1</sub>",  text, perl = TRUE)

  # Duplicate the space after bold/italic/underline closing tags: one copy

  # inside the tag (measured with the styled font metrics) and the original
  # outside.  Graphviz tends to underestimate node width when mixing styled
  # and plain text; the extra space compensates for the difference.
  text <- gsub("</b> ", " </b> ", text, fixed = TRUE)
  text <- gsub("</i> ", " </i> ", text, fixed = TRUE)
  text <- gsub("</u> ", " </u> ", text, fixed = TRUE)

  # Newlines to <br/>
  text <- gsub("\n", "<br/>", text, fixed = TRUE)

  text
}

# --- Parser tests -----------------------------------------------------------

test_that("has_markup returns FALSE when parse_markup is disabled", {
  init_consort_defaults()
  expect_false(has_markup("**bold**"))
  expect_false(has_markup("*italic*"))
})

test_that("has_markup detects markup patterns when enabled", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  expect_true(has_markup("**bold**"))
  expect_true(has_markup("*italic*"))
  expect_true(has_markup("^{super}"))
  expect_true(has_markup("_{sub}"))
  expect_true(has_markup("__underline__"))
  expect_true(has_markup("text **with** markup"))

  expect_false(has_markup("plain text"))
  expect_false(has_markup(""))
  expect_false(has_markup(NULL))
  expect_false(has_markup(NA_character_))
})

test_that("parse_markup handles plain text", {
  segs <- parse_markup("hello world")
  expect_length(segs, 1)
  expect_equal(segs[[1]]$text, "hello world")
  expect_equal(segs[[1]]$style, "plain")
})

test_that("parse_markup handles bold", {
  segs <- parse_markup("**bold**")
  expect_length(segs, 1)
  expect_equal(segs[[1]]$text, "bold")
  expect_equal(segs[[1]]$style, "bold")
})

test_that("parse_markup handles italic", {
  segs <- parse_markup("*italic*")
  expect_length(segs, 1)
  expect_equal(segs[[1]]$text, "italic")
  expect_equal(segs[[1]]$style, "italic")
})

test_that("parse_markup handles superscript and subscript", {
  segs <- parse_markup("x^{2}")
  expect_length(segs, 2)
  expect_equal(segs[[1]]$text, "x")
  expect_equal(segs[[2]]$text, "2")
  expect_equal(segs[[2]]$style, "superscript")

  segs2 <- parse_markup("H_{2}O")
  expect_length(segs2, 3)
  expect_equal(segs2[[2]]$text, "2")
  expect_equal(segs2[[2]]$style, "subscript")
})

test_that("parse_markup handles underline", {
  segs <- parse_markup("__underline__")
  expect_length(segs, 1)
  expect_equal(segs[[1]]$text, "underline")
  expect_equal(segs[[1]]$style, "underline")
})

test_that("parse_markup handles mixed markup", {
  segs <- parse_markup("**Enrolled** (n=300)\n*p* < 0.05^{1}")
  styles <- sapply(segs, "[[", "style")
  texts  <- sapply(segs, "[[", "text")

  expect_equal(styles, c("bold", "plain", "italic", "plain", "superscript"))
  expect_equal(texts[[1]], "Enrolled")
  expect_equal(texts[[3]], "p")
  expect_equal(texts[[5]], "1")
})

test_that("parse_markup returns plain for empty/NULL", {
  segs <- parse_markup("")
  expect_equal(segs[[1]]$text, "")
  expect_equal(segs[[1]]$style, "plain")

  segs2 <- parse_markup(NULL)
  expect_equal(segs2[[1]]$text, "")
})

# --- Newline splitting -------------------------------------------------------

test_that("split_segments_by_newline splits plain text at newlines", {
  segs <- list(list(text = "line1\nline2", style = "plain"))
  lines <- split_segments_by_newline(segs)
  expect_length(lines, 2)
  expect_equal(lines[[1]][[1]]$text, "line1")
  expect_equal(lines[[2]][[1]]$text, "line2")
})

test_that("split_segments_by_newline keeps non-plain segments intact", {
  segs <- list(
    list(text = "before\n", style = "plain"),
    list(text = "bold", style = "bold"),
    list(text = "\nafter", style = "plain")
  )
  lines <- split_segments_by_newline(segs)
  expect_length(lines, 3)
  expect_equal(lines[[2]][[1]]$text, "bold")
  expect_equal(lines[[2]][[1]]$style, "bold")
})

# --- HTML conversion ---------------------------------------------------------

test_that("markup_to_html converts tags correctly", {
  expect_equal(markup_to_html("**bold**"), "<b>bold</b>")
  expect_equal(markup_to_html("*italic*"), "<i>italic</i>")
  expect_equal(markup_to_html("^{sup}"), "<sup>sup</sup>")
  expect_equal(markup_to_html("_{sub}"), "<sub>sub</sub>")
  expect_equal(markup_to_html("__underline__"), "<u>underline</u>")
})

test_that("markup_to_html escapes HTML special characters", {
  expect_equal(markup_to_html("a < b & c"), "a &lt; b &amp; c")
  expect_equal(markup_to_html("**a < b**"), "<b>a &lt; b</b>")
})

test_that("markup_to_html converts newlines to <br/>", {
  expect_equal(markup_to_html("a\nb"), "a<br/>b")
})

test_that("markup_to_html duplicates space inside bold/italic/underline tags", {
  # Space is duplicated: one inside the tag + one outside
  expect_equal(markup_to_html("**bold** text"), "<b>bold </b> text")
  expect_equal(markup_to_html("*italic* text"), "<i>italic </i> text")
  expect_equal(markup_to_html("__underline__ text"), "<u>underline </u> text")
  # No space after tag — nothing to duplicate
  expect_equal(markup_to_html("**bold**\ntext"), "<b>bold</b><br/>text")
})

# --- segment_gpar ------------------------------------------------------------

test_that("segment_gpar sets fontface for bold/italic", {
  base <- gpar(fontsize = 12)
  expect_equal(segment_gpar("bold", base)$fontface, "bold")
  expect_equal(segment_gpar("italic", base)$fontface, "italic")
})

test_that("segment_gpar scales cex for super/subscript", {
  base <- gpar(cex = 1)
  expect_equal(segment_gpar("superscript", base)$cex, 0.7)
  expect_equal(segment_gpar("subscript", base)$cex, 0.7)
})

test_that("segment_gpar leaves plain text unchanged", {
  base <- gpar(fontsize = 12)
  gp <- segment_gpar("plain", base)
  expect_null(gp$fontface)
  expect_equal(gp$fontsize, 12)
})

# --- Grid rendering (smoke tests) -------------------------------------------

test_that("textbox with markup creates valid grob when enabled", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  bx <- textbox(text = "**Enrolled** (n=300)")
  expect_s3_class(bx, "textbox")

  bx2 <- textbox(text = "H_{2}O is *water*")
  expect_s3_class(bx2, "textbox")
})

test_that("textbox with markup renders without error", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  skip_on_ci()
  png(tf <- tempfile(fileext = ".png"), width = 400, height = 200)
  on.exit({ dev.off(); unlink(tf) }, add = TRUE)

  grid::grid.newpage()
  expect_no_error(grid::grid.draw(textbox(text = "**Bold** and *italic*")))
  expect_no_error(grid::grid.draw(
    textbox(text = "x^{2} + H_{2}O\n__underlined__", y = 0.3)
  ))
})

test_that("textbox ignores markup when parse_markup is FALSE", {
  init_consort_defaults()  # parse_markup = FALSE
  bx <- textbox(text = "**bold** text")
  # label is stored as-is (not parsed)
  expect_equal(bx$label, "**bold** text")
})

# --- grViz integration -------------------------------------------------------

test_that("mk_text_align uses HTML label for markup text when enabled", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  result <- mk_text_align("**bold** text", just = "center")
  expect_true(grepl("label = <", result, fixed = TRUE))
  # Space duplicated: one inside + one outside the bold tag
  expect_true(grepl("<b>bold </b> ", result, fixed = TRUE))
})

test_that("mk_text_align uses plain label without markup", {
  init_consort_defaults()
  result <- mk_text_align("plain text", just = "center")
  expect_true(grepl('label = "', result, fixed = TRUE))
})

test_that("mk_text_align uses plain label when parse_markup is FALSE", {
  init_consort_defaults()  # parse_markup = FALSE
  result <- mk_text_align("**bold** text", just = "center")
  expect_true(grepl('label = "', result, fixed = TRUE))
  expect_false(grepl("<b>", result, fixed = TRUE))
})

test_that("mk_text_align HTML label handles alignment", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  result_left <- mk_text_align("**bold**\nmore", just = "left")
  expect_true(grepl('align="left"', result_left, fixed = TRUE))

  result_right <- mk_text_align("**bold**\nmore", just = "right")
  expect_true(grepl('align="right"', result_right, fixed = TRUE))
})

# --- Full pipeline (build_grviz) --------------------------------------------

test_that("build_grviz handles markup in nodes when enabled", {
  old <- set_consort_defaults(parse_markup = TRUE)
  on.exit(set_consort_defaults(parse_markup = old$parse_markup), add = TRUE)

  g <- add_box(txt = "**Enrolled** (n=300)")
  g <- add_side_box(g, txt = "Excluded^{1} (n=15)")
  g <- add_box(g, txt = "*Randomised*")

  txt <- build_grviz(g)
  # Space duplicated: one inside + one outside the bold tag
  expect_true(grepl("<b>Enrolled </b>", txt, fixed = TRUE))
  expect_true(grepl("<sup>1</sup>", txt, fixed = TRUE))
  # No space after *Randomised* (end of text), so tag is unchanged
  expect_true(grepl("<i>Randomised</i>", txt, fixed = TRUE))
})

# --- set_consort_defaults validation -----------------------------------------

test_that("set_consort_defaults validates parse_markup", {
  expect_error(set_consort_defaults(parse_markup = "yes"),
               "parse_markup")
  expect_error(set_consort_defaults(parse_markup = NA),
               "parse_markup")
  expect_error(set_consort_defaults(parse_markup = c(TRUE, FALSE)),
               "parse_markup")
})

init_consort_defaults()

#' Connect grob box with arrow.
#'
#' This function is used to create an arrow line to connect two boxes. User should
#' provide the starting and ending side of the arrow.
#'
#' @param start Starting point of the arrow.
#' @param end Ending point of the arrow.
#' @param connect The connection of the box. It should be the combination of the
#' position. The \code{t} refers to "top", \code{l} for "left", \code{b} for
#' "bottom" and \code{r} for "right". The first letter is the starting point of
#' the start box, the second is the ending point of the end box. For example, if
#' one wants to connect the left side of the start box with right side of the
#' left side of the end box, the value should be \code{"lr"}. All the connection
#' will be started in the middle point.
#' @param type Should be one the \code{"s"} (strait line), or \code{"p"} (polyline).
#' @param name A character identifier of the line grob, passed to \code{\link[grid]{linesGrob}}.
#' @param arrow_gp Graphical parameters \code{\link[grid]{gpar}} object for the 
#' connector line. Defaults to \code{grid::gpar(col = "black", lwd = 1)}.
#' @param arrow_length Length of the arrowhead as a \code{\link[grid]{unit}} object. 
#'   Defaults to \code{unit(0.1, "inches")}.
#' @param arrow_type Arrow type, either \code{"closed"} or \code{"open"}. Defaults to 
#'   \code{"closed"}.
#'
#' @return A lines grob with arrow.
#' @export
#' @rdname connect_box
#'
#' @examples
#' fg1 <- textbox(text = "This is a test")
#' fg2 <- textbox(text = "This is an other test", 0.7, 0.2)
#' grid::grid.draw(fg1)
#' grid::grid.draw(fg2)
#' connect_box(fg1, fg2, connect = "bl", type = "p")
#'
#' # Customize line width and color
#' connect_box(fg1, fg2, connect = "bl", type = "p", arrow_gp = grid::gpar(lwd = 2, col = "red"))
#'
#' # Or set globally via set_consort_defaults
#' set_consort_defaults(arrow_gp = grid::gpar(lwd = 2, col = "blue"))
connect_box <- function(start, end,
                        connect,
                        type = c("s", "p"),
                        name = NULL,
                        arrow_gp = consort_opt("arrow_gp"),
                        arrow_length = consort_opt("arrow_length"),
                        arrow_type = consort_opt("arrow_type")) {
  
  type <- match.arg(type)
  
  if (nchar(connect) != 2) {
    stop("Connect must be of two characters")
  }
  
  # Validate arrow_type
  arrow_type <- match.arg(arrow_type, choices = c("closed", "open"))
  
  # Convert numeric to unit — default unit is inches
  if (!is.unit(arrow_length)) {
    arrow_length <- unit(arrow_length, "in")
  }
  
  start_s <- substr(connect, 1, 1)
  end_s <- substr(connect, 2, 2)
  
  if (!grepl(start_s, "trbl", fixed = TRUE) | !grepl(end_s, "trbl", fixed = TRUE)) {
    stop("Connect must be a combination of \'t', \'r', \'b', \'l'.")
  }
  
  start_coords <- get_coords(start)
  end_coords <- get_coords(end)
  
  # X and Y coordinates for starting box
  x_s <- switch(start_s,
                "t" = start_coords$x,
                "r" = start_coords$right,
                "b" = start_coords$x,
                "l" = start_coords$left
  )
  
  y_s <- switch(start_s,
                "t" = start_coords$top,
                "r" = start_coords$y,
                "b" = start_coords$bottom,
                "l" = start_coords$y
  )
  
  # X and Y coordinates for ending box
  x_e <- switch(end_s,
                "t" = end_coords$x,
                "r" = end_coords$right,
                "b" = end_coords$x,
                "l" = end_coords$left
  )
  
  y_e <- switch(end_s,
                "t" = end_coords$top,
                "r" = end_coords$y,
                "b" = end_coords$bottom,
                "l" = end_coords$y
  )
  
  get_dist <- function(a, b) {
    res <- a - b
    # Consider the arrow head — use the configurable arrow_length
    if (as.numeric(convertUnit(res, "mm", valueOnly = TRUE)) < 0) {
      0.5 * (res + arrow_length)
    } else {
      0.5 * (res - arrow_length)
    }
  }
  
  if (type == "s") {
    line_coords <- list(
      x = unit.c(x_s, x_e),
      y = unit.c(y_s, y_e)
    )
  } else {
    if (start_s %in% c("t", "b")) {
      if (end_s %in% c("t", "b")) {
        y_dist <- get_dist(y_s, y_e)
        x_mid <- unit.c(x_s, x_e)
        y_mid <- unit.c(y_s - y_dist, y_s - y_dist)
      } else {
        x_mid <- x_s
        y_mid <- y_e
      }
    }
    
    if (start_s %in% c("r", "l")) {
      if (end_s %in% c("r", "l")) {
        x_dist <- get_dist(x_s, x_e)
        x_mid <- unit.c(x_s - x_dist, x_s - x_dist)
        y_mid <- unit.c(y_s, y_e)
      } else {
        x_mid <- x_e
        y_mid <- y_s
      }
    }
    
    line_coords <- list(
      x = unit.c(x_s, x_mid, x_e),
      y = unit.c(y_s, y_mid, y_e)
    )
  }

  arrow_gp$fill <- arrow_gp$col
  
  linesGrob(
    x = line_coords$x,
    y = line_coords$y,
    gp = arrow_gp,
    arrow = arrow(
      length = arrow_length,
      ends = "last", type = arrow_type
    ),
    name = name
  )
}

---
title: "Self-generating CONSORT diagram"
output: 
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteIndexEntry{Self-generating CONSORT diagram}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(consort)
data(dispos.data)
```

The goal of this package is to make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved. Below is the illustration of the CONSORT diagram creating process for the two different methods.

# Prepare test data
In a clinical research, we have a participants disposition data. One column is the participants' ID, and the following columns indicating the status of the participants at different stage of the study. One can easily derive the number of participants at different stage by counting the number of participants on-study excluding the participants who are excluded. 

# Description

This function developed to populate consort diagram automatically. But to do so, a population disposition data should be prepared. The following data is prepared for demonstration provided in the package. The data can be loaded with `data(dispos.data)`, the variables in this table is explained below: 

-   `trialno`: Participants ID of the participants
-   `exclusion1, exclusion2`: Exclusion reason before and after induction
-   `induction`: Participants ID of the participants who are included in the induction phase, an extra treatment before randomisation.
-   `exclusion`: Exclusion reason before randomisation, including before and after induction
-   `arm, arm3`: Arms pariticipants randomised to.
-   `sbujid_dosed`: Participants ID of the participants who had at least one dose of the protocol treatment.
-   `subjid_notdosed`: Reason for participants not dosed.
-   `followup`: Participants ID planned for follow-up.
-   `lost_followup`: Reason for participants not dosed.
-   `assessed`: Participants ID participants attended assessment.
-   `no_value`: Reason for participants missing final assessment.
-   `mitt`: Participants ID included in the mITT analysis.

```{r data-head, echo=FALSE}
head(dispos.data)
```

# Usage

Basic logic:

1.  The vertical node are the number of patients in the current node, no dropout reasons of inclusion reasons should be provided.
2.  Side box is only used for the drop outs. It include information about the number of patients excluded and reasons. 
3.  Any participants with exclusion reasons will not be included in the next vertical node box after excluded. So the subject id can be used multiple times to indicate how many patients left in the current node. 
4.  The node labels, for example visit number or phase, can only horizontally align to a vertical main nodes, not an exclusion box.
5.  If more than 2 treatment allocation is present, all the exclusion box after the allocation will be aligned to the right

**At each stage, number of non-missing values will be counted**. Set `kickoff_sidebox=TRUE` to remove the observations counted in the side box in the remaining nodes. 

# Self-generating function

To generate consort diagram with data.frame, one should prepare a disposition `data.frame`.

```{r example, eval=FALSE}
consort_plot(data,
             orders,
             side_box,
             allocation = NULL,
             labels = NULL,
             cex = 0.8,
             text_width = NULL,
             widths = c(0.1, 0.9))
```

-   `data`: Dataset prepared above
-   `orders`: A named vector or a list, names as the variable and values as labels in the box. The order of the diagram will be based on this. Variables listed here, but not included in other parameters will calculate the number of non-missing values. A list can be provided with multiple variables in each list element and these variables will be stack into a single node. The side box should only have 1 variable. 
-   `side_box`: Variable vector, appeared as side box in the diagram. The next box will be count the number of non-missing values, so users need to make sure this is correct. The pariticpants id variable can be used multiple times, since only the number of non-missing is calculated for the vertical box.
-   `allocation`: Name of the grouping/treatment variable (optional), the diagram will split into branches on this variables.
-   `labels`: Named vector, names is the location of the vertical node excluding the side box. The position location should plus 1 after the allocation variables if the allocation is defined.
-   `kickoff_sidebox`: kickoff of the observation from the data with following a side box.
-   `cex`: Multiplier applied to font size, Default is 0.6
-   `text_width`: A positive integer giving the target column for wrapping lines in the output.

# Manual 
Functions are mainly in three categories, main box, side box and label box. Others include building function. These are the functions used by the self generating function. These box functions require the previous node and text label.
-   `add_box`: add main box, no previous nodes should be provided if this is the first node.
-   `add_side_box`: add exclusion box.
-   `add_split`: add allocation box, all nodes will be split into groups. The label text for this node and following nodes should be a vector with a length larger than 1. 
-   `add_label_box`: add visiting or phasing label given a reference node.
-   `build_grid`: you don't need this unless you want the `grob` object.
-   `build_grviz`: you don't need this unless you want the `Graphviz` code.

# Plotting engine
The package supports two plotting engines: `grid` (`grid` package) and `Graphviz`. The default was `grid`, `plot(g)` (assuming `g` is the consort plot object) will use the `grid` to draw the plot. The coordinates are calculated internally, one can simply use `build_grviz(g)` to get a built `grob` object. This means you can do use `patchwork` package to combine different plots. You can use the example below to add title and footnotes. 

```{r patchwork, eval=FALSE}
library(patchwork)

wrap_elements(build_grid(g)) + plot_annotation(
  title = 'Consort diagram',
  subtitle = 'Flow chart of the XX study',
  caption = 'Disclaimer: None of these plots are insightful'
)
```

Although many efforts have been made to draw the plot as easy as possible, you may still find some plots are difficult to draw. You may not be happy about the final output or want the plot in a `svg` format to be used in the websites. You can simply `plot(g, grViz = TRUE)` to draw the plot with `Graphviz`. You need to install [DiagrammeR](https://CRAN.R-project.org/package=DiagrammeR) package to do so. If you want to do some twicking to the plot, you can use `build_grviz(g)` to extract the `dot` file and edit. You can use `Rstudio` to render the plot or other tools you prefer. Use the example below to dump the `dot` file.
```{r dump-dotfile, eval = FALSE}
cat(build_grviz(g), file = "consort.gv")
```

# Working example (self generation)
## Single arm

```{r single-arms, message=FALSE, fig.width  = 6, fig.height = 6}
out <- consort_plot(data = dispos.data,
                    orders = c(trialno = "Population",
                               exclusion = "Excluded",
                               trialno = "Allocated",
                               subjid_notdosed = "Not dosed",
                               followup = "Followup",
                               lost_followup = "Not evaluable for the final analysis",
                               mitt = "Final Analysis"),
                    side_box = c("exclusion", "subjid_notdosed", "lost_followup"),
                    cex = 0.9)
plot(out)
```

## Multiple phase and multiple arms

```{r multiple-phase, fig.width  = 9, fig.height = 6}
g <- consort_plot(data = dispos.data,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm3     = "Randomized patient",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = "arm3",
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "5" = "End of study"),
                  cex = 0.7)
plot(g)
```

## Multiple stratification with node stack
In some cases, two level randomisation/stratification is desired. A typical case is the factorial design. Simply provide two allocation variables, but more than two splits are not supported. 

Here is a simple example on how to do it.
```{r multiple-split, fig.width  = 12, fig.height = 8, out.width = "95%"}
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
g <- consort_plot(data = df,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm = "Randomized patient",
                             arm3     = "",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "6" = "End of study"),
                  cex = 0.7)
plot(g)
```

In a more general cases, one may want to keep the count inside the vertical box instead of excluding. In this cases, one can provide multiple variables in a list. The first variable will simply be counted, the remaining variables will be itemised. 
```{r multiple-split-stack, fig.width  = 13, fig.height = 10, out.width = "95%"}
# We will exclude one arm to avoid too many arms.
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                # The following two variables will be stacked together
                                c(arm3     = "", # Should not provide a value to show the actual arm
                                  subjid_notdosed="Participants not treated"),
                                # The following two variables will be stacked together
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"), 
                  allocation = c("arm", "arm3"), # Two level randomisation
                  kickoff_sidebox = FALSE,
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)

plot(p)
```

# Working example (human effort)
The previous is to easily generate a consort diagram based on a disposition data, here we show how to create a consort diagram by providing the label text manually. 

## Provide text
```{r providetext1, fig.width  = 7, fig.height = 4}
library(grid)
# Might want to change some settings
options(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)
```

## Missing nodes and multiple split
There might be some cases that the nodes will be missing, this can be handled as well. You can also have multiple splits, but multiple splits for the `grid` hasn't been implemented yet. You can use `plot(g, grViz = TRUE)` to produce the consort plot.

```{r providetext2}
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
# Length needs to be the same as previous one, use a list here.
g <- add_split(g, txt = list(c("Process 1 (n=140)", "Process 2 (n=140)",
                               "Process 3 (n=142)")))

plot(g, grViz = TRUE)

```

## Two-level stratification
Some studies may first stratify patients then randomise, a factorial design for example. This will require two-level stratification. One can simply provide **a list** of the same length of the previous node after a split. More than two splits are not supported. 
```{r manual-multisplit, fig.width  = 13, fig.height = 6, out.width = "80%"}
g <- add_box(txt = "Patient consented (n=200)")
g <- add_side_box(g, txt = "Excluded (n=10):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=7)")
g <- add_box(g, txt = "Randomised (n=190)")
g <- add_split(g, txt = c("Randomised study (n=144)", "Preference study (n=146)"))

# Provide a list here
g <- add_split(g, txt = list(c("Allocated to surgery (n=70)", 
                                "Allocated to medicine (n=74)"),
                             c("Allocated to surgery (n=47)", 
                              "Allocated to medicine (n=48)",
                               "Placebo (n=51)")))

g <- add_side_box(g, txt = c("Excluded (n=3):\n\u2022 Withdrawn before surgery (n=2)\n\u2022 Declined (n=1)", 
                             "", 
                             "Excluded (n=1):\n\u2022 Withdrawn before surgery (n=1)",
                             "Excluded (n=3):\n\u2022 Withdrawn before surgery (n=1)\n\u2022 Declined (n=1)",
                             "Excluded (n=10):6\n\u2022 Declined (n=10)"),
                  side = rep("right", 5))
g <- add_box(g, txt = c("Analysed (n=67)", "Analysed (n=74)",
                        "Analysed (n=46)", "Analysed (n=45)", "Analysed (n=41)"))

plot(g)

```

## Using disposition table
```{r disposition-data, fig.width  = 7, fig.height = 6.5}
options(txt_gp = gpar(cex = 0.8))

dispos.data$arm <- factor(dispos.data$arm)

txt <- gen_text(dispos.data$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(dispos.data$exclusion, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

g <- add_box(g, txt = gen_text(dispos.data$arm, label = "Patients randomised")) 

txt <- gen_text(dispos.data$arm)
g <- add_split(g, txt = txt)

# Exclude subjects
dispos.data <- dispos.data[is.na(dispos.data$exclusion), ]

txt <- gen_text(split(dispos.data[,c("subjid_notdosed")], dispos.data$arm),
                label = "Not dosed", bullet = TRUE)
g <- add_box(g, txt = txt, just = "left")

txt <- gen_text(split(dispos.data$mitt, dispos.data$arm),
                label = "Primary mITT analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "5" = "Final analysis"))

plot(g)
```

## For Shiny and HTML
Although all the efforts has been made to precisely calculate the coordinates of the nodes, it is not very accurate due to limit of my own knowledge. But you can utilize the [DiagrammeR](https://CRAN.R-project.org/package=DiagrammeR) to produce plots for Shiny and HTML by setting `grViz = TRUE` in `plot`. You can get `Graphviz` code with `build_grviz` of the plot. In addition, use [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) save plot in various formats.
```{r save-plot}
plot(g, grViz = TRUE)
```

## Use with Quarto 
Quarto has native [support](https://quarto.org/docs/authoring/diagrams.html) for embedding Graphviz diagrams. You can plot the flowchart without any printing method. 

````{verbatim, lang = "markdown"}
```{r}
cat(build_grviz(g), file = "consort.gv")
```

```{dot}
//| label: consort-diagram
//| fig-cap: "CONSORT diagram of study XXX"
//| file: consort.gv
```

````

# Saving plot
In order to export the plot to fit a page properly, you need to provide the width and height of the output plot. You might need to try different width and height to get a satisfying plot.You can use R basic device destination for the output. Below is how to save a plot in `png` format:
```{r eval=FALSE}
# save plots
png("consort_diagram.png", width = 29, 
    height = 21, res = 300, units = "cm", type = "cairo") 
plot(g)
dev.off() 

```

Or you can use `ggplot2::ggsave` function to save the plot object:
```{r eval=FALSE}
ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))
```

Or save with [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) to `png` or `pdf`
```{r eval=FALSE}
plot(g, grViz = TRUE) |> 
    DiagrammeRsvg::export_svg() |> 
    charToRaw() |> 
    rsvg::rsvg_pdf("svg_graph.pdf")
```

---
title: "Self-generating CONSORT diagram"
output: 
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteIndexEntry{Self-generating CONSORT diagram}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  dpi = 300,
  collapse = TRUE,
  comment = "#>",
  message = FALSE
)
```

```{r setup}
library(consort)
data(dispos.data)
```

The goal of this package is to make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved. Below is the illustration of the CONSORT diagram creating process for the two different methods.

# Prepare test data
In a clinical research, we have a participants disposition data. One column is the participants' ID, and the following columns indicating the status of the participants at different stage of the study. One can easily derive the number of participants at different stage by counting the number of participants on-study excluding the participants who are excluded. 

# Description

This function developed to populate consort diagram automatically. But to do so, a population disposition data should be prepared. The following data is prepared for demonstration provided in the package. The data can be loaded with `data(dispos.data)`, the variables in this table is explained below: 

-   `trialno`: Participants ID of the participants
-   `exclusion1, exclusion2`: Exclusion reason before and after induction
-   `induction`: Participants ID of the participants who are included in the induction phase, an extra treatment before randomisation.
-   `exclusion`: Exclusion reason before randomisation, including before and after induction
-   `arm, arm3`: Arms pariticipants randomised to.
-   `sbujid_dosed`: Participants ID of the participants who had at least one dose of the protocol treatment.
-   `subjid_notdosed`: Reason for participants not dosed.
-   `followup`: Participants ID planned for follow-up.
-   `lost_followup`: Reason for participants not dosed.
-   `assessed`: Participants ID participants attended assessment.
-   `no_value`: Reason for participants missing final assessment.
-   `mitt`: Participants ID included in the mITT analysis.

```{r data-head, echo=FALSE}
head(dispos.data)
```

# Usage

Basic logic:

1.  The vertical node are the number of patients in the current node, no dropout reasons of inclusion reasons should be provided.
2.  Side box is only used for the drop outs. It include information about the number of patients excluded and reasons. 
3.  Any participants with exclusion reasons will not be included in the next vertical node box after excluded. So the subject id can be used multiple times to indicate how many patients left in the current node. 
4.  The node labels, for example visit number or phase, can only horizontally align to a vertical main nodes, not an exclusion box.
5.  If more than 2 treatment allocation is present, all the exclusion box after the allocation will be aligned to the right

**At each stage, number of non-missing values will be counted**. Set `kickoff_sidebox=TRUE` to remove the observations counted in the side box in the remaining nodes. 

# Self-generating function

To generate consort diagram with data.frame, one should prepare a disposition `data.frame`.

```{r example, eval=FALSE}
consort_plot(data,
             orders,
             side_box,
             allocation = NULL,
             labels = NULL,
             cex = 0.8,
             text_width = NULL,
             widths = c(0.1, 0.9))
```

-   `data`: Dataset prepared above
-   `orders`: A named vector or a list, names as the variable and values as labels in the box. The order of the diagram will be based on this. Variables listed here, but not included in other parameters will calculate the number of non-missing values. A list can be provided with multiple variables in each list element and these variables will be stack into a single node. The side box should only have 1 variable. 
-   `side_box`: Variable vector, appeared as side box in the diagram. The next box will be count the number of non-missing values, so users need to make sure this is correct. The pariticpants id variable can be used multiple times, since only the number of non-missing is calculated for the vertical box.
-   `allocation`: Name of the grouping/treatment variable (optional), the diagram will split into branches on this variables.
-   `labels`: Named vector, names is the location of the vertical node excluding the side box. The position location should plus 1 after the allocation variables if the allocation is defined.
-   `kickoff_sidebox`: kickoff of the observation from the data with following a side box.
-   `cex`: Multiplier applied to font size, Default is 0.6
-   `text_width`: A positive integer giving the target column for wrapping lines in the output.

# Manual 
Functions are mainly in three categories, main box, side box and label box. Others include building function. These are the functions used by the self generating function. These box functions require the previous node and text label.
-   `add_box`: add main box, no previous nodes should be provided if this is the first node.
-   `add_side_box`: add exclusion box.
-   `add_split`: add allocation box, all nodes will be split into groups. The label text for this node and following nodes should be a vector with a length larger than 1. 
-   `add_label_box`: add visiting or phasing label given a reference node.
-   `build_grid`: you don't need this unless you want the `grob` object.
-   `build_grviz`: you don't need this unless you want the `Graphviz` code.

# Plotting engine
The package supports two plotting engines: `grid` (`grid` package) and `Graphviz`. The default was `grid`, `plot(g)` (assuming `g` is the consort plot object) will use the `grid` to draw the plot. The coordinates are calculated internally, one can simply use `build_grviz(g)` to get a built `grob` object. This means you can do use `patchwork` package to combine different plots. You can use the example below to add title and footnotes. 

```{r patchwork, eval=FALSE}
library(patchwork)

wrap_elements(build_grid(g)) + plot_annotation(
  title = 'Consort diagram',
  subtitle = 'Flow chart of the XX study',
  caption = 'Disclaimer: None of these plots are insightful'
)
```

Although many efforts have been made to draw the plot as easy as possible, you may still find some plots are difficult to draw. You may not be happy about the final output or want the plot in a `svg` format to be used in the websites. You can simply `plot(g, grViz = TRUE)` to draw the plot with `Graphviz`. You need to install [DiagrammeR](https://CRAN.R-project.org/package=DiagrammeR) package to do so. If you want to do some twicking to the plot, you can use `build_grviz(g)` to extract the `dot` file and edit. You can use `Rstudio` to render the plot or other tools you prefer. Use the example below to dump the `dot` file.
```{r dump-dotfile, eval = FALSE}
cat(build_grviz(g), file = "consort.gv")
```

# Some default settings
The package has some default settings for the line width, color, arrow type and length. These can be easily changed with `set_consort_defaults()`. For example, you can set `set_consort_defaults(arrow_gp = gpar(col = "red", lwd = 2))` to change the arrow graphical parameters. Use `get_consort_defaults()` to view current settings. The default parameters are below:
```{r defualts}
get_consort_defaults()
```

The available options are:

| Option | Type | Default | Description |
|---|---|---|---|
| `arrow_gp` | `gpar()` | `gpar(col = "black", lwd = 1)` | Graphical parameters for arrows (colour, line width, line type, etc.). |
| `txt_gp` | `gpar()` | `gpar(cex = 1, col = "black")` | Graphical parameters for the text inside boxes (font size, colour, font family, etc.). |
| `box_gp` | `gpar()` | `gpar(fill = "white")` | Graphical parameters for the box border and fill colour. |
| `label_txt_gp` | `gpar()` | `gpar(col = "#4F81BD", cex = 1, fontface = "bold")` | Graphical parameters for the phase/stage label text. |
| `label_box_gp` | `gpar()` | `gpar(fill = "#A9C7FD")` | Graphical parameters for the phase/stage label box. |
| `arrow_length` | numeric | `0.1` | Length of the arrowhead in inches. |
| `arrow_type` | character | `"closed"` | Arrow type: `"closed"` (filled) or `"open"`. |
| `pad_u` | numeric | `3` | Padding between nodes (in grid unit multiples). |
| `bullet` | character | `"\u2022"` | Bullet character used in side box item lists (e.g. `"\u2013"` for en-dash, `"-"` for hyphen). |
| `parse_markup` | logical | `FALSE` | Whether to parse lightweight markup (`**bold**`, `*italic*`, etc.) in node labels. See [Text formatting with markup] below. |

The `gpar()` options are merged with the existing defaults, so you only need to specify the properties you want to change. Previous settings can be restored by capturing the return value:

```{r set-restore, eval = FALSE}
set_consort_defaults(arrow_gp = gpar(col = "red", lwd = 2))
# ... draw diagram ...
init_consort_defaults() # restore
```


# Text formatting with markup

Node labels support a lightweight markup syntax for text formatting. This works with both the `grid` and `Graphviz` rendering backends. Markup parsing is **off by default**; enable it with:

```r
set_consort_defaults(parse_markup = TRUE)
```

The following styles are available:

| Markup | Result |
|---|---|
| `**text**` | **Bold** |
| `*text*` | *Italic* |
| `^{text}` | Superscript |
| `_{text}` | Subscript |
| `__text__` | Underline |

Markup can be mixed with plain text and line breaks (`\n`) as usual.


*Note*: the markup may not be perfect. There is known issue with Graphviz that the combination of bold and normal text may not render correctly and it may overlap with the surrounding text. If you encounter this issue, you can try to add some extra space around the styled text.



## Example

```{r markup-grid, fig.width = 7, fig.height = 5, out.width="70%"}
library(grid)
set_consort_defaults(txt_gp = gpar(cex = 0.8), parse_markup = TRUE)

g <- add_box(txt = "**Patients screened** (n=300)") |>
  add_side_box(txt = "Excluded^{1} (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=12)") |>
  add_box(txt = "__Randomised__ (n=285)") |>
  add_split(txt = c("*Arm A* (n=143)", "*Arm B* (n=142)")) |>
  add_box(txt = c("Analysed (n=128)", "Analysed (n=135)")) |>
  add_label_box(txt = c("1" = "Screening", "3" = "Randomisation"))
plot(g)
```

The same diagram can be rendered with `Graphviz`:

```{r markup-grviz, out.width="70%", fig.width  = 3, fig.height = 1.5}
plot(g, grViz = TRUE)
```

Formatting is applied per-segment, so you can combine styles within a single label — for example `"**Enrolled** (n=300)\n*p* < 0.05^{1}"` renders "Enrolled" in bold, "p" in italic, and "1" as a superscript footnote marker.

Plain text without any markup is rendered exactly as before.

# Working example (self generation)
## Single arm

```{r single-arms, fig.width  = 6, fig.height = 6, out.width="70%"}
out <- consort_plot(data = dispos.data,
                    orders = c(trialno = "Population",
                               exclusion = "Excluded",
                               trialno = "Allocated",
                               subjid_notdosed = "Not dosed",
                               followup = "Followup",
                               lost_followup = "Not evaluable for the final analysis",
                               mitt = "Final Analysis"),
                    side_box = c("exclusion", "subjid_notdosed", "lost_followup"),
                    cex = 0.9)
plot(out)
```

## Multiple phase and multiple arms

```{r multiple-phase, fig.width  = 9, fig.height = 6, out.width="90%"}
g <- consort_plot(data = dispos.data,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm3     = "Randomized patient",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = "arm3",
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "5" = "End of study"),
                  cex = 0.7)
plot(g)
```

## Multiple stratification with node stack
In some cases, two level randomisation/stratification is desired. A typical case is the factorial design. Simply provide two allocation variables, but more than two splits are not supported. 

Here is a simple example on how to do it.
```{r multiple-split, fig.width  = 12, fig.height = 8, out.width = "95%"}
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
g <- consort_plot(data = df,
                  orders = c(trialno = "Population",
                             exclusion1    = "Excluded",
                             induction   = "Induction",
                             exclusion2    = "Excluded",
                             arm = "Randomized patient",
                             arm3     = "",
                             subjid_notdosed = "Not dosed",
                             mitt = "Final miTT Analysis"),
                  side_box = c("exclusion1", "exclusion2", "subjid_notdosed"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Month 4",
                             "3" = "Randomization", 
                             "6" = "End of study"),
                  cex = 0.7)
plot(g)
```

In a more general cases, one may want to keep the count inside the vertical box instead of excluding. In this cases, one can provide multiple variables in a list. The first variable will simply be counted, the remaining variables will be itemised. 
```{r multiple-split-stack, fig.width  = 13, fig.height = 10, out.width = "95%"}
# We will exclude one arm to avoid too many arms.
df <- dispos.data[!dispos.data$arm3 %in% "Trt C",]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                # The following two variables will be stacked together
                                c(arm3     = "", # Should not provide a value to show the actual arm
                                  subjid_notdosed="Participants not treated"),
                                # The following two variables will be stacked together
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"), 
                  allocation = c("arm", "arm3"), # Two level randomisation
                  kickoff_sidebox = FALSE,
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)

plot(p)
```

# Working example (human effort)
The previous is to easily generate a consort diagram based on a disposition data, here we show how to create a consort diagram by providing the label text manually. 

## Provide text
```{r providetext1, fig.width  = 7, fig.height = 4, out.width="60%"}
library(grid)
# Might want to change some settings
set_consort_defaults(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)
```

## Missing nodes and multiple split
There might be some cases that the nodes will be missing, this can be handled as well. You can also have multiple splits, but multiple splits for the `grid` hasn't been implemented yet. You can use `plot(g, grViz = TRUE)` to produce the consort plot.

```{r providetext2, fig.width  = 3, fig.height = 1.5, out.width="80%"}
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
# Length needs to be the same as previous one, use a list here.
g <- add_split(g, txt = list(c("Process 1 (n=140)", "Process 2 (n=140)",
                               "Process 3 (n=142)")))

plot(g, grViz = TRUE)

```

## Two-level stratification
Some studies may first stratify patients then randomise, a factorial design for example. This will require two-level stratification. One can simply provide **a list** of the same length of the previous node after a split. More than two splits are not supported. 
```{r manual-multisplit, fig.width  = 13, fig.height = 6, out.width = "80%"}
g <- add_box(txt = "Patient consented (n=200)")
g <- add_side_box(g, txt = "Excluded (n=10):\n\u2022 MRI not collected (n=3)\n\u2022 Other (n=7)")
g <- add_box(g, txt = "Randomised (n=190)")
g <- add_split(g, txt = c("Randomised study (n=144)", "Preference study (n=146)"))

# Provide a list here
g <- add_split(g, txt = list(c("Allocated to surgery (n=70)", 
                                "Allocated to medicine (n=74)"),
                             c("Allocated to surgery (n=47)", 
                              "Allocated to medicine (n=48)",
                               "Placebo (n=51)")))

g <- add_side_box(g, txt = c("Excluded (n=3):\n\u2022 Withdrawn before surgery (n=2)\n\u2022 Declined (n=1)", 
                             "", 
                             "Excluded (n=1):\n\u2022 Withdrawn before surgery (n=1)",
                             "Excluded (n=3):\n\u2022 Withdrawn before surgery (n=1)\n\u2022 Declined (n=1)",
                             "Excluded (n=10):6\n\u2022 Declined (n=10)"),
                  side = rep("right", 5))
g <- add_box(g, txt = c("Analysed (n=67)", "Analysed (n=74)",
                        "Analysed (n=46)", "Analysed (n=45)", "Analysed (n=41)"))

plot(g)

```

## Using disposition table
```{r disposition-data, fig.width  = 7, fig.height = 6.5, out.width="70%"}
set_consort_defaults(txt_gp = gpar(cex = 0.8))

dispos.data$arm <- factor(dispos.data$arm)

txt <- gen_text(dispos.data$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(dispos.data$exclusion, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

g <- add_box(g, txt = gen_text(dispos.data$arm, label = "Patients randomised")) 

txt <- gen_text(dispos.data$arm)
g <- add_split(g, txt = txt)

# Exclude subjects
dispos.data <- dispos.data[is.na(dispos.data$exclusion), ]

txt <- gen_text(split(dispos.data[,c("subjid_notdosed")], dispos.data$arm),
                label = "Not dosed", bullet = TRUE)
g <- add_box(g, txt = txt, just = "left")

txt <- gen_text(split(dispos.data$mitt, dispos.data$arm),
                label = "Primary mITT analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "5" = "Final analysis"))

plot(g)
```

## For Shiny and HTML
Although all the efforts has been made to precisely calculate the coordinates of the nodes, it is not very accurate due to limit of my own knowledge. But you can utilize the [DiagrammeR](https://CRAN.R-project.org/package=DiagrammeR) to produce plots for Shiny and HTML by setting `grViz = TRUE` in `plot`. You can get `Graphviz` code with `build_grviz` of the plot. In addition, use [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) save plot in various formats.
```{r save-plot, fig.width  = 3, fig.height = 1.5, out.width="70%"}
plot(g, grViz = TRUE)
```

## Use with Quarto 
Quarto has native [support](https://quarto.org/docs/authoring/diagrams.html) for embedding Graphviz diagrams. You can plot the flowchart without any printing method. 

````{verbatim, lang = "markdown"}
```{r}
cat(build_grviz(g), file = "consort.gv")
```

```{dot}
//| label: consort-diagram
//| fig-cap: "CONSORT diagram of study XXX"
//| file: consort.gv
```

````

# Saving plot
In order to export the plot to fit a page properly, you need to provide the width and height of the output plot. You might need to try different width and height to get a satisfying plot.You can use R basic device destination for the output. Below is how to save a plot in `png` format:
```{r eval=FALSE}
# save plots
png("consort_diagram.png", width = 29, 
    height = 21, res = 300, units = "cm", type = "cairo") 
plot(g)
dev.off() 

```

Or you can use `ggplot2::ggsave` function to save the plot object:
```{r eval=FALSE}
ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))
```

Or save with [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) to `png` or `pdf`
```{r eval=FALSE}
plot(g, grViz = TRUE) |> 
    DiagrammeRsvg::export_svg() |> 
    charToRaw() |> 
    rsvg::rsvg_pdf("svg_graph.pdf")
```

---
title: "Self-generating CONSORT diagram"
output: 
  rmarkdown::html_vignette:
    toc: yes
vignette: >
  %\VignetteIndexEntry{Self-generating CONSORT diagram}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r setup}
library(consort)
```

The goal of this package is to make it easy to create CONSORT diagrams for the transparent reporting of participant allocation in randomized, controlled clinical trials. This is done by creating a standardized disposition data, and using this data as the source for the creation a standard CONSORT diagram. Human effort by supplying text labels on the node can also be achieved. Below is the illustration of the CONSORT diagram creating process for the two different methods.

# Prepare test data
In a clinical research, we normally will have a participants disposition data. One column is the participants' ID, and the following columns indicating the status of the participants at different stage of the study. One can easily derive the number of participants at different stage by counting the number of participants on-study excluding the participants who are excluded. 

```{r}
set.seed(1001)
N <- 300

trialno <- sample(c(1000:2000), N)
exc1 <- rep(NA, N)
exc1[sample(1:N, 15)] <- sample(c("Sample not collected", "MRI not collected",
                                  "Other"), 15, replace = T, prob = c(0.4, 0.4, 0.2))

induc <- rep(NA, N)
induc[is.na(exc1)] <- trialno[is.na(exc1)]

exc2 <- rep(NA, N)
exc2[sample(1:N, 20)] <- sample(c("Sample not collected", "Dead",
                                  "Other"), 20, replace = T, prob = c(0.4, 0.4, 0.2))
exc2[is.na(induc)] <- NA

exc <- ifelse(is.na(exc2), exc1, exc2)

arm <- rep(NA, N)
arm[is.na(exc)] <- sample(c("Conc", "Seq"), sum(is.na(exc)), replace = T)
arm3 <- sample(c("Trt A", "Trt B", "Trt C"), N, replace = T)
arm3[is.na(arm)] <- NA

fow1 <- rep(NA, N)
fow1[!is.na(arm)] <- sample(c("Withdraw", "Discontinued", "Death", "Other", NA),
                            sum(!is.na(arm)), replace = T, 
                            prob = c(0.05, 0.05, 0.05, 0.05, 0.8))
fow2 <- rep(NA, N)
fow2[!is.na(arm) & is.na(fow1)] <- sample(c("Protocol deviation", "Outcome missing", NA),
                                          sum(!is.na(arm) & is.na(fow1)), replace = T, 
                                          prob = c(0.05, 0.05, 0.9))

df <- data.frame(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2)

df$reas1[!is.na(arm)] <- sample(c("Protocol deviation", "Outcome missing", NA),
                                sum(!is.na(arm)), replace = T, 
                                prob = c(0.08, 0.07, 0.85))

df$reas2[!is.na(df$reas1)] <- sample(c("Withdraw", "Discontinued", "Death", "Other"),
                                     sum(!is.na(df$reas1)), replace = T, 
                                     prob = c(0.05, 0.05, 0.05, 0.05))

rm(trialno, exc1, induc, exc2, exc, arm, arm3, fow1, fow2, N)
```

# Description

This function developed to populate consort diagram automatically. But to do so, a population disposition data should be prepared. The following data is prepared for demonstration.

-   `trailno`: List of the subject ID.
-   `exc1`: reason for exclusion after enrolled, should be missing if not excluded.
-   `induc`: list of subject ID entered the induction phase.
-   `exc2`: reason for exclusion in the induction phase, should be missing if not excluded.
-   `exc`: reason for exclusion after enrolled. This is the combination of `exc1` and `exc2`.
-   `arm`: treatment arm, two groups.
-   `fow1`: reason for exclusion in the follow-up, should be missing if not excluded.
-   `fow2`: reason for exclusion in the final analysis, should be missing if not excluded.
-   `arm3`: treatment arm, three groups.

```{r cars, echo=FALSE}
head(df)
```

# Usage

Basic logic:

1.  The vertical node are the number of patients in the current node, no dropout reasons of inclusion reasons should be provided.
2.  Side box is only used for the drop outs. It include information about the number of patients excluded and reasons. 
3.  Any subjects with exclusion reasons will not be included in the next vertical node box after excluded. So the subject id can be used multiple times to indicate how many patients left in the current node. 
4.  The node labels, for example visit number or phase, can only horizontally align to a vertical main nodes, not an exclusion box.
5.  If more than 2 treatment allocation is present, all the exclusion box after the allocation will be aligned to the right

# Self-generating function

To generate consort diagram with data.frame, one should prepare a disposition data.frame.

```{r eval=FALSE}
consort_plot(data,
             orders,
             side_box,
             allocation = NULL,
             labels = NULL,
             cex = 0.8,
             text_width = NULL,
             widths = c(0.1, 0.9))
```

-   `data`: Dataset prepared above
-   `orders`: A named vector or a list, names as the variable and values as labels in the box. The order of the diagram will be based on this. Variables listed here, but not included in other parameters will calculate the number of non-missing values.
-   `side_box`: Variable vector, appeared as side box in the diagram. The next box will be the subset of the missing values of these variables. The subject id variable can be used multiple times, since only the number of non-missing is calculated for the vertical box.
-   `allocation`: Name of the grouping/treatment variable (optional), the diagram will split into branches on this variables.
-   `labels`: Named vector, names is the location of the vertical node excluding the side box. The position location should plus 1 after the allocation variables if the allocation is defined.
-   `cex`: Multiplier applied to font size, Default is 0.6
-   `text_width`: A positive integer giving the target column for wrapping lines in the output.

# Manual 
Functions are mainly in three categories, main box, side box and label box. Others include building function. These are the functions used by the self generating function. These box functions require the previous node and text label.
-   `add_box`: add main box, no previous nodes should be provided if this is the first node.
-   `add_side_box`: add exclusion box.
-   `add_split`: add allocation box, all nodes will be split into groups. The label text for this node and following nodes should be a vector with a length larger than 1. 
-   `add_label_box`: add visiting or phasing label given a reference node.
-   `build_grid`: you don't need this unless you want the `grob` object.
-   `build_grviz`: you don't need this unless you want the `Graphviz` code.

# Working example (self generation)
## Single arm

```{r message=FALSE, fig.width  = 6, fig.height = 6}
out <- consort_plot(data = df,
             orders = c(trialno = "Population",
                          exc1    = "Excluded",
                          trialno = "Allocated",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable for the final analysis",
                          trialno = "Final Analysis"),
             side_box = c("exc1", "fow1", "fow2"),
             cex = 0.9)
plot(out)

```

## Two arms

```{r fig.width  = 7, fig.height = 6}
out <- consort_plot(data = df,
             orders = c(trialno = "Population",
                          exc    = "Excluded",
                          arm     = "Randomized patient",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable",
                          trialno = "Final Analysis"),
             side_box = c("exc", "fow1", "fow2"),
             allocation = "arm",
             labels = c("1" = "Screening", "2" = "Randomization",
                        "5" = "Final"))

plot(out)
```

## Multiple phase

```{r fig.width  = 9, fig.height = 6}
g <- consort_plot(data = df,
             orders = list(trialno = "Population",
                          exc1    = "Excluded",
                          induc   = "Induction",
                          exc2    = "Excluded",
                          arm3     = "Randomized patient",
                          fow1    = "Lost of Follow-up",
                          trialno = "Finished Followup",
                          fow2    = "Not evaluable",
                          trialno = "Final Analysis"),
             side_box = c("exc1", "exc2", "fow1", "fow2"),
             allocation = "arm3",
             labels = c("1" = "Screening", "2" = "Month 4",
                        "3" = "Randomization", "5" = "Month 24",
                        "6" = "End of study"),
             cex = 0.7)
plot(g)
```


# Working example (human effort)
The previous is to easily generate a consort diagram based on a disposition data, here we show how to create a consort diagram by providing the label text manually. 

## Provide text
```{r fig.width  = 7, fig.height = 4}
library(grid)
# Might want to change some settings
options(txt_gp = gpar(cex = 0.8))

txt0 <- c("Study 1 (n=160)", "Study 2 (n=140)")
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)"

# supports pipeline operator
g <- add_box(txt = txt0) |>
  add_box(txt = txt1) |>
  add_side_box(txt = txt1_side) |> 
  add_box(txt = "Randomized (n=200)") |> 
  add_split(txt = c("Arm A (n=100)", "Arm B (n=100)")) |> 
  add_side_box(txt = c("Excluded (n=15):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)\n\u2022 Other (n=8)",
                       "Excluded (n=7):\n\u2022 MRI not collected (n=3)\n\u2022 Tissues not collected (n=4)")) |> 
  add_box(txt = c("Final analysis (n=85)", "Final analysis (n=93)")) |> 
  add_label_box(txt = c("1" = "Screening",
                        "3" = "Randomized",
                        "4" = "Final analysis"))
plot(g)
```

## Missing nodes and multiple split
There might be some cases that the nodes will be missing, this can be handled as well. You can also have multiple splits, but multiple splits for the `grid` hasn't been implemented yet. You can use `plot(g, grViz = TRUE)` to produce the consort plot.

```{r}
g <- add_box(txt = c("Study 1 (n=8)", "Study 2 (n=12)", "Study 3 (n=12)"))
g <- add_box(g, txt = "Included All (n=20)")
g <- add_side_box(g, txt = "Excluded (n=7):\n\u2022 MRI not collected (n=3)")
g <- add_box(g, txt = "Randomised")
g <- add_split(g, txt = c("Arm A (n=143)", "Arm B (n=142)"))
g <- add_box(g, txt = c("", "From Arm B"))
g <- add_box(g, txt = "Combine all")
g <- add_split(g, txt = c("Process 1 (n=140)", "Process 2 (n=140)", "Process 3 (n=142)"))

plot(g, grViz = TRUE)

```

## Using disposition table
```{r fig.width  = 7, fig.height = 6}

df$arm <- factor(df$arm)

txt <- gen_text(df$trialno, label = "Patient consented")
g <- add_box(txt = txt)

txt <- gen_text(df$exc, label = "Excluded", bullet = TRUE)
g <- add_side_box(g, txt = txt)   

# Exclude subjects
df <- df[is.na(df$exc), ]

g <- add_box(g, txt = gen_text(df$arm, label = "Patients randomised")) 

txt <- gen_text(df$arm)
g <- add_split(g, txt = txt)

txt <- gen_text(split(df[,c("reas1", "reas2")], df$arm),
                label = "Lost to follow-up")
g <- add_box(g, txt = txt, just = "left")

df <- df[complete.cases(df[,c("reas1", "reas2")]), ]
txt <- gen_text(split(df$trialno, df$arm),
                label = "Primary analysis")
g <- add_box(g, txt = txt)

g <- add_label_box(g, txt = c("1" = "Baseline",
                              "3" = "First Stage"))

plot(g)
```

## For Shiny and HTML
Although all the efforts has been made to precisely calculate the coordinates of the nodes, it is not very accurate due to limit of my own knowledge. But you can utilize the [DiagrammeR](https://CRAN.R-project.org/package=DiagrammeR) to produce plots for Shiny and HTML by setting `grViz = TRUE` in `plot`. You can get `Graphviz` code with `build_grviz` of the plot. In addition, use [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) save plot in various formats.
```{r}
plot(g, grViz = TRUE)
```

## Use with Quarto 
Quarto has native [support](https://quarto.org/docs/authoring/diagrams.html) for embedding Graphviz diagrams. You can plot the flowchart without any printing method. 

````{verbatim, lang = "markdown"}
```{r}
cat(build_grviz(g), file = "consort.gv")
```

```{dot}
//| label: consort-diagram
//| fig-cap: "CONSORT diagram of study XXX"
//| file: consort.gv
```

````

# Saving plot
In order to export the plot to fit a page properly, you need to provide the width and height of the output plot. You might need to try different width and height to get a satisfying plot.You can use R basic device destination for the output. Below is how to save a plot in `png` format:
```{r eval=FALSE}
# save plots
png("consort_diagram.png", width = 29, 
    height = 21, res = 300, units = "cm", type = "cairo") 
plot(g)
dev.off() 

```

Or you can use `ggplot2::ggsave` function to save the plot object:
```{r eval=FALSE}
ggplot2::ggsave("consort_diagram.pdf", plot = build_grid(g))
```

Or save with [DiagrammeRsvg](https://CRAN.R-project.org/package=DiagrammeRsvg) and [rsvg](https://CRAN.R-project.org/package=rsvg) to `png` or `pdf`
```{r eval=FALSE}
plot(g, grViz = TRUE) |> 
    DiagrammeRsvg::export_svg() |> 
    charToRaw() |> 
    rsvg::rsvg_pdf("svg_graph.pdf")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gen_text.R
\name{gen_text}
\alias{gen_text}
\title{Generate label and bullet points}
\usage{
gen_text(x, label = NULL, bullet = FALSE, bullet_char = consort_opt("bullet"))
}
\arguments{
\item{x}{A list or a vector to be used. \code{x} can be atomic vector, a 
\code{data.frame} or a \code{list}. A \code{data.frame} is particular useful
 if the there's a nested reason or a \code{list} split nested reasons by group.
The nested reasons only support two columns and the \code{bullet} will be ignored.}

\item{label}{A character string as a label at the beginning of the text label.
The count for each categories will be returned if no label is provided.}

\item{bullet}{If shows bullet points. If the value is \code{TRUE}, the bullet points
will be tabulated, default is \code{FALSE}.}

\item{bullet_char}{A single character used as the bullet symbol. Defaults to
\code{consort_opt("bullet")}. Can be any Unicode
character such as \code{"\u2013"} (en-dash), \code{"\u25CB"} (circle),
\code{"\u25A0"} (square), \code{"-"}, etc.}
}
\value{
A character string of vector.
}
\description{
This function use the data to generate label and bullet points for the box.
}
\examples{
val <- data.frame(
  am = factor(ifelse(mtcars$am == 1, "Automatic", "Manual"), ordered = TRUE),
  vs = factor(ifelse(mtcars$vs == 1, "Straight", "V-shaped"), ordered = TRUE),
  car = row.names(mtcars)
)

gen_text(val$car, label = "Cars in the data")
gen_text(val$car, label = "Cars in the data", bullet = FALSE)
gen_text(split(val$car, val$am), label = "Cars in the data")
gen_text(split(val$car, val$am), label = "Cars in the data", bullet = FALSE)
gen_text(split(val[,c("vs", "car")], val$am), label = "Cars in the data", bullet = FALSE)
gen_text(val[,c("vs", "car")], label = "Cars in the data", bullet = FALSE)

# Use a custom bullet character
gen_text(val$car, label = "Cars in the data", bullet = TRUE, bullet_char = "-")

# Or set globally via set_consort_defaults
set_consort_defaults(bullet = "\u25CB")
gen_text(val$car, label = "Cars in the data", bullet = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/build_grviz.R
\name{build_grviz}
\alias{build_grviz}
\title{Build consort diagram}
\usage{
build_grviz(x)
}
\arguments{
\item{x}{A conosrt object.}
}
\value{
A \code{Graphviz} code
}
\description{
Build a \code{grob} consort diagram, use this if you want
to save plots with \code{\link[ggplot2]{ggsave}}
}
\examples{
\dontrun{
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15): \n
              \u2022 MRI not collected (n=3)\n
              \u2022 Tissues not collected (n=4)\n
              \u2022 Other (n=8)"

g <- add_box(txt = txt1)

g <- add_side_box(g, txt = txt1_side)

g <- add_box(g, txt = "Randomized (n=200)")
# plot(g, grViz = TRUE)
}
}
\seealso{
\code{\link[DiagrammeR]{grViz}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connect_box.R
\name{connect_box}
\alias{connect_box}
\title{Connect grob box with arrow.}
\usage{
connect_box(
  start,
  end,
  connect,
  type = c("s", "p"),
  name = NULL,
  arrow_gp = consort_opt("arrow_gp"),
  arrow_length = consort_opt("arrow_length"),
  arrow_type = consort_opt("arrow_type")
)
}
\arguments{
\item{start}{Starting point of the arrow.}

\item{end}{Ending point of the arrow.}

\item{connect}{The connection of the box. It should be the combination of the
position. The \code{t} refers to "top", \code{l} for "left", \code{b} for
"bottom" and \code{r} for "right". The first letter is the starting point of
the start box, the second is the ending point of the end box. For example, if
one wants to connect the left side of the start box with right side of the
left side of the end box, the value should be \code{"lr"}. All the connection
will be started in the middle point.}

\item{type}{Should be one the \code{"s"} (strait line), or \code{"p"} (polyline).}

\item{name}{A character identifier of the line grob, passed to \code{\link[grid]{linesGrob}}.}

\item{arrow_gp}{Graphical parameters \code{\link[grid]{gpar}} object for the 
connector line. Defaults to \code{grid::gpar(col = "black", lwd = 1)}.}

\item{arrow_length}{Length of the arrowhead as a \code{\link[grid]{unit}} object. 
Defaults to \code{unit(0.1, "inches")}.}

\item{arrow_type}{Arrow type, either \code{"closed"} or \code{"open"}. Defaults to 
\code{"closed"}.}
}
\value{
A lines grob with arrow.
}
\description{
This function is used to create an arrow line to connect two boxes. User should
provide the starting and ending side of the arrow.
}
\examples{
fg1 <- textbox(text = "This is a test")
fg2 <- textbox(text = "This is an other test", 0.7, 0.2)
grid::grid.draw(fg1)
grid::grid.draw(fg2)
connect_box(fg1, fg2, connect = "bl", type = "p")

# Customize line width and color
connect_box(fg1, fg2, connect = "bl", type = "p", arrow_gp = grid::gpar(lwd = 2, col = "red"))

# Or set globally via set_consort_defaults
set_consort_defaults(arrow_gp = grid::gpar(lwd = 2, col = "blue"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consort.R
\docType{package}
\name{consort}
\alias{consort-package}
\alias{consort}
\title{Create Consort diagram}
\description{
To make it easy to create CONSORT diagrams for the transparent reporting
 of participant allocation in randomized, controlled clinical trials. This
  is done by creating a standardized disposition data, and using this data
   as the source for the creation a standard CONSORT diagram. Human effort
    by supplying text labels on the node can also be achieved.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/adayim/consort/}
  \item Report bugs at \url{https://github.com/adayim/consort/issues}
}

}
\author{
\strong{Maintainer}: Alim Dayim \email{ad938@cam.ac.uk} (\href{https://orcid.org/0000-0001-9998-7463}{ORCID})

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_side_box.R
\name{add_side_box}
\alias{add_side_box}
\title{Add a side node}
\usage{
add_side_box(prev_box, txt, side = NULL, text_width = NULL, ...)
}
\arguments{
\item{prev_box}{Previous node object, the created new node will be aligned
at the right bottom of the `prev_box`.}

\item{txt}{Text in the node. If the `prev_box` is a horizontally aligned multiple
nodes, a vector of with the same length must be provided.}

\item{side}{Position of the side box, `left` or `right` side of the terminal box.
Will be aligned on the left and right side if only two groups, right otherwise.
This will be ignored for for `grViz` plot, see \link[DiagrammeR]{grViz}.}

\item{text_width}{a positive integer giving the target column for wrapping
lines in the output. String will not be wrapped if not defined (default).
The \code{\link[stringi]{stri_wrap}} function will be used if \code{stringi}
package installed as it is a better options for non-Latin language, otherwise
 \code{\link[base]{strwrap}} will be used.}

\item{...}{Other parameters pass to \link{textbox},}
}
\value{
A \code{consort} object.
}
\description{
Add an exclusion node on the right side. If the length of text label is two, then
the first one will be aligned on the left and the second on the right. Otherwise,
all the side nodes will be aligned on the right.
}
\examples{
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15): \n
              \u2022 MRI not collected (n=3)\n
              \u2022 Tissues not collected (n=4)\n
              \u2022 Other (n=8)"

g <- add_box(txt = txt1)

g <- add_side_box(g, txt = txt1_side)

g <- add_box(g, txt = "Randomized (n=200)")

g <- add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100"))
g <- add_side_box(g,
  txt = c(
    "Excluded (n=15):\n
                  \u2022 MRI not collected (n=3)\n
                  \u2022 Tissues not collected (n=4)\n
                   \u2022 Other (n=8)",
    "Excluded (n=15):\n
                   \u2022 MRI not collected (n=3)\n
                   \u2022 Tissues not collected (n=4)"
  )
)

g <- add_box(g, txt = c("Final analysis (n=100)", "Final analysis (n=100"))
g <- add_label_box(g, txt = c("1" = "Screening", "3" = "Randomized", "4" = "Final analysis"))
}
\seealso{
\code{\link{add_box}} \code{\link{add_split}} \code{\link{textbox}} 
\code{\link{add_label_box}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_box.R
\name{add_box}
\alias{add_box}
\title{Add nodes}
\usage{
add_box(
  prev_box = NULL,
  txt,
  just = c("center", "left", "right"),
  text_width = NULL,
  ...
)
}
\arguments{
\item{prev_box}{Previous node object, the created new node will be vertically
aligned with this node. Left this as `NULL` if this is the first node. The first
node will be aligned in the top center.}

\item{txt}{Text in the node. If the `prev_box` is a horizontally aligned multiple
nodes, a vector of with the same length must be provided.}

\item{just}{The justification for the text: left, center or right.}

\item{text_width}{a positive integer giving the target column for wrapping
lines in the output. String will not be wrapped if not defined (default).
The \code{\link[stringi]{stri_wrap}} function will be used if \code{stringi}
package installed as it is a better options for non-Latin language, otherwise
 \code{\link[base]{strwrap}} will be used.}

\item{...}{Other parameters pass to \link{textbox},}
}
\value{
A \code{consort} object.
}
\description{
Create/add vertically aligned labeled nodes or side nodes.
}
\examples{
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15): \n
              \u2022 MRI not collected (n=3)\n
              \u2022 Tissues not collected (n=4)\n
              \u2022 Other (n=8)"

g <- add_box(txt = txt1)

g <- add_side_box(g, txt = txt1_side)

g <- add_box(g, txt = "Randomized (n=200)")

g <- add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100"))
g <- add_side_box(g,
  txt = c(
    "Excluded (n=15):\n
                  \u2022 MRI not collected (n=3)\n
                  \u2022 Tissues not collected (n=4)\n
                   \u2022 Other (n=8)",
    "Excluded (n=15):\n
                   \u2022 MRI not collected (n=3)\n
                   \u2022 Tissues not collected (n=4)"
  )
)

g <- add_box(g, txt = c("Final analysis (n=100)", "Final analysis (n=100"))
g <- add_label_box(g, txt = c("1" = "Screening", "3" = "Randomized", "4" = "Final analysis"))
}
\seealso{
\code{\link{add_side_box}} \code{\link{add_split}} \code{\link{textbox}} 
\code{\link{add_label_box}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_coords.R
\name{get_coords}
\alias{get_coords}
\title{Get the coordinates of the textbox object}
\usage{
get_coords(x)
}
\arguments{
\item{x}{A textbox object}
}
\value{
A list of coordinates will return:
\item{left}{Left (x-min) side coordinate.}
\item{right}{Right (x-max) side coordinate.}
\item{bottom}{Bottom (y-min) side coordinate.}
\item{top}{Top (y-max) side coordinate.}
\item{top.mid }{Coordinate vector of top middle, measured by grob.}
\item{left.mid}{Coordinate vector of left middle, measured by grob.}
\item{bottom.mid}{Coordinate vector of bottom middle, measured by grob.}
\item{right.mid}{Coordinate vector of right middle, measured by grob.}
\item{x}{X (center x) coordinate.}
\item{y}{Y (center y) coordinate.}
\item{width}{Width of the textbox, derived with \code{grobWidth}.}
\item{height}{Height of the textbox, derived with \code{grobHeight}.}
\item{half_width}{Half width of the box.}
\item{half_height}{Half height of the box.}
}
\description{
This function will get the coordinates of the textbox object.
}
\examples{
fg <- textbox(text = "This is a test")
get_coords(fg)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consort_plot.R
\name{consort_plot}
\alias{consort_plot}
\title{Self generating consort diagram}
\usage{
consort_plot(
  data,
  orders,
  side_box,
  allocation = NULL,
  labels = NULL,
  kickoff_sidebox = TRUE,
  cex = 0.8,
  text_width = NULL
)
}
\arguments{
\item{data}{Data set with disposition information for each participants.}

\item{orders}{A named vector or a list, names as the variable in the dataset
and values as labels in the box. The order of the diagram will be based on this.
A list can be used to report multiple variable in a single node, the first 
variable in a list element will be used to report the total and the exact items
will be summarised for the remaining variable. This is limitted to non-side box.}

\item{side_box}{Variable vector, appeared as side box in the diagram. The next
box will be the subset of the missing values of these variables.}

\item{allocation}{Name of the grouping/treatment variable (optional), the
 diagram will split into branches on this variables forward. For a factorial 
design, with two splits for example, a character vector with a maximum of 
length two can be provided. The extra box will be skipped if the values
 in the \code{orders} blank.}

\item{labels}{Named vector, names is the location of the terminal node. The
position location should plus 1 after the allocation variables if the allocation
is defined.}

\item{kickoff_sidebox}{remove (default) the side box observations from the 
following counting.}

\item{cex}{Multiplier applied to font size, default is 0.8. Prefer using
\code{\link{set_consort_defaults}(txt_gp = gpar(cex = ...))} instead.}

\item{text_width}{a positive integer giving the target column for wrapping
lines in the output. String will not be wrapped if not defined (default).
The \code{\link[stringi]{stri_wrap}} function will be used if \code{stringi}
package installed, otherwise \code{\link[base]{strwrap}} will be used.}
}
\value{
A \code{consort} object.
}
\description{
Create CONSORT diagram from a participant disposition data.
}
\details{
The calculation of numbers is as in an analogous to Kirchhoff's Laws of
electricity. The numbers in terminal nodes must sum to those in the ancestor
 nodes. All the drop outs will be populated as a side box. Which was different
  from the official CONSORT diagram template, which has dropout inside a
  vertical node.
}
\examples{

## Prepare test data
data(dispos.data)

df <- dispos.data[!dispos.data$arm3 \%in\% "Trt C", ]
p <- consort_plot(data = df,
                  orders = list(c(trialno = "Population"),
                                c(exclusion = "Excluded"),
                                c(arm     = "Randomized patient"),
                                c(arm3     = "", 
                                  subjid_notdosed="Participants not treated"),
                                c(followup    = "Pariticpants planned for follow-up",
                                  lost_followup = "Reason for tot followed"),
                                c(assessed = "Assessed for final outcome"),
                                c(no_value = "Reason for not assessed"),
                                c(mitt = "Included in the mITT analysis")),
                  side_box = c("exclusion", "no_value"),
                  allocation = c("arm", "arm3"),
                  labels = c("1" = "Screening", "2" = "Randomization",
                             "5" = "Follow-up", "7" = "Final analysis"),
                  cex = 0.7)
}
\seealso{
\code{\link{add_side_box}},\code{\link{add_split}},
\code{\link{add_side_box}} \code{\link{textbox}} 
\code{\link{set_consort_defaults}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/move_box.R
\name{move_box}
\alias{move_box}
\title{Move a box grob}
\usage{
move_box(obj, x = NULL, y = NULL, pos_type = c("absolute", "relative"))
}
\arguments{
\item{obj}{A \code{box} object.}

\item{x}{A unit element or a number that can be converted to \code{npc}, see
\link[grid]{unit}.}

\item{y}{A unit element or a number that can be converted to \code{npc}, see
\link[grid]{unit}.}

\item{pos_type}{If the provided coordinates are \code{absolute} position the
box will be moved to or it's a \code{relative} position to it's current.}
}
\value{
A box object with updated x and y coordinates.
}
\description{
This function can be used to move the box to a given position with
\link[grid]{editGrob} changing the \code{x} and \code{y} value.
}
\examples{
fg <- textbox(text = "This is a test")
fg2 <- move_box(fg, 0.3, 0.3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{dispos.data}
\alias{dispos.data}
\title{Demo clinical trial disposition data}
\format{
A data frame with 300 rows and 11 columns:
\describe{
  \item{trialno}{Participants ID of the participants}
  \item{exclusion1, exclusion2}{Exclusion reason before and after induction}
  \item{induction}{Participants ID of the participants who are included in the induction
  phase, an extra treatment before randomisation.}
  \item{exclusion}{Exclusion reason before randomisation, including before and after
  induction}
  \item{arm, arm3}{Arms pariticipants randomised to.}
  \item{sbujid_dosed}{Participants ID of the participants who had at least one dose of 
  the protocol treatment.}
  \item{subjid_notdosed}{Reason for participants not dosed.}
  \item{followup}{Participants ID planned for follow-up.}
  \item{lost_followup}{Reason for participants not dosed.}
  \item{assessed}{Participants ID participants attended assessment.}
  \item{no_value}{Reason for participants missing final assessment.}
  \item{mitt}{Participants ID included in the mITT analysis.}
}
}
\usage{
dispos.data
}
\description{
This is a demo data to demonstrate the use of the package. One row per
participant. Participants who are excluded should provide a reason, missing
otherwise.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/textbox.R
\name{textbox}
\alias{textbox}
\alias{grid.textbox}
\title{Create a box with text}
\usage{
textbox(
  text,
  x = unit(0.5, "npc"),
  y = unit(0.5, "npc"),
  just = c("center", "left", "right"),
  txt_gp = consort_opt("txt_gp"),
  leftrotate = FALSE,
  box_fn = roundrectGrob,
  box_gp = consort_opt("box_gp"),
  name = "textbox"
)

grid.textbox(...)
}
\arguments{
\item{text}{A character text to be passed to \link[grid]{textGrob}.}

\item{x}{A number or unit object specifying x-location.}

\item{y}{A number or unit object specifying y-location.}

\item{just}{The justification of the text, `"left"`, `"right` and `"center"`.
See \link[grid]{textGrob} for more details.}

\item{txt_gp}{An object of class \link[grid]{gpar} style to be applied to the
text. Defaults are read from \code{\link{set_consort_defaults}}. For
example, to set a font size for all text inside boxes, use
\code{set_consort_defaults(txt_gp = gpar(cex = 0.8))}.}

\item{leftrotate}{If the text box will be rotated 90 degrees counter-clockwise.
Default is \code{FALSE}.}

\item{box_fn}{Function to create box for the text. Parameters of `x=0.5`,
`y=0.5` and `box_gp` will be passed to this function and return a \code{grob}
object.}

\item{box_gp}{An object of class \link[grid]{gpar} style to be applied to the
box.}

\item{name}{A character identifier.}

\item{...}{Parameters passed to \code{textbox}}
}
\value{
A text box grob. grid.textbox() returns the value invisibly.
}
\description{
Create a \link[grid]{grob} with text inside. To extract the units describing
grob boundary location can be accessed with \link[grid]{grobX} and
 \link[grid]{grobY}. The units describing width and height can be accessed with
 \link[grid]{grobWidth} and \link[grid]{grobHeight}.
}
\examples{
fg <- textbox(text = "This is a test")
grid::grid.draw(fg)
grid.textbox(text = "This is a test")
grid.textbox(text = "This is a test")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defaults.R
\name{set_consort_defaults}
\alias{set_consort_defaults}
\alias{get_consort_defaults}
\alias{init_consort_defaults}
\alias{print.consort_defaults}
\title{Set consort diagram default options}
\usage{
set_consort_defaults(
  arrow_gp = NULL,
  txt_gp = NULL,
  box_gp = NULL,
  label_txt_gp = NULL,
  label_box_gp = NULL,
  arrow_length = NULL,
  arrow_type = NULL,
  pad_u = NULL,
  bullet = NULL,
  parse_markup = NULL
)

get_consort_defaults()

init_consort_defaults()

\method{print}{consort_defaults}(x, ...)
}
\arguments{
\item{arrow_gp}{A \code{\link[grid]{gpar}} object for the arrow line.}

\item{txt_gp}{A \code{\link[grid]{gpar}} object for the text inside boxes.}

\item{box_gp}{A \code{\link[grid]{gpar}} object for the box border and fill.}

\item{label_txt_gp}{A \code{\link[grid]{gpar}} object for the label text.}

\item{label_box_gp}{A \code{\link[grid]{gpar}} object for the label box.}

\item{arrow_length}{Numeric, length of the arrowhead in inches.}

\item{arrow_type}{Character, arrow type: \code{"closed"} or \code{"open"}.}

\item{pad_u}{Numeric, padding between nodes.}

\item{bullet}{Character, bullet character for side box items.}

\item{parse_markup}{Logical, whether to parse lightweight markup syntax
(\code{**bold**}, \code{*italic*}, \code{^{superscript}},
\code{_{subscript}}, \code{__underline__}) in node labels.
Default is \code{FALSE}.}

\item{x}{A \code{consort_defaults} object.}

\item{...}{Not used.}
}
\value{
Invisibly returns the previous defaults (a \code{consort_defaults} object).

A \code{consort_defaults} object containing all current default settings.
}
\description{
Modify the default graphical parameters and other settings for consort diagrams.
Any parameter set to \code{NULL} (the default) will remain unchanged.
}
\examples{
# Change text color and box fill
old <- set_consort_defaults(
  txt_gp = grid::gpar(col = "navy", cex = 0.9),
  box_gp = grid::gpar(fill = "#F0F0F0")
)

# View current defaults
get_consort_defaults()

# Restore previous defaults
set_consort_defaults(
  txt_gp = old$txt_gp,
  box_gp = old$box_gp
)
get_consort_defaults()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_label_box.R
\name{add_label_box}
\alias{add_label_box}
\title{Add a vertically aligned label nodes on the left side.}
\usage{
add_label_box(
  prev_box,
  txt,
  only_terminal = TRUE,
  just = c("center", "left", "right"),
  box_fn = roundrectGrob,
  ...
)
}
\arguments{
\item{prev_box}{A completed diagram created with \code{add_box}, \code{add_side_box} etc.}

\item{txt}{Text in the node. If a character string is provided, the label will be aligned
 to the last box if a character is provided. If a named vector, the labels will align to
corresponding row of the node. And the names is the number indicating row number of box to
 horizontally align with and value is the text in the box.}

\item{only_terminal}{If the txt is only for the terminal box, default. Otherwise, the side box will
also be accounted for.}

\item{just}{The justification for the text: center (default), left or right.}

\item{box_fn}{The function to create the box grob. Default is \code{\link[grid]{roundrectGrob}}.}

\item{...}{Other parameters pass to \link{textbox}.}
}
\value{
A \code{consort} object.
}
\description{
In a consort diagram, this can be used to indicate different stage.
}
\examples{
txt1 <- "Population (n=300)"
txt1_side <- "Excluded (n=15): \n
              \u2022 MRI not collected (n=3)\n
              \u2022 Tissues not collected (n=4)\n
              \u2022 Other (n=8)"

g <- add_box(txt = txt1)

g <- add_side_box(g, txt = txt1_side)

g <- add_box(g, txt = "Randomized (n=200)")

g <- add_split(g, txt = c("Arm A (n=100)", "Arm B (n=100"))
g <- add_side_box(g,
  txt = c(
    "Excluded (n=15):\n
                  \u2022 MRI not collected (n=3)\n
                  \u2022 Tissues not collected (n=4)\n
                   \u2022 Other (n=8)",
    "Excluded (n=15):\n
                   \u2022 MRI not collected (n=3)\n
                   \u2022 Tissues not collected (n=4)"
  )
)

g <- add_box(g, txt = c("Final analysis (n=100)", "Final analysis (n=100"))
g <- add_label_box(g, txt = c("1" = "Screening", "3" = "Randomized", "4" = "Final analysis"))
}
\seealso{
\code{\link{add_side_box}} \code{\link{add_split}} \code{\link{textbox}} 
\code{\link{add_box}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{plot.consort}
\alias{plot.consort}
\alias{print.consort}
\title{Add methods to print function}
\usage{
\method{plot}{consort}(x, grViz = FALSE, ...)

\method{print}{consort}(x, grViz = FALSE, ...)
}
\arguments{
\item{x}{A \code{consort} object.}

\item{grViz}{If use \link[DiagrammeR]{grViz} to print the plot. 
Default is \code{FALSE} to use \link[grid]{grid.draw}}

\item{...}{Not used.}
}
\value{
None.
}
\description{
Method for plot objects and display the output in on a grid device.
}
\seealso{
\code{\link{add_side_box}},\code{\link{add_split}},
\code{\link{add_side_box}}, \link[grid]{grid.draw}
}
